// 
//  Copyright (C) 2000-2001,2003-2005,2007,2011,2016  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef DS_ANCILLARY_OBJ_H
#define DS_ANCILLARY_OBJ_H

#ifndef LIBERR_H
#include <liberr.h>
#endif

#include <vector>
#include <deque>
using std::vector;
using std::deque;

/* class to hold a bit array.  could be used for either DM BIT or BYTE types */
class dsBitArray
{
public:

  /* constructors/destructor */
  dsBitArray(void);

  dsBitArray(unsigned char* new_data_p,
	     long new_num_bits,
	     long new_size_array);

  dsBitArray(long new_num_bits);

  ~dsBitArray(void);

  /* copy constructor */
  dsBitArray(const dsBitArray &copy);

  /* assignment operator */
  dsBitArray &operator = (const dsBitArray &right);

  /* modifiers */
  void SetNumBits(long new_num_bits);

  void SetValue(unsigned char* new_data_p,
		long new_num_bits,
		long new_size_array);

  dsErrBool SetBit(long bit_num,
		   short bit_val);

  /* accessors */
  inline long GetNumBits(void) const {return num_bits;};

  inline long GetNumChars(void) const {return size_array;};

  void GetBitData(unsigned char *&data_holder_p);

  dsErrBool IsBitSet(long bit_num);

  void Print(void);

  /* boolean operators */
  bool operator == (const dsBitArray &other);
  bool operator != (const dsBitArray &other);
  bool operator ! (void);

protected:

  void HelpClean(void);

  unsigned char *data_p;
  long num_bits;
  long size_array;
};


class dsDimensions 
{
public:

  inline dsDimensions(void);

  inline dsDimensions(long new_num_dims,
	       vector<long> &new_axes);

  inline dsDimensions(long new_num_dims,
	       long *new_axes_p);

  inline ~dsDimensions(void) { HelpClean(); }

  inline dsDimensions(const dsDimensions &copy) { HelpCopy(copy);}

  inline dsDimensions &operator = (const dsDimensions &right);

  inline dsErrBool AreCompatible(const dsDimensions &other) const;

  inline dsErrBool CompatibleAsIndex(dsDimensions &index);

  inline dsErrBool ConvertToIndex(dsDimensions &index,
			   unsigned long &out_index);

  inline long GetNumDimensions(void) const {return num_dims;};

  inline unsigned long GetArraySize(void) const {return array_size;};

  inline long GetAxisSize(long index) { if(index < num_dims && index >= 0)return axes[index]; else return 0;};

  vector<long> &GetAxes(void) {return axes;};

  inline bool operator == (const dsDimensions &other) const; 

  bool operator != (dsDimensions &other) {return !(*this == other);};

protected:
  inline void HelpCopy(const dsDimensions &other);
  inline void HelpClean(void);

  long num_dims;
  unsigned long array_size;
  vector<long> axes;
};

void dsDimensions::HelpClean(void)
{
  num_dims = 0;
  array_size = 0;
  axes.clear();
}

void dsDimensions::HelpCopy(const dsDimensions &other)
{
  long ctr = 0;
  long limit = other.num_dims;

  num_dims = other.num_dims;
  array_size = other.array_size;

  for(ctr=0; ctr < limit; ctr++)
    {
      axes.push_back(other.axes[ctr]);
    }
}

bool dsDimensions::operator == (const dsDimensions &other) const 

{
  bool return_val = 1;
  long ctr = 0;
  
  if((num_dims == other.num_dims))
    { 
      for(ctr=0; ctr < num_dims && return_val == 1; ctr++)
	{
	  return_val = (axes[ctr] == other.axes[ctr]);
	}
    } else {
    return_val  = 0;
  }

  return return_val;
}

dsErrBool dsDimensions::ConvertToIndex(dsDimensions &index,
				       unsigned long &out_index)
{
  dsErrBool return_val_e = dsErrTrue;

  out_index = 0;

  if((return_val_e = CompatibleAsIndex(index)) == dsErrTrue)
    {
      long ctr = num_dims - 1;
      long prev_size = 1;
      deque<long > cumulative_sizes;
      
      cumulative_sizes.push_front(1);
      for(; ctr > 0; ctr--)
	{
	  prev_size *= axes[ctr];
	  cumulative_sizes.push_front(prev_size);
	}

      for(; ctr < num_dims; ctr++)
	{
	  out_index += (cumulative_sizes[ctr] * index.axes[ctr]);
	}
    }

  return return_val_e;
}

dsErrBool dsDimensions::CompatibleAsIndex(dsDimensions &index)
{
  dsErrBool return_val_e = dsErrTrue;

  if(num_dims <= 1 && index.array_size >= array_size)
    {
      return_val_e = dsErrFalse;
    }
  else
    {
      long ctr = 0;

      for(ctr=0; ctr < num_dims && return_val_e == dsErrTrue; ctr++)
	{
	  if(axes[ctr] <= index.axes[ctr])
	    {
	      return_val_e = dsErrFalse;
	    }
	}
    }

  return return_val_e;
}
dsErrBool dsDimensions::AreCompatible(const dsDimensions &other) const
{
  dsErrBool return_val_e = dsErrFalse;

  if(( other.num_dims == 0) || (num_dims == 0 ) ||
     (operator==(other)) || 
     (( (other.num_dims == 1 ) || ( num_dims == 1)) 
      && (other.array_size == array_size)) )
    {
      return_val_e = dsErrTrue;
    }
  
  return return_val_e;
}
dsDimensions &dsDimensions::operator = (const dsDimensions &right)
{
  if(&right != this)
    {
      HelpClean();
      HelpCopy(right);
    }
  return *this;
}



dsDimensions::dsDimensions(void)
{

  num_dims = 0;
  array_size = 1;
  axes.push_back(0);
}

dsDimensions::dsDimensions(long new_num_dims,
			   vector<long> &new_axes)
{
  long ctr = 0;
  
  array_size = 1;
  num_dims = new_num_dims;
  for(ctr=0; ctr < num_dims; ctr++)
    {
      axes.push_back(new_axes[ctr]);
      array_size *= axes[ctr];
    }
}

dsDimensions::dsDimensions(long new_num_dims,
			   long *new_axes_p)
{
  long ctr = 0;
  
  array_size = 1;
  num_dims = new_num_dims;
  for(ctr=0; ctr < num_dims; ctr++)
    {
      axes.push_back(new_axes_p[ctr]);
      array_size *= axes[ctr];
    }
}

#endif
