/*                                                                
**  Copyright (C) 1997-2003,2007,2010,2013,2015,2019-2020  Smithsonian Astrophysical Observatory 
*/                                                                
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


#ifndef EVENT_DEFS_H
#define EVENT_DEFS_H

#include <stdio.h>
#include <stddef.h>
#include <string.h>
#include <stdlib.h>

#ifndef BOOL_H
#include <bool.h>
#define BOOL_H
#endif 

#ifndef PIXLIB_H
#include "pixlib.h"
#define PIXLIB_H 
#endif 

#ifndef LIBERR_H
#include "liberr.h"
#define LIBERR_H 
#endif 

#ifndef STKLIB 
#include "stack.h" 
#define STKLIB 
#endif 

#ifndef DSNAN_H
#include "dsnan.h"
#define DSNAN_H
#endif 

/* data model */
#ifndef ASCDM_H
#include "ascdm.h"              /* data model */
#define ASCDM_H
#endif

#ifdef USE_BADPIXEL 
#ifndef BAD_PIXEL_DEFS_H 
#include "bad_pixel_defs.h"
#define BAD_PIXEL_DEFS_H
#endif
#endif

/*  This file contains defines used by the level 1 Acis I/O library. The 
 *  library is intended to be used for the reading and writing of qpoe 
 *  data in the level 1 pipeline tools (ie. etvrawformat, evtfbformat, and
 *  evtacisdet).
 */

#define LEV_1_NO_ERROR       0   /* */
#define LEV_1_WRITE_ERR      1   /* */ 
#define LEV_1_FILE_DNE_ERR   2   /* */ 

/*  the following define is used for determining whether all of the pulse
 *  height values are contained in the input event file. If so, then a counter
 *  which is incremented each time a pulse height column is read in, should
 *  total upto ALL_PHS_EXIST.
 *
 *  EXISTENCE OF PH VALUES
 */

#define MAX_NUM_PHS        25 /* number of ph's - used for loops, offsets */ 
#define MAX_NUM_BIAS        9 /* number of bias columns                   */
#define ALL_PHS_EXIST      25 /* all expected ph's read from input file*/
#define MAX_ISL_COLS        5 /* max dimension of an event island axis */

/*  the following define is used in determining whether or not a given column
 *  exists or needs to be created. Indices of the various column attributes 
 *  are initialized to COLUMN_DNE and are subsequently changed to the correct
 *  index upon being read in, or created. The code can later check if an index
 *  exists by whether the respective indice is set to COLUMN_DNE.
 *
 *  EXISTENCE OF ATTRIBUTE COLUMNS
 */

#define COLUMN_DNE       -1 /* changed to a valid array index if col exists */ 


/*  the following defines are used to set various debugging levels for the
 *  evtacisdet module. These level values are compared against the debug
 *  parameter of evtacisdet.par file to determine the extent of debugging
 *  information to log to a 'runlog' file.
 *
 *  DEBUG LEVELS  
 */
 
#define DEBUG_LEVEL_0     0   /* default - do not log any debugging info */ 
#define DEBUG_LEVEL_1     1   /* only log major info                     */  
#define DEBUG_LEVEL_2     2   /* log input and output attributes         */  
#define DEBUG_LEVEL_3     3   /* imtermediate detailed logging           */ 
#define DEBUG_LEVEL_4     4   /* log column data values                  */  
#define DEBUG_LEVEL_5     5   /* log everything                          */  

#define ADET_LOG_TO_STDOUT "stdout" /* log goes to screen instead of a file */



/*  the following macros define the various modes of level 1 acis data.
 *  They are used by the level 1 tools to determine how to process the
 *  various level 1 modes
 *
 *                             512 256 128  64  32  16   8   4   2   1
 *     ---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---
 *    |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
 *     ---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---
 *                              H : R   R : G   G : V   F   F   F   F :
 *                              i : a   a : r   r : e   n   n   n   n :
 *                              s : w   w : d   d : r   t   t   t   t :
 *                              t :       :       : y                 :
 *                                : C   T : C   T :     B   B   C   T :
 *                                : C   E : C   E : F   S   S   C   E :
 *                                :       :       : n   M             :
 *                                | raw   | grade | t   P    faint    |
 *                                | format| format|          format   |
 */
#define MODE_UNKNOWN          0
#define MODE_FNT_TE           1
#define MODE_FNT_CC           2
#define MODE_FNT_BS           4
#define MODE_FNT_BSMP         8
#define MODE_VFNT_TE         16  
#define MODE_GRD_TE          32 
#define MODE_GRD_CC          64 
#define MODE_RAW_TE         128 
#define MODE_RAW_CC         256 
#define MODE_HISTOGRAM      512 
#define MODE_GRD_HISTOGRAM 1024
#define MODE_SPECIAL       2048
#define MODE_FNT_CC3x3     4096
#define MODE_GRD_CC3x3     8192
 
#define VFAINT_KEY_ALT  "VERYFAINT" 
#define VFAINT_KEY      "VFAINT" 
#define FAINT_KEY       "FAINT"
#define FAINT_BS_KEY    "FAINTBIAS"
#define FAINT_BS_KEY_ALT1  "FAINT_BIAS"
#define FAINT_BS_KEY_ALT2  "FAINT-BIAS"
#define GRADE_KEY       "GRADED"
#define RAW_KEY         "RAW"
#define HISTOGRAM_KEY   "HISTO"
#define GRADED_HISTO_KEY "GRADED_HISTO"
#define CC_RAW_KEY      "CC_RAW"
#define CC_FAINT_KEY    "CC_FAINT"
#define CC_GRADED_KEY   "CC_GRADED"
#define CC3x3_FAINT_KEY    "CC33_FAINT"
#define CC3x3_GRADED_KEY   "CC33_GRADED"
#define SPECIAL_KEY     "SPECIAL"
 
#define TE_MODE_KEY     "TIMED"
#define CC_MODE_KEY     "CONTINUOUS"
 
#define DATAMODE_KEY        "DATAMODE"
#define READMODE_KEY        "READMODE"
#define ISLE_SZ_KEY         "EVTIMODE"
#define GRADE_MODE_KEY      "GRADMODE"
 
#define KEY_WORD_LENGTH     16
 
#define FAINT_MODES        (MODE_FNT_TE | MODE_FNT_CC | MODE_FNT_BS | \
                             MODE_VFNT_TE | MODE_FNT_BSMP | MODE_FNT_CC3x3)
#define GRADED_MODES       (MODE_GRD_TE | MODE_GRD_CC | MODE_GRD_CC3x3)
#define RAW_MODES          (MODE_RAW_TE | MODE_RAW_CC)
#define CC_MODES           (MODE_FNT_CC | MODE_GRD_CC | MODE_RAW_CC | \
                              MODE_GRD_CC3x3 | MODE_FNT_CC3x3)
#define CC33_MODES         (MODE_GRD_CC3x3 | MODE_FNT_CC3x3)
#define TE_MODES           (MODE_FNT_TE | MODE_FNT_BS | MODE_GRD_TE | MODE_VFNT_TE)
#define TE_NOFNT_MODES     (MODE_FNT_TE | MODE_FNT_BS | MODE_GRD_TE)

#define CONTENT_TGEVT1        "TGEVT1"
#define CONTENT_EVT0          "EVT0"
#define CONTENT_EVT1          "EVT1"
#define CONTENT_EVT2          "EVT2"
#define CONTENT_ASPSOL        "ASPSOL"
#define CONTENT_ASPSOLOBI     "ASPSOLOBI"
#define CONTENT_OBCSOL        "OBCSOL"
#define CONTENT_LEV3ASPSOL    "ASPSOL3"
#define CONTENT_BADPIX        "BADPIX"
#define CONTENT_BADPIX_CDB    "CDB_ACIS_BADPIX"
#define CONTENT_GTI           "GTI"

#define CC_ISLAND_SZ         3
#define TE_ISLAND_SZ         9 
#define BK_SD_ISLAND_SZ     25 

#define GRADE_MODE_ASCA        "ASCA"   /* asca grading scheme */
#define GRADE_MODE_ACIS        "ACIS"   /* acis grading scheme */
#define GRADE_MODE_NONE        "NONE"   /* no grading scheme used */


/*  the following defines are used by level 1 event tools to determine the
 *  format of the data. This is necessary since the ordering of event island
 *  columns differ from SAC/ACIS-2C and FLIGHT.
 *
 *  DATA FORMATS
 */
#define EVT_FLT_FORMAT       0 
#define EVT_XRCF_FORMAT      1 
#define EVT_A2C_FORMAT       2 
#define EVT_SAC_FORMAT       3 

#define SZ_PARM_STRING      20 
 
/*  the following macro's are used in dealing with ccd node boundaries.
 *  The values assigned are TBR and are subject to change
 *
 * CCD NODES
 */
#define XMAXNODE1     256
#define XMAXNODE2     512
#define XMAXNODE3     768
#define XMAXNODE4    1024
#define YMAXNODES    1023
#define MINPIXEL        1
#define MAXPIXEL     1024
#define OUTSIDE_RGN    -2
#define OVR_CLK_RGN    -1
#define NODE_1_RGN      0
#define NODE_2_RGN      1
#define NODE_3_RGN      2
#define NODE_4_RGN      3
#define NUM_NODES       4 

#define CCD_UNKNOWN      -1  /* unknown or invalid ccdid value */
#define CCDID_KEY       "CCD_ID"   /* header keyword for ccd identifier */
#define MIN_VALID_CCD     0         /* ccd 0 - minimum accepted ccd value */
#define MAX_VALID_CCD     9         /* ccd 9 - minimum accepted ccd value */
#define NUM_CCDS         10 

#define CC_CHIPY_MIN       1 
#define CC_CHIPY_MAX     512
#define TE_CHIPX_MIN       2
#define TE_CHIPX_MAX    1023
#define TE_CHIPY_MIN       2
#define TE_CHIPY_MAX    1023


/*  the following is a list of all of the possible fields which may exist
 *  in the level 1 event file. These defines are used to map input event
 *  file columns to an event data stucture which will hold all data
 *  required for detector coordinate calculations on given events. The
 *  defines will also be used for mapping the data from the event data
 *  structure to the output event file.
 *
 *  ACIS EVENT FILE COLUMNS
 */

#define EVT_UNKNOWN            0  /* unknown column default              */ 
#define EVT_TIME               1  /* time of event                       */
#define EVT_DET                2  /* vector column so wcs can be added   */
#define EVT_DET_X              3  /*                                     */
#define EVT_DET_Y              4  /*                                     */
#define EVT_CHIP               5  /* vector column so wcs can be added   */
#define EVT_CHIPX              6  /*                                     */
#define EVT_CHIPY              7  /*                                     */
#define EVT_CCD                8  /* vector column so wcs can be added   */
#define EVT_CCD_X              9  /* ccd_x coord from level 0            */ 
#define EVT_CCD_Y             10  /* ccd_y coord from level 0            */ 
#define EVT_PHAS              11  /* pha array (fits) */
#define EVT_PHAS1             12  /* bias corrected pulse height column  */
#define EVT_PHAS2             13  /* bias corrected pulse height column  */
#define EVT_PHAS3             14  /* bias corrected pulse height column  */
#define EVT_PHAS4             15  /* bias corrected pulse height column  */
#define EVT_PHAS5             16  /* bias corrected pulse height column  */
#define EVT_PHAS6             17  /* bias corrected pulse height column  */
#define EVT_PHAS7             18  /* bias corrected pulse height column  */
#define EVT_PHAS8             19  /* bias corrected pulse height column  */
#define EVT_PHAS9             20  /* bias corrected pulse height column  */

#define EVT_PHAS10            21  /* bias corrected pulse height column  */
#define EVT_PHAS11            22  /* bias corrected pulse height column  */
#define EVT_PHAS12            23  /* bias corrected pulse height column  */
#define EVT_PHAS13            24  /* bias corrected pulse height column  */
#define EVT_PHAS14            25  /* bias corrected pulse height column  */
#define EVT_PHAS15            26  /* bias corrected pulse height column  */
#define EVT_PHAS16            27  /* bias corrected pulse height column  */
#define EVT_PHAS17            28  /* bias corrected pulse height column  */
#define EVT_PHAS18            29  /* bias corrected pulse height column  */
#define EVT_PHAS19            30  /* bias corrected pulse height column  */
#define EVT_PHAS20            31  /* bias corrected pulse height column  */
#define EVT_PHAS21            32  /* bias corrected pulse height column  */
#define EVT_PHAS22            33  /* bias corrected pulse height column  */
#define EVT_PHAS23            34  /* bias corrected pulse height column  */
#define EVT_PHAS24            35  /* bias corrected pulse height column  */
#define EVT_PHAS25            36  /* bias corrected pulse height column  */
#define EVT_BIAS1             37  /* input bias map data                 */
#define EVT_BIAS2             38  /* input bias map data                 */
#define EVT_BIAS3             39  /* input bias map data                 */
#define EVT_BIAS4             40  /* input bias map data                 */
#define EVT_BIAS5             41  /* input bias map data                 */
#define EVT_BIAS6             42  /* input bias map data                 */
#define EVT_BIAS7             43  /* input bias map data                 */
#define EVT_BIAS8             44  /* input bias map data                 */
#define EVT_BIAS9             45  /* input bias map data                 */
#define EVT_CCDID             62  /* id number of ccd event hit          */
#define EVT_EXPNO             63  /* exposure number                     */
#define EVT_PHA               64  /* event amplitude- sum of phas        */
#define EVT_FLTGRADE          65  /* event grade based on 9 pixel island */  
#define EVT_GRADE             66  /* ASCA or ACIS grade                  */
#define EVT_CCDNODE           67  /* node where event took place         */ 
#define EVT_SKY               68  /* vector column so wcs can be added   */
#define EVT_SKY_X             69  /* x axis sky coordinate               */ 
#define EVT_SKY_Y             70  /* y axis sky coordinate               */ 
#define EVT_SKY_1D            71  /*                                     */
#define EVT_TDET              72  /* vector column so wcs can be added   */
#define EVT_TDET_X            73  /* x axis tiled detector coordinate    */  
#define EVT_TDET_Y            74  /* y axis tiled detector coordinate    */  
#define EVT_FPZ               75  /* focal plane z coord (sky z)         */ 
#define EVT_RAWPHAS1          76  /* uncorrected input pha column        */
#define EVT_RAWPHAS2          77  /* uncorrected input pha column        */
#define EVT_RAWPHAS3          78  /* uncorrected input pha column        */
#define EVT_RAWPHAS4          79  /* uncorrected input pha column        */
#define EVT_RAWPHAS5          80  /* uncorrected input pha column        */
#define EVT_RAWPHAS6          81  /* uncorrected input pha column        */
#define EVT_RAWPHAS7          82  /* uncorrected input pha column        */
#define EVT_RAWPHAS8          83  /* uncorrected input pha column        */
#define EVT_RAWPHAS9          84  /* uncorrected input pha column        */
#define EVT_RAWPHAS10         85  /* uncorrected input pha column        */
#define EVT_RAWPHAS11         86  /* uncorrected input pha column        */
#define EVT_RAWPHAS12         87  /* uncorrected input pha column        */
#define EVT_RAWPHAS13         88  /* uncorrected input pha column        */
#define EVT_RAWPHAS14         89  /* uncorrected input pha column        */
#define EVT_RAWPHAS15         90  /* uncorrected input pha column        */
#define EVT_RAWPHAS16         91  /* uncorrected input pha column        */
#define EVT_RAWPHAS17         92  /* uncorrected input pha column        */
#define EVT_RAWPHAS18         93  /* uncorrected input pha column        */
#define EVT_RAWPHAS19         94  /* uncorrected input pha column        */
#define EVT_RAWPHAS20         95  /* uncorrected input pha column        */
#define EVT_RAWPHAS21         96  /* uncorrected input pha column        */
#define EVT_RAWPHAS22         97  /* uncorrected input pha column        */
#define EVT_RAWPHAS23         98  /* uncorrected input pha column        */
#define EVT_RAWPHAS24         99  /* uncorrected input pha column        */
#define EVT_RAWPHAS25        100  /* uncorrected input pha column        */

#define EVT_PHA_MEAN         101  /* pha_mean for graded mode            */
#define EVT_STATUS           102  /* status column                       */ 
#define EVT_FEP_ID           103  /* front end processor id              */
#define EVT_RAWBIAS1         104   /* clocking mode corrected bias column */
#define EVT_RAWBIAS2         105   /* clocking mode corrected bias column */
#define EVT_RAWBIAS3         106   /* clocking mode corrected bias column */
#define EVT_RAWBIAS4         107   /* clocking mode corrected bias column */
#define EVT_RAWBIAS5         108   /* clocking mode corrected bias column */
#define EVT_RAWBIAS6         109   /* clocking mode corrected bias column */
#define EVT_RAWBIAS7         110   /* clocking mode corrected bias column */
#define EVT_RAWBIAS8         111   /* clocking mode corrected bias column */
#define EVT_RAWBIAS9         112   /* clocking mode corrected bias column */
#define EVT_PULSE_INV        129   /* pulse invarience                    */
#define EVT_TROW             130   /* trow (ccd_y) for cc mode data       */
#define EVT_BIAS             131   /* bias array (fits) */
#define EVT_RAWPHAS          132  /* raw_pha array (fits) */
#define EVT_ENERGY           133   /* event energy (binned to pi)         */

#define EVT_ADJPHAS1         134  /* cti adjusted pha column            */
#define EVT_ADJPHAS2         135  /* cti adjusted pha column            */
#define EVT_ADJPHAS3         136  /* cti adjusted pha column            */
#define EVT_ADJPHAS4         137  /* cti adjusted pha column            */
#define EVT_ADJPHAS5         138  /* cti adjusted pha column            */
#define EVT_ADJPHAS6         139  /* cti adjusted pha column            */
#define EVT_ADJPHAS7         140  /* cti adjusted pha column            */
#define EVT_ADJPHAS8         141  /* cti adjusted pha column            */
#define EVT_ADJPHAS9         142  /* cti adjusted pha column            */
#define EVT_ADJPHAS10        143  /* cti adjusted pha column            */
#define EVT_ADJPHAS11        144  /* cti adjusted pha column            */
#define EVT_ADJPHAS12        145  /* cti adjusted pha column            */
#define EVT_ADJPHAS13        146  /* cti adjusted pha column            */
#define EVT_ADJPHAS14        147  /* cti adjusted pha column            */
#define EVT_ADJPHAS15        148  /* cti adjusted pha column            */
#define EVT_ADJPHAS16        149  /* cti adjusted pha column            */
#define EVT_ADJPHAS17        150  /* cti adjusted pha column            */
#define EVT_ADJPHAS18        151  /* cti adjusted pha column            */
#define EVT_ADJPHAS19        152  /* cti adjusted pha column            */
#define EVT_ADJPHAS20        153  /* cti adjusted pha column            */
#define EVT_ADJPHAS21        154  /* cti adjusted pha column            */
#define EVT_ADJPHAS22        155  /* cti adjusted pha column            */
#define EVT_ADJPHAS23        156  /* cti adjusted pha column            */
#define EVT_ADJPHAS24        157  /* cti adjusted pha column            */
#define EVT_ADJPHAS25        158  /* cti adjusted pha column            */
#define EVT_ADJPHAS          159  /* cti adjusted phas array            */

#define EVT_TIME_RO          160  /* read out time for cc-mode          */
#define EVT_PHA_RO           161  /* read out pha                       */

#define EVT_CHIP_ADJ         162
#define EVT_CHIPX_ADJ        163
#define EVT_CHIPY_ADJ        164

#define TG_M                 165
#define TG_CHIPY_TG          166
#define TG_CHIPY_ZO          167
#define TG_R                 168
#define TG_D                 169
#define TG_MLAM              170
#define TG_PART              171
#define TG_SRCID             172
#define TG_LAM               173
#define TG_SMAP              174
#define TG_GDP_X             175
#define TG_GDP_Y             176
#define TG_GAC               177 /* used for rd(tg_r, tg_d)*/
#define TG_GDP               178 /* used for gdp(gdpx, gdpy)*/

#define EVT_NUM_COLS         179  /*  total number of different fields   */

#define TG_X_AXIS             0
#define TG_Y_AXIS             1
#define TG_Z_AXIS             2

/*  the following defines represent the possible string column names that
 *  will be used to map event data to the appropriate fields in the event
 *  record structure
 *
 *  VALID COLUMN NAMES
 */

#define ADET_TIME_COL          "TIME"
#define ADET_TIME_RO_COL       "TIME_RO"
#define ADET_PHA_COL           "PHA"
#define ADET_PHA_RO_COL        "PHA_RO"
#define ADET_TOT_AMP           "TOT_AMP" 
#define ADET_DET_X_COL         "DETX"
#define ADET_DET_X_COL_ALT1    "DET_X"
#define ADET_DET_Y_COL         "DETY"
#define ADET_DET_Y_COL_ALT1    "DET_Y"
#define ADET_CHIP_X_COL        "CHIPX"
#define ADET_CHIP_X_COL_ALT1   "CHIP_X"
#define ADET_CHIP_Y_COL        "CHIPY"
#define ADET_CHIP_Y_COL_ALT1   "CHIP_Y"

#define ADET_CHIP_ADJ_COL      "CHIP_ADJ"
#define ADET_CHIPX_ADJ_COL     "CHIPX_ADJ"
#define ADET_CHIPY_ADJ_COL     "CHIPY_ADJ"

#define ADET_PHAS_1_COL        "PHAS1" 
#define ADET_PHAS_2_COL        "PHAS2" 
#define ADET_PHAS_3_COL        "PHAS3" 
#define ADET_PHAS_4_COL        "PHAS4" 
#define ADET_PHAS_5_COL        "PHAS5" 
#define ADET_PHAS_6_COL        "PHAS6" 
#define ADET_PHAS_7_COL        "PHAS7" 
#define ADET_PHAS_8_COL        "PHAS8" 
#define ADET_PHAS_9_COL        "PHAS9" 
#define ADET_PHAS_10_COL       "PHAS10" 
#define ADET_PHAS_11_COL       "PHAS11" 
#define ADET_PHAS_12_COL       "PHAS12" 
#define ADET_PHAS_13_COL       "PHAS13" 
#define ADET_PHAS_14_COL       "PHAS14" 
#define ADET_PHAS_15_COL       "PHAS15" 
#define ADET_PHAS_16_COL       "PHAS16" 
#define ADET_PHAS_17_COL       "PHAS17" 
#define ADET_PHAS_18_COL       "PHAS18" 
#define ADET_PHAS_19_COL       "PHAS19" 
#define ADET_PHAS_20_COL       "PHAS20" 
#define ADET_PHAS_21_COL       "PHAS21" 
#define ADET_PHAS_22_COL       "PHAS22" 
#define ADET_PHAS_23_COL       "PHAS23" 
#define ADET_PHAS_24_COL       "PHAS24" 
#define ADET_PHAS_25_COL       "PHAS25" 
#define ADET_BIAS_1_COL        "BIAS1"
#define ADET_BIAS_2_COL        "BIAS2"
#define ADET_BIAS_3_COL        "BIAS3"
#define ADET_BIAS_4_COL        "BIAS4"
#define ADET_BIAS_5_COL        "BIAS5"
#define ADET_BIAS_6_COL        "BIAS6"
#define ADET_BIAS_7_COL        "BIAS7"
#define ADET_BIAS_8_COL        "BIAS8"
#define ADET_BIAS_9_COL        "BIAS9"
#define ADET_CCD_ID_COL        "CCDID"
#define ADET_CCD_ID_COL_ALT1   "CCD_ID"
#define ADET_CCD_NODE_COL      "CCDNODE"
#define ADET_CCD_NODE_COL_ALT1 "CCD_NODE"
#define ADET_CCD_NODE_COL_ALT2 "NODE_ID"
#define ADET_CCD_NODE_COL_ALT3 "NODEID"
#define ADET_EXP_NO_COL        "EXPNO"
#define ADET_FLT_GRD_COL       "FLTGRADE"
#define ADET_FLT_GRD_COL_ALT1  "FLT_GRD"
#define ADET_GRADE_COL         "GRADE" 
#define ADET_SKY_X_COL         "X"
#define ADET_SKY_Y_COL         "Y"
#define ADET_TDET_X_COL        "TDETX" 
#define ADET_TDET_X_COL_ALT1   "TDET_X" 
#define ADET_TDET_Y_COL        "TDETY" 
#define ADET_TDET_Y_COL_ALT1   "TDET_Y" 
#define ADET_PHA_MEAN_COL      "PHA_MEAN"
#define ADET_PHA_MEAN_COL_ALT1 "CORN_PHA"
#define ADET_RAWPHAS_1_COL     "RAWPHAS1"
#define ADET_RAWPHAS_2_COL     "RAWPHAS2"
#define ADET_RAWPHAS_3_COL     "RAWPHAS3"
#define ADET_RAWPHAS_4_COL     "RAWPHAS4"
#define ADET_RAWPHAS_5_COL     "RAWPHAS5"
#define ADET_RAWPHAS_6_COL     "RAWPHAS6"
#define ADET_RAWPHAS_7_COL     "RAWPHAS7"
#define ADET_RAWPHAS_8_COL     "RAWPHAS8"
#define ADET_RAWPHAS_9_COL     "RAWPHAS9"
#define ADET_RAWPHAS_10_COL    "RAWPHAS10"
#define ADET_RAWPHAS_11_COL    "RAWPHAS11"
#define ADET_RAWPHAS_12_COL    "RAWPHAS12"
#define ADET_RAWPHAS_13_COL    "RAWPHAS13"
#define ADET_RAWPHAS_14_COL    "RAWPHAS14"
#define ADET_RAWPHAS_15_COL    "RAWPHAS15"
#define ADET_RAWPHAS_16_COL    "RAWPHAS16"
#define ADET_RAWPHAS_17_COL    "RAWPHAS17"
#define ADET_RAWPHAS_18_COL    "RAWPHAS18"
#define ADET_RAWPHAS_19_COL    "RAWPHAS19"
#define ADET_RAWPHAS_20_COL    "RAWPHAS20"
#define ADET_RAWPHAS_21_COL    "RAWPHAS21"
#define ADET_RAWPHAS_22_COL    "RAWPHAS22"
#define ADET_RAWPHAS_23_COL    "RAWPHAS23"
#define ADET_RAWPHAS_24_COL    "RAWPHAS24"
#define ADET_RAWPHAS_25_COL    "RAWPHAS25"
#define ADET_RAWPHAS_1_COL_ALT1  "RAW_PHAS1"
#define ADET_RAWPHAS_2_COL_ALT1  "RAW_PHAS2"
#define ADET_RAWPHAS_3_COL_ALT1  "RAW_PHAS3"
#define ADET_RAWPHAS_4_COL_ALT1  "RAW_PHAS4"
#define ADET_RAWPHAS_5_COL_ALT1  "RAW_PHAS5"
#define ADET_RAWPHAS_6_COL_ALT1  "RAW_PHAS6"
#define ADET_RAWPHAS_7_COL_ALT1  "RAW_PHAS7"
#define ADET_RAWPHAS_8_COL_ALT1  "RAW_PHAS8"
#define ADET_RAWPHAS_9_COL_ALT1  "RAW_PHAS9"
#define ADET_RAWPHAS_10_COL_ALT1  "RAW_PHAS10"
#define ADET_RAWPHAS_11_COL_ALT1  "RAW_PHAS11"
#define ADET_RAWPHAS_12_COL_ALT1  "RAW_PHAS12"
#define ADET_RAWPHAS_13_COL_ALT1  "RAW_PHAS13"
#define ADET_RAWPHAS_14_COL_ALT1  "RAW_PHAS14"
#define ADET_RAWPHAS_15_COL_ALT1  "RAW_PHAS15"
#define ADET_RAWPHAS_16_COL_ALT1  "RAW_PHAS16"
#define ADET_RAWPHAS_17_COL_ALT1  "RAW_PHAS17"
#define ADET_RAWPHAS_18_COL_ALT1  "RAW_PHAS18"
#define ADET_RAWPHAS_19_COL_ALT1  "RAW_PHAS19"
#define ADET_RAWPHAS_20_COL_ALT1  "RAW_PHAS20"
#define ADET_RAWPHAS_21_COL_ALT1  "RAW_PHAS21"
#define ADET_RAWPHAS_22_COL_ALT1  "RAW_PHAS22"
#define ADET_RAWPHAS_23_COL_ALT1  "RAW_PHAS23"
#define ADET_RAWPHAS_24_COL_ALT1  "RAW_PHAS24"
#define ADET_RAWPHAS_25_COL_ALT1  "RAW_PHAS25"
#define ADET_FPZ_COL           "FPZ"
#define ADET_STATUS_COL        "STATUS" 
#define ADET_FEP_ID_COL        "FEP_ID" 
#define ADET_RAW_BIAS_1_COL    "RAW_BIAS1"
#define ADET_RAW_BIAS_2_COL    "RAW_BIAS2"
#define ADET_RAW_BIAS_3_COL    "RAW_BIAS3"
#define ADET_RAW_BIAS_4_COL    "RAW_BIAS4"
#define ADET_RAW_BIAS_5_COL    "RAW_BIAS5"
#define ADET_RAW_BIAS_6_COL    "RAW_BIAS6"
#define ADET_RAW_BIAS_7_COL    "RAW_BIAS7"
#define ADET_RAW_BIAS_8_COL    "RAW_BIAS8"
#define ADET_RAW_BIAS_9_COL    "RAW_BIAS9"
#define ADET_PULSE_INV_COL     "PI" 
#define ADET_CCD_X_COL         "CCDX"
#define ADET_CCD_Y_COL         "CCDY"
#define ADET_CCD_X_COL_ALT1    "CCD_X"
#define ADET_CCD_Y_COL_ALT1    "CCD_Y"
#define ADET_TROW_COL          "TROW"
#define ADET_PHAS_COL          "PHAS" 
#define ADET_RAWPHAS_COL       "RAW_PHAS" 
#define ADET_ADJPHAS_COL       "PHAS_ADJ" 
#define ADET_BIAS_COL          "BIAS" 
#define ADET_SKY_COL           "SKY" 
#define ADET_SKY_1D_COL        "SKY_1D" 
#define ADET_CCD_COL           "CCD" 
#define ADET_CHIP_COL          "CHIP"
#define ADET_DET_COL           "DET"
#define ADET_TDET_COL          "TDET" 
#define ADET_ENERGY_COL        "ENERGY"

#define ADET_ADJPHAS_1_COL     "PHAS_ADJ1"
#define ADET_ADJPHAS_2_COL     "PHAS_ADJ2"
#define ADET_ADJPHAS_3_COL     "PHAS_ADJ3"
#define ADET_ADJPHAS_4_COL     "PHAS_ADJ4"
#define ADET_ADJPHAS_5_COL     "PHAS_ADJ5"
#define ADET_ADJPHAS_6_COL     "PHAS_ADJ6"
#define ADET_ADJPHAS_7_COL     "PHAS_ADJ7"
#define ADET_ADJPHAS_8_COL     "PHAS_ADJ8"
#define ADET_ADJPHAS_9_COL     "PHAS_ADJ9"
#define ADET_ADJPHAS_10_COL    "PHAS_ADJ10"
#define ADET_ADJPHAS_11_COL    "PHAS_ADJ11"
#define ADET_ADJPHAS_12_COL    "PHAS_ADJ12"
#define ADET_ADJPHAS_13_COL    "PHAS_ADJ13"
#define ADET_ADJPHAS_14_COL    "PHAS_ADJ14"
#define ADET_ADJPHAS_15_COL    "PHAS_ADJ15"
#define ADET_ADJPHAS_16_COL    "PHAS_ADJ16"
#define ADET_ADJPHAS_17_COL    "PHAS_ADJ17"
#define ADET_ADJPHAS_18_COL    "PHAS_ADJ18"
#define ADET_ADJPHAS_19_COL    "PHAS_ADJ19"
#define ADET_ADJPHAS_20_COL    "PHAS_ADJ20"
#define ADET_ADJPHAS_21_COL    "PHAS_ADJ21"
#define ADET_ADJPHAS_22_COL    "PHAS_ADJ22"
#define ADET_ADJPHAS_23_COL    "PHAS_ADJ23"
#define ADET_ADJPHAS_24_COL    "PHAS_ADJ24"
#define ADET_ADJPHAS_25_COL    "PHAS_ADJ25"

#define TG_M_COL                "TG_M"
#define TG_CHIPY_TG_COL         "CHIPY_TG"
#define TG_CHIPY_ZO_COL         "CHIPY_ZO"
#define TG_R_COL                "TG_R"
#define TG_D_COL                "TG_D"
#define TG_GAC_COL              "RD"         /* column  rd( tg_r, tg_d )*/
#define TG_MLAM_COL             "TG_MLAM"
#define TG_PART_COL             "TG_PART"
#define TG_SRCID_COL            "TG_SRCID"
#define TG_LAM_COL              "TG_LAM"
#define TG_SMAP_COL             "TG_SMAP"
#define TG_GDP_X_COL            "GDPX"       /* column  gdpx */
#define TG_GDP_Y_COL            "GDPY"       /* column  gdpy */
#define TG_GDP_COL              "GDP"        /* column  gdp(gdpx, gdpy) */

 /* ranges for events */
#define DS_GRT_TG_R_TLMIN        -2.0  /* gratings tg_r minimum value   */
#define DS_GRT_TG_R_TLMAX         2.0  /* gratings tg_r maximum value   */
#define DS_GRT_TG_R_TDBIN    1.593E-4  /* gratings tg_r tdbin value     */
#define DS_GRT_TG_D_TLMIN        -2.0  /* gratings tg_d minimum value   */
#define DS_GRT_TG_D_TLMAX         2.0  /* gratings tg_d maximum value   */
#define DS_GRT_TG_D_TDBIN    1.593E-4  /* gratings tg_d tdbin value     */
#define DS_GRT_TG_MLAM_TLMIN   -400.0  /* gratings tg_mlam minimum value*/
#define DS_GRT_TG_MLAM_TLMAX    400.0  /* gratings tg_mlam maximum value*/
#define DS_GRT_TG_MLAM_TDBIN   2.0E-3  /* gratings tg_mlam tdbin value  */
#define DS_GRT_TG_M_TLMIN         -62  /* gratings tg_m minimum value   */
#define DS_GRT_TG_M_TLMAX          62  /* gratings tg_m maximum value   */
#define DS_GRT_TG_LAM_TLMIN       0.0  /* gratings tg_lam minimum value */
#define DS_GRT_TG_LAM_TLMAX     400.0  /* gratings tg_lam maximum value */
#define DS_GRT_TG_LAM_TDBIN    2.0E-3  /* gratings tg_lam tdbin value   */
#define DS_GRT_TG_PART_TLMIN        0  /* gratings tg_part minimum value*/
#define DS_GRT_TG_PART_TLMAX       99  /* gratings tg_part maximum value*/
#define DS_GRT_TG_SMAP_TLMIN        0  /* gratings tg_smap minimum value*/
#define DS_GRT_TG_SMAP_TLMAX    32767  /* gratings tg_smap maximum value*/
#define DS_GRT_TG_SRCID_TLMIN       0  /* gratings tg_srcid minimum val */
#define DS_GRT_TG_SRCID_TLMAX   32767  /* gratings tg_srcid maximum val */
#define DS_GRT_GDPX_TLMIN           1  /* gratings gdpx minimum value   */
#define DS_GRT_GDPX_TLMAX       65535  /* gratings gdpx maximum value   */
#define DS_GRT_GDPX_TDBIN          10  /* gratings gdpx tdbin value     */
#define DS_GRT_GDPY_TLMIN           1  /* gratings gdpy minimum value   */
#define DS_GRT_GDPY_TLMAX       32767  /* gratings gdpy maximum value   */
#define DS_GRT_GDPY_TDBIN          10  /* gratings gdpy tdbin value     */

/* units pertaining to events extensions */
#define DS_GRT_TG_RD_TUNITS         "deg"  /* units for event tg_r/d columns */
#define DS_GRT_TG_MLAM_TUNITS  "angstrom"  /* units for event tl_mlam column */
#define DS_GRT_TG_LAM_TUNITS   "angstrom"  /* units for event tg_lam column  */
#define DS_GRT_GDP_TUNITS         "pixel"  /* units for event gdp x/y columns*/
#define DS_GRT_X_TUNITS           "pixel"  /* units for region x column      */
#define DS_GRT_Y_TUNITS           "pixel"  /* units for region y column      */
#define DS_GRT_R_TUNITS           "pixel"  /* units for region r column      */
#define DS_GRT_ROTANG_TUNITS        "deg"  /* units for region rotang column */

/*  the following are a list of maximum valid chip values for the various
 *  instruments based on values specified in the level 1 to archive ICD's.
 *  (minimums are 1, and maximums are non-inclusive)
 *
 *    acis level 1 to archive ICD     rev 1.7   10/22/98
 *    hrc  level 1 to archive ICD     rev 1.5   11/03/98
 *
 *  CHIP RANGES
 */
#define TG_ACIS_MIN_CHIP_VAL           1.0
#define TG_ACIS_MAX_CHIP_VAL        1024.0
#define TG_HRCI_MIN_CHIP_VAL           0.0
#define TG_HRCI_MAX_CHIP_VAL       16385.0
#define TG_HRCS_MIN_CHIP_VAL           0.0
#define TG_HRCS_MAX_CHIPX_VAL       4097.0
#define TG_HRCS_MAX_CHIPY_VAL      16385.0
/* default acis-s ccd size */
#define DEFAULT_ACIS_S_CCD_SIZE        1024
#define DEFAULT_ACIS_S_CHIP_Y_MIDDLE    512

/*  the following macros define the ranges of various arrays used in the
 *  evtacisdet code. They are meant to assist in the readability of the code
 *  If these values are changed to adjust structure sizes, the corresponding
 *  code must be compensated as well.
 *
 *  MISC. DEFINES
 */

#define ADET_NUM_AXES   2  
#define ADET_NUM_COLS   MAX_NUM_PHS 
#define ADET_X_AXIS     0
#define ADET_Y_AXIS     1 
#define ADET_Z_AXIS     2  
#define ACIS_SYS_LEN          32 /* length of acis system string */ 
#define DEFAULT_CC_MODE_Y    512 /* value for cc chipy for det/sky coords */


/*  The following layout and masks define the current status bit implementation
 *  for level 1 acis tools. In the future these bits will most likely be 
 *  concatenated with other acis status bits to create a generalized acis
 *  status word.  
 *
 *
 *          15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
 *         +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
 *         |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
 *         +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
 * 
 *  EVENT POSITION
 *  - 00 - This bit is set to 1 if an event's chip coordinates fall outside of
 *         the valid region. This may be due to windowing, active-inactive ccds,
 *         etc...
 *  
 *  PULSE HEIGHTS
 *  - 01 - This bit is used to flag events in which the center pixel of the 3x3
 *         event island does not contain the local maxima (highest pha value in
 *         the island).
 * 
 *  - 02 - This bit is set to 1 for any event which has one or more of its
 *         individual island pixels > 4095.
 * 
 *  - 03 - This bit is set to 1 to announce the detection of an overflow
 *         condition in the summing of event island columns. Since 'pha' (sum of
 *         event island pulse heights) have a range from 0-36855 and the pha
 *         category can only handle short integers (numbers upto 32767), this
 *         bit will flag events which exceed the limit. In addition, the 'pha'
 *         sum of such an event will be set to the default value of 32767.
 * 
 * BAD PIXELS
 *  - 04 - A value of 1 in this bit means that the event's local maxima falls on
 *         a pixel identified in bad pixel map as being 'bad'.
 * 
 *  - 05 - This bit is used to indicate that one or more of an event's edge or
 *         corner pixels falls on a pixel identified within the bad pixel map.
 * 
 * BAD BIAS
 *  - 06 - This pixel flags events which have bad bias map values. The flag is
 *         set when bias values are set to 4095 by the onboard software.
 * 
 *  - 07 - This bit position indicates missing or unknown bias values for an
 *         event pixel. This bit is set due to situations such as telemetry
 *         dropout.
 * 
 *  - 08 - Parity errors are recorded for events by setting this bit to 1. The
 *         bit is set when the onboard software returns a bias value of 4094.
 * 
 * BAD OVERCLOCK VALUES
 *  - 09 - This bit is set for events for which overclock information is missing
 *         or does not exist.
 * 
 *  - 10 - Setting this status bit to 1 indicates that the corresponding event's
 *         overclock values fall outside of a specified range (nominal case
 *         0>= oc_val >= 500)
 * 
 * BAD CORNER MEAN (GRADED MODE ONLY)
 *  - 11 - corner mean below -4095
 *         This bit is used to flag events where the onboard software has set
 *         the corner mean value to -4096 to indicate an extremely low corner
 *         mean value (see ECO 36-946).
 * 
 *  - 12/13 - used to indicate the number of corners missing from the
 *         corner mean (ie bit pattern 01=1 missing, 10=2 missing, and 11=3
 *         missing corners). These bits will be set based on the results of
 *         checking the bad pixel and bad bias values of the corners surrounding
 *         the center position of graded mode data. These bits tell the number
 *         of corners missing but does not identify which corners are missing.
 *         These bits should be mutually exclusive from bit 14 (see below).
 * 
 *  - 14 - corner mean missing (ie. no valid corner pixels)
 *         This bit is set to 1 if the corner mean value telemetered from the
 *         onboard software is set to 4095 (see ECO 36-946) and represents a
 *         graded mode event which has no valid corner pixels. Whether or not to
 *         set this if the level 1 software receives an event with 4 bad corners
 *         that wasn't flagged with a value of 4095 by the onboard software, is
 *         polemical.
 * 
 *  - 15 - unused - will eventually be used for destreak
 *
 * AFTERGLOW COUNTS 
 *  - 16:19 - afterglow chain length
 *         These four bits are interpreted as an integer value of 0 to 15.
 *         The value is one less than the length of the flaring pixel chain. 
 *         A value of zero indicates no flaring pixels while a value of 15
 *         indicates a chain of 16 or more flaring pixels.  
 * 
 * Notes: The status word and bit masks identified are specific to front-end
 *      level 1 processing. Back-end level 1 status bits (TBD) will be logically
 *      grouped- either as an independant status word or as the high end bits of
 *      a 32 bit long.
 *
 *
 *                STATUS                MASK VALUE
 *         ---------------------        ----------
 *         L1_STS_GOOD_EVENT              0x00000
 *         L1_STS_BAD_POS                 0x00001
 *         L1_STS_LM_BAD_VAL              0x00002
 *         L1_STS_PIX_RANGE               0x00004
 *         L1_STS_BAD_SUM                 0x00008
 *         L1_STS_LM_BAD_PIX              0x00010
 *         L1_STS_ADJ_BAD_PIX             0x00020
 *         L1_STS_BIAS_BAD_PIX            0x00040
 *         L1_STS_BIAS_MISSING            0x00080
 *         L1_STS_BIAS_PARITY             0x00100
 *         L1_STS_OC_MISSING              0x00200
 *         L1_STS_OC_RANGE                0x00400
 *         L1_STS_CM_LOW                  0x00800
 *         L1_STS_CM_1_MISSING            0x01000
 *         L1_STS_CM_2_MISSING            0x02000
 *         L1_STS_CM_3_MISSING            0x03000
 *         L1_STS_CM_4_MISSING            0x04000
 *         UNUSED (destreak)              0x08000
 *         L1_STS_CHAIN_2                 0x10000
 *         L1_STS_CHAIN_3                 0x20000
 *         L1_STS_CHAIN_4                 0x30000
 *         L1_STS_CHAIN_5                 0x40000
 *         L1_STS_CHAIN_6                 0x50000
 *         L1_STS_CHAIN_7                 0x60000
 *         L1_STS_CHAIN_8                 0x70000
 *         L1_STS_CHAIN_9                 0x80000
 *         L1_STS_CHAIN_10                0x90000
 *         L1_STS_CHAIN_11                0xA0000
 *         L1_STS_CHAIN_12                0xB0000
 *         L1_STS_CHAIN_13                0xC0000
 *         L1_STS_CHAIN_14                0xD0000
 *         L1_STS_CHAIN_15                0xE0000
 *         L1_STS_CHAIN_N                 0xF0000

 *         
 *
 *  LEVEL 1 STATUS MASK
 */
 
typedef unsigned long L1_STATUS_T; 
 
#define L1_STS_GOOD_EVENT           ((L1_STATUS_T) 0x00000000) 
#define L1_STS_BAD_POS              ((L1_STATUS_T) 0x00000001) 
#define L1_STS_LM_BAD_VAL           ((L1_STATUS_T) 0x00000002) 
#define L1_STS_PIX_RANGE            ((L1_STATUS_T) 0x00000004) 
#define L1_STS_BAD_SUM              ((L1_STATUS_T) 0x00000008) 
#define L1_STS_LM_BAD_PIX           ((L1_STATUS_T) 0x00000010) 
#define L1_STS_ADJ_BAD_PIX          ((L1_STATUS_T) 0x00000020) 
#define L1_STS_BIAS_BAD_PIX         ((L1_STATUS_T) 0x00000040) 
#define L1_STS_BIAS_MISSING         ((L1_STATUS_T) 0x00000080) 
#define L1_STS_BIAS_PARITY          ((L1_STATUS_T) 0x00000100) 
#define L1_STS_OC_MISSING           ((L1_STATUS_T) 0x00000200) 
#define L1_STS_OC_RANGE             ((L1_STATUS_T) 0x00000400) 
#define L1_STS_CM_LOW               ((L1_STATUS_T) 0x00000800) 
#define L1_STS_CM_1_MISSING         ((L1_STATUS_T) 0x00001000) 
#define L1_STS_CM_2_MISSING         ((L1_STATUS_T) 0x00002000) 
#define L1_STS_CM_3_MISSING         ((L1_STATUS_T) 0x00003000) 
#define L1_STS_CM_4_MISSING         ((L1_STATUS_T) 0x00004000) 

#define L1_STS_CHAIN_2              ((L1_STATUS_T) 0x00010000) 
#define L1_STS_CHAIN_3              ((L1_STATUS_T) 0x00020000) 
#define L1_STS_CHAIN_4              ((L1_STATUS_T) 0x00030000) 
#define L1_STS_CHAIN_5              ((L1_STATUS_T) 0x00040000) 
#define L1_STS_CHAIN_6              ((L1_STATUS_T) 0x00050000) 
#define L1_STS_CHAIN_7              ((L1_STATUS_T) 0x00060000) 
#define L1_STS_CHAIN_8              ((L1_STATUS_T) 0x00070000) 
#define L1_STS_CHAIN_9              ((L1_STATUS_T) 0x00080000) 
#define L1_STS_CHAIN_10             ((L1_STATUS_T) 0x00090000) 
#define L1_STS_CHAIN_11             ((L1_STATUS_T) 0x000A0000) 
#define L1_STS_CHAIN_12             ((L1_STATUS_T) 0x000B0000) 
#define L1_STS_CHAIN_13             ((L1_STATUS_T) 0x000C0000) 
#define L1_STS_CHAIN_14             ((L1_STATUS_T) 0x000D0000) 
#define L1_STS_CHAIN_15             ((L1_STATUS_T) 0x000E0000) 
#define L1_STS_CHAIN_N              ((L1_STATUS_T) 0x000F0000) 

#define L1_STS_BAD_CTI              ((L1_STATUS_T) 0x00100000) 
#define L1_STS_VF_BAD_PIX           ((L1_STATUS_T) 0x00800000) 
#define L1_BAD_CENTROID             ((L1_STATUS_T) 0x00200000)

#define L1_STAT_TO_ZERO             ((L1_STATUS_T) 0x008F403E)
#define L1_STAT_TO_ZERO_EVTGRD      ((L1_STATUS_T) 0x009F403E)

/*Not in status bit spec, using bit 24 from unused section
 * 3/28/2013
 * */
#define L1_STS_PHASZERO             ((L1_STATUS_T) 0x01000000)

/*  the following structure holds all information pertaining to any single
 *  event being processed. Data from the input file is loaded into this
 *  structure and is processed. Relevant calculations such as raw x and
 *  y coordinates are stored after being calculated. Upon completion of
 *  processing, evtacisdet writes out specified fields from this structure
 *  to an output event file. These output fields are cast as appropriate.
 *
 *  EVENT RECORD STRUCTURE
 */


typedef struct event_rec_t {
   double    time;                    /* time of event                       */
   double    time_ro;                 /* readout time of event (cc-mode)     */
   double    time_work;               /* intermedi time of event (cc-mode)   */
   VEC2_DBLE det;                     /* detector x and y coords             */
   VEC2_DBLE ccd;                     /* ccd x and y coords (chip coords -1) */
   VEC2_DBLE chip;                    /* chip x and y coords (ccdx+1,ccdy+1) */
   VEC2_DBLE centchip;                /* centroided chip coords              */
   VEC2_DBLE work;                    /* coords used in transformations      */
   VEC2_DBLE tdet;                    /* x and y tiled detector coords       */
   VEC2_DBLE fppos;                   /* x, y, z focal plane coords          */
   VEC2_DBLE sky;                     /* x and y sky (XY) coordinates        */
   double    radec[2];                /* dm computed ra/dec                  */
   double    sky_1d;                  /* cc mode sky y coordinate            */
   double    trow;                    /* trow (ccd_y) for cc mode data       */
   long      expno;                   /* exposure number                     */
   float     work_phas[ADET_NUM_COLS]; /* phas values to use in computations */
   float     phas_adj[ADET_NUM_COLS]; /* cti correct phas values             */
   float     energy;                  /* event ENERGY (binned into PI)       */
   long      pha;                     /* sum of pulse height columns         */
   long      pha_ro;                  /* readout sum of pulse height columns */
   long      pi;                      /* pulse invarience                    */
   L1_STATUS_T status;                /* front end level 1 status word       */
   L1_STATUS_T status_centroid[9]; /*status used for pix_adjust = centroid*/
   bool status_cent_valid[ADET_NUM_COLS]; /*status used for pix_adjust = centroid*/
   short     rawphas[ADET_NUM_COLS];  /* pha value from level 0              */
   short     phas[ADET_NUM_COLS];     /* pixels that make up an event island */
   short     bias[MAX_NUM_BIAS];      /* bias value for event pixel          */
   short     ccdid;                   /* identifies ccd event occurred on    */
   short     ccdnode;                 /*                                     */
   short     fltgrade;                /* flt grade of event (0..255)         */
   short     grade;                   /* asca (0..7) or acis grade (0..8)    */
   short     pha_mean;                /* mean of corrected corner phas       */
   short     fep_id;                  /* front end processor id              */
   /* used for ccd mode gratings calculations */
   short tg_m;
   VEC2_DBLE chip_adj;
   double chipy_adj_work;
   double chipy_tg;
   double chipy_zo;
   double tg_r;
   double tg_d;
   double tg_gdp_x;
   double tg_gdp_y;
   double tg_mlam;
   short tg_part;
   short tg_srcid;
   double tg_lam;
   short tg_smap;
   VEC2_DBLE tg_gdp_xy; /* Grating diffraction pixel; cols gdpx,gdpy or gdp(gdpx,gdpy)*/
   VEC2_DBLE tg_gac_rd; /* Linear diffraction coords; cols tg_r,tgd,rd(tg_r,tg_d)*/
   /*****/
   struct event_rec_t *next;          /* pointer to next event               */
} EVENT_REC_T, *EVENT_REC_P_T;



/*  the following macro's are used to represent the various keywords which are
 *  used to identify various data systems
 */
 
#define MAX_INST_KYWRD_LEN      80
#define AC_FP_SYS               "ACSYSFP"
#define AC_CHP_SYS              "ACSYSCHP"
#define AC_TDET_SYS             "ACSYSTDT"
#define TELESCOP_KEY            "TELESCOP"



/*  the following typedef is used to store the various parameter file keywords
 *  which are read in from the instrument parameter file and written out to
 *  the output event file.
 *
 *  INSTRUMENT KEYWORDS STRUCTURE
 */

typedef struct inst_keywords_t {
   VECL_CHAR fp_system;
   VECL_CHAR chip_system;
   VECL_CHAR tdet_system;
   VECS_CHAR detname;
} INST_KEYWORDS_T, *INST_KEYWORDS_P_T;
 

/*  The following structure is used by acis level 1 tools to store information
 *  pertaining to event files (both input and output). the fields in this
 *  structure are populated via a call to setup_event_files().
 *
 *  EVENT FILE SETUP STRUCTURE
 */
typedef struct event_setup_t {
   dmDataset*    dataset;  /* dataset (file) handle */
   dmBlock*      primary;  /* header keyword extension handle */
   dmBlock*      extension; /* event data extension handle */
   dmDescriptor**  desc;   /* event attributes */
   long*         dim;      /* column dimension (> 1 for vectors) */ 
   int           num_cols; /* number of columns in event file */
   int           num_rows; /* number of rows in the event file */
   dmDataType*   types;    /* ptr to array of event data column types */
   short*        mapping;  /* mapping between data structure and file columns */
   short         island_sz; /* size of pha island array (1,3,9,25) */ 
} EVENT_SETUP_T, *EVENT_SETUP_P_T;

/* function to read data into a level 1 event structure from a qpoe file */ 
extern void load_event_data(EVENT_SETUP_P_T, EVENT_REC_P_T);

/* function to parse column names into a numerical mapping */
extern bool parse_acis_evt_columns(char**, int, short*);

/* function to write specified event columns to an output qpoe file */
extern int write_acis_events(EVENT_SETUP_P_T, EVENT_REC_P_T);

/* function to verify the existence of a specified file */ 
extern boolean check_file_existance (char*); 

/* routine to write out instrument parameter file keywords */
extern void write_instrume_params(dmBlock*, INST_KEYWORDS_P_T);

/* function to read a header keyword and determine the vent dat's mode */
extern short determine_acis_mode(dmBlock*);

/* routine to determine the size of an acis event island */
extern short determine_island_size(dmBlock*, short); 
 
/* function to determine the node a pair of (x,y) coordinates falls on */
extern int FindNode(short, short);

/* function to remap acis-2c format data to flight format */
extern void convert_acis2c_to_flight(short*, short);

/* function to remap flight data to acis-2c format */
extern void convert_flight_to_acis2c(short*, short);

/* function to map a column of the badpixel file to the exp data structure */
extern short map_badp_table_column(char[]);
 
#ifdef USE_BADPIXEL 
/* function to load an badpixel column data field based on mapping */
extern void load_badpixel_column(XPOINTER, XPOINTER, BADPIX_REC_P_T,
                 short, int);
 
/* function to open an badpixel file table and setup the mapping */
extern boolean open_badpixel_file(char*, XPOINTER*, XPOINTER**,
                               int*, int*, short**);
 
/* function to determine if an event contains any bad pixels */
extern L1_STATUS_T check_event_for_badpixels (long[][BP_MAX_COORD], int, 
                 int, short, short);
 
/* function to set up the internal bad pixel mask for a particular bad pixel */
extern boolean set_badpixel_map_values (long[][BP_MAX_COORD], int, int, short);
 
/* function to load a stack of bad pixel data into an internal bad pixel mask */
extern void load_bad_pixel_files(char*, long[][BP_MAX_COORD]);
#endif 

/* function to specify the array subscript of an event islands local maxima */
extern short determine_local_maxima(short); 

/* function to determine ccd_id of an event file */
extern short determine_event_ccdid (dmBlock*);

/*  function to convert strings to lowercase */ 
void string_to_lowercase(char*, short); 

/* function to convert datamode/readmode strings to a mode value */
extern short set_acis_mode(char*, char*); 

/* function to open a block based on a string keyword value */
extern dmBlock* get_extension_by_string_keyvalue(dmDataset*, char*, char*);

/* function to check if chip coordinates are on the ccd */ 
extern L1_STATUS_T verify_chip_range(double, double, short); 

/* function to check if chip coordinates are on the ccd */
extern L1_STATUS_T verify_chip_range_with_subarray(double, double, short, short, short, short);

/* function to check if chip coordinates are on the ccd */
extern L1_STATUS_T verify_chip_range_phas(double, double, short, short, short, short);

#endif   /* last line of header file- closes #ifndef EVENT_DEFS_H */  
