// 
//  Copyright (C) 1999-2002,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef MTL_PARSER_H
#define MTL_PARSER_H

#ifndef DS_ALLDATATYPES_H
#include "dsAllDataTypes.hh"
#endif

#ifndef DS_ROW_H
#include "dsRow.hh"
#endif

#include <vector>
#include <algorithm>

#define mtlPTTEXTLENGTH 1024
#define MTLTRUE         1
#define MTLFALSE        0
#define MTLPARSERROWNUM "internal_parser_rownum"

extern char *mtl_limits_p;
extern char *mtl_limits_end_p;
extern dsRow mtl_parse_data;
extern short mtl_syntax_error;

extern void mtlLimitYYerror( const char*);


enum mtlOpType {OPLESS, OPLESSEQUAL, OPGREAT, OPGREATEQUAL, OPEQUAL, OPNOTEQUAL, OPNOT, OPAT, OPAND, OPOR, OPPLUS, OPMULT, OPMINUS, OPDIV, OPMODULO, OPPOWER, OPPARENTHESIS, OPASSIGNMENT, OPBITEQUAL, OPBITASSIGNMENT, OPIFELSE, OPIF, OPCAST, OPARRAY, OPSUBSTR};

enum mtlParseType {mtlBool = 1, mtlUShort = 2, mtlShort = 3, mtlUChar = 4, mtlULong = 5, mtlInt = 6, mtlLong = 7, mtlFloat = 8, mtlDouble = 9, mtlText = 10, mtlBit = 11, mtlBitPair = 12, mtlBitList = 13, mtlUnknown = 14};

enum mtlMemoryType {mtlMemIndex, mtlMemBit, mtlMemExpr};

typedef struct parse_bit {
  long bit_index;
  unsigned short bit_val;
} mtlParseBit;

typedef struct parse_bit_node {
  parse_bit_node *next_p;
  mtlParseBit data;
} mtlParseBitNode;

typedef struct parse_index_node {
  long index;
  parse_index_node *next_p;
} mtlParseIndexNode;

typedef struct parse_declaration {
  mtlParseType type_e;
  long size;
} mtlDeclaration;

extern char *which_cols_p;
extern char *which_cols_end_p;

extern short is_expression_true(dsAllDataTypes expr_val);

extern dsDataTypes parse_to_all_type(mtlParseType in_type_e);

extern mtlParseType all_to_parse_type(dsDataTypes in_type_e);

extern dsErrBool limits_parse(char *string_2_parse_p);

extern dsDimensions convert_to_dimensions(mtlParseIndexNode *new_index_p);

extern short check_conflict(mtlParseType arg_one_type,
			    mtlParseType arg_three_type,
			    dsAllDataTypes &arg_three,
			    mtlOpType op_type);

extern dsErrBool evaluate_expression(char *expr_2_evaluate_p);

extern dsErrBool mtl_check_dependency(char *expr_2_check_p);

extern dsErrBool mtl_check_conflicts(char *expr_2_check_p);

extern dsErrBool mtl_check_dimensions(char *expr_2_check_p);

extern dsAllDataTypes mtl_evaluate(char *expr_2_check_p);

extern void mtl_free_memory(mtlMemoryType type_e,
			    void *pointer_p);

extern void mtl_clean(void);

typedef map<string, long, less<string> > mtlRegistry;

class mtlExpression
{
public:
  
  mtlExpression(void){max_dims = dsDimensions();};

  virtual ~mtlExpression(void){};

  virtual mtlExpression *Clone(void) = 0;

  virtual dsAllDataTypes Evaluate(void) = 0;

  virtual dsErrBool CheckDataDependency(void) = 0;

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e) = 0;

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions) = 0;

protected:
  virtual void HelpClean(void) = 0;

  dsDimensions max_dims;
};


typedef struct parse_expr_node {
  parse_expr_node *next_p;
  mtlExpression *expr_p;
} mtlParseExprNode;


class mtlValue : public mtlExpression
{
public:
  
  mtlValue(dsAllDataTypes new_value) {value = new_value;};

  ~mtlValue(void) {};

  mtlExpression *Clone(void) {return new mtlValue(*this);};

  virtual dsAllDataTypes Evaluate(void) {return value;};

  virtual dsErrBool CheckDataDependency(void) {return dsErrTrue;};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e)
    {
      data_type_e = all_to_parse_type(value.GetType());
      return dsErrTrue;};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions) {
	dimensions = value.GetDimensions();
	max_dims = dimensions; return dsErrTrue;};

protected:
  void HelpClean(void) {};

  dsAllDataTypes value;
};



class mtlRand : public mtlExpression
{
public:

  mtlRand(long seed) {if(seed_set == 0) {srand(seed); seed_set = 1;}};

  mtlRand(void) {if(seed_set == 0) {srand(1); seed_set = 1;}};
  
  ~mtlRand(void) {};

  mtlExpression *Clone(void) {return new mtlRand(*this);};

  virtual dsAllDataTypes Evaluate(void) { 
    return dsAllDataTypes((double)((rand() * 1.0)/RAND_MAX));};

  virtual dsErrBool CheckDataDependency(void) {return dsErrTrue;};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){
    data_type_e = mtlDouble; return dsErrTrue;};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions) {
    dimensions = dsDimensions(); return dsErrTrue;};

protected:
  void HelpClean(void) {};

  static short seed_set;
};



class mtlArrayValue : public mtlExpression
{
public:
  
  mtlArrayValue(void){};

  mtlArrayValue(mtlParseExprNode *expressions_p);

  ~mtlArrayValue(void) {HelpClean();};

  mtlArrayValue(const mtlArrayValue &copy) {HelpCopy(copy);};

  mtlArrayValue &operator = (const mtlArrayValue &right) {if(&right != this) {
    HelpClean(); HelpCopy(right);} return *this;};

  mtlExpression *Clone(void) {return new mtlArrayValue(*this);};

  virtual dsAllDataTypes Evaluate(void) ;

  virtual dsErrBool CheckDataDependency(void);

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e);

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions);

protected:
  void HelpClean(void);
  void HelpCopy(const mtlArrayValue &other);

  vector<mtlExpression *> expressions;
  dsAllDataTypes value;
  mtlParseType local_data_type_e;
  long cell_length;
};



class mtlExprList : public mtlExpression
{
public:
  
  mtlExprList(void){};

  mtlExprList(mtlParseExprNode *expressions_p);

  ~mtlExprList(void) {HelpClean();};

  mtlExprList(const mtlExprList &copy) {HelpCopy(copy);};

  mtlExprList &operator = (const mtlExprList &right) {if(&right != this) {
    HelpClean(); HelpCopy(right);} return *this;};

  mtlExpression *Clone(void) {return new mtlExprList(*this);};

  virtual dsAllDataTypes Evaluate(void) ;

  virtual dsErrBool CheckDataDependency(void);

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e);

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions);

protected:
  void HelpClean(void);
  void HelpCopy(const mtlExprList &other);

  vector<mtlExpression *> expressions;
};



class mtlIdentifier : public mtlExpression
{
public:
  mtlIdentifier(char *new_name_a) {strcpy(name_a, new_name_a);};

  ~mtlIdentifier(void) {};

  mtlExpression *Clone(void) {return new mtlIdentifier(*this);};

  virtual dsAllDataTypes Evaluate(void);

  virtual dsErrBool CheckDataDependency(void);

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e);

  string GetName(void){return string(name_a);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions);

protected:
  void HelpClean(void) {};

  char name_a[mtlPTTEXTLENGTH];
};


class mtlFunction : public mtlExpression
{
public:

  mtlFunction(void) {};
  
  mtlFunction(const mtlFunction &copy) : mtlExpression(copy) {HelpFuncCopy(copy);};

  mtlFunction &operator= (const mtlFunction &right){
    if(&right != this) 
      { mtlExpression::operator=(right); HelpFuncCopy(right);}
    return *this;};
  
  ~mtlFunction(void) {HelpClean();};

  // virtual dsErrBool CheckDimensions(dsDimensions &dimensions);

  void AddArgument(mtlExpression *expression_p)
    {
      arguments.push_back(expression_p->Clone());
      num_arguments++;
    };

protected:
  void HelpClean(void);
  void HelpFuncCopy(const mtlFunction &other);

  vector<mtlExpression *> arguments;
  long num_arguments;
  vector<mtlParseType> argument_types;

};



class mtlFunc1DblArg : public mtlFunction
{
public:
  
  mtlFunc1DblArg(double (*func_p)(double)) {f_p = func_p;num_arguments = 1;
  					    argument_types.push_back(mtlDouble);};

  mtlFunc1DblArg(const mtlFunc1DblArg &copy) : mtlFunction(copy){HelpF1DCopy(copy);};

  mtlFunc1DblArg &operator= (const mtlFunc1DblArg &right) {
    if(&right != this)
      { mtlFunction::operator=(right); HelpF1DCopy(right);}
    return *this;};

  ~mtlFunc1DblArg(void){HelpClean();};

  mtlExpression *Clone(void) {return new mtlFunc1DblArg(*this);};

  virtual dsErrBool CheckDataDependency(void);
    
  virtual dsAllDataTypes Evaluate(void);

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e);      

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions) {
    dsErrBool return_val_e = arguments[0]->CheckDimensions(dimensions);
    max_dims = dimensions; return return_val_e;};

protected:
  void HelpClean(void) {f_p = NULL;};
  void HelpF1DCopy(const mtlFunc1DblArg &other) {f_p = other.f_p;};

  double (*f_p)(double);
};


class mtlOperand : public mtlExpression
{
public:

  mtlOperand(void) {};

  mtlOperand(const mtlOperand &copy) : mtlExpression(copy){HelpCopy(copy);};

  mtlOperand &operator= (const mtlOperand &right){
    if(&right != this)
      {mtlExpression::operator=(right); HelpCopy(right);}
    return *this;}

  ~mtlOperand(void) {HelpClean();};

  void AddExpression(mtlExpression *expression_p)
    {
      expressions.push_back(expression_p->Clone());
    };

protected:
  void HelpClean(void);
  void HelpCopy(const mtlOperand &other);

  mtlOpType operation;
  vector<mtlExpression *> expressions;

};



class mtlUnaryOperand : public mtlOperand
{
public:

  mtlUnaryOperand(void) {};

  ~mtlUnaryOperand(void) {};

  virtual dsErrBool CheckDataDependency(void)
    {
      if(expressions.size() > 0)
	{
	  return expressions[0]->CheckDataDependency();
	}
      else
	{
	  return dsErrFalse;
	}
    };

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions) {
    dsErrBool return_val_e = expressions[0]->CheckDimensions(dimensions);
    max_dims = dimensions; return return_val_e;};

};



class mtlIndexValue : public mtlUnaryOperand
{
public:
  
  mtlIndexValue(void) {index = dsDimensions(); oneD_index = 0; index_set = 0;};

  ~mtlIndexValue(void){};
  
  mtlExpression *Clone(void) {return new mtlIndexValue(*this);};

  virtual dsAllDataTypes Evaluate(void);

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e)
    {return expressions[0]->CheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions);

  void SetIndex(dsDimensions new_index);

protected:

  dsDimensions index;
  short index_set;
  unsigned long oneD_index;
};



class mtlCast : public mtlUnaryOperand
{
public:

  mtlCast(void) {cast_type_e = mtlBool;};
  mtlCast(mtlParseType new_type_e) {cast_type_e = new_type_e;};

  ~mtlCast(void) {};

  mtlExpression *Clone(void) {return new mtlCast(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e);

  virtual dsAllDataTypes Evaluate(void); 

protected:

  mtlParseType cast_type_e;
};

      
  

class mtlNot : public mtlUnaryOperand
{
public:

  mtlNot(void){operation = OPNOT;};

  ~mtlNot(void){};

  mtlExpression *Clone(void) {return new mtlNot(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e)
    {
      mtlParseType expr_type_e;
      data_type_e = mtlBool;

      return expressions[0]->CheckTypeConflict(expr_type_e);
    };

  virtual dsAllDataTypes Evaluate(void) {
    return !(expressions[0]->Evaluate());
  }
};



class mtlParenthesis : public mtlUnaryOperand
{
public:

  mtlParenthesis(void){operation = OPPARENTHESIS;};

  ~mtlParenthesis(void){};

  mtlExpression *Clone(void) {return new mtlParenthesis(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e)
    {
      return expressions[0]->CheckTypeConflict(data_type_e);
    };

  virtual dsAllDataTypes Evaluate(void) {
    return expressions[0]->Evaluate();
  };
};



class mtlBinaryOperand : public mtlOperand
{
public:

  mtlBinaryOperand(void) {};

  ~mtlBinaryOperand(void) {};

  virtual dsErrBool CheckDataDependency(void);

  //  virtual dsErrBool CheckDimensions(dsDimensions &dimensions);

protected:
  dsErrBool ArithCheckTypeConflict(mtlParseType &data_type_e);
  dsErrBool BoolCheckTypeConflict(mtlParseType &data_type_e);
  dsErrBool ArithCheckDimensions(dsDimensions &dimensions);
  dsErrBool BoolCheckDimensions(dsDimensions &dimensions);

};


class mtlPlus : public mtlBinaryOperand
{
public:
  mtlPlus(void) {operation = OPPLUS;};

  ~mtlPlus(void) {};

  mtlExpression *Clone(void) {return new mtlPlus(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e) {return ArithCheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return ArithCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void)
    { return ((expressions[0]->Evaluate()).operator +(expressions[1]->Evaluate()));};
};  



class mtlMinus : public mtlBinaryOperand
{
public:
  mtlMinus(void) {operation = OPMINUS;};

  ~mtlMinus(void) {};

  mtlExpression *Clone(void) {return new mtlMinus(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){return ArithCheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return ArithCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void)
    { return ((expressions[0]->Evaluate()).operator -(expressions[1]->Evaluate()));};
};  



class mtlMultiply : public mtlBinaryOperand
{
public:
  mtlMultiply(void) {operation = OPMULT;};

  ~mtlMultiply(void) {};

  mtlExpression *Clone(void) {return new mtlMultiply(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){return ArithCheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return ArithCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void)
    { return ((expressions[0]->Evaluate()).operator *(expressions[1]->Evaluate()));};
};



class mtlDivide : public mtlBinaryOperand
{
public:
  mtlDivide(void) {operation = OPDIV;};

  ~mtlDivide(void) {};

  mtlExpression *Clone(void) {return new mtlDivide(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){return ArithCheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return ArithCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void)
    { return ((expressions[0]->Evaluate()).operator /(expressions[1]->Evaluate()));};
}; 



class mtlModulo : public mtlBinaryOperand
{
public:
  mtlModulo(void) {operation = OPMODULO;};

  ~mtlModulo(void) {};

  mtlExpression *Clone(void) {return new mtlModulo(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){return ArithCheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return ArithCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void)
    { return ((expressions[0]->Evaluate()).operator %(expressions[1]->Evaluate()));};
};  



class mtlPower : public mtlBinaryOperand
{
public:
  mtlPower(void) {operation = OPPOWER;};

  ~mtlPower(void) {};

  mtlExpression *Clone(void) {return new mtlPower(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e);

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return ArithCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void);
};


class mtlAssignment : public mtlBinaryOperand
{
public:
  mtlAssignment(void) {operation = OPASSIGNMENT; identifier_p = NULL;};

  mtlAssignment(const mtlAssignment &copy) : mtlBinaryOperand(copy) {identifier_p = NULL;
    HelpAssignCopy(copy);};

  mtlAssignment &operator= (const mtlAssignment &right) {
    if(&right != this) 
      {mtlOperand::operator=(right); HelpAssignCopy(right);}
    return *this;};

  ~mtlAssignment(void) {HelpClean();};

  mtlExpression *Clone(void) {return new mtlAssignment(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e);

  virtual dsErrBool CheckDataDependency(void);

  virtual dsAllDataTypes Evaluate(void);

  void SetIdentifier(mtlIdentifier *new_identifier_p);

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions);

protected:
  void HelpAssignCopy(const mtlAssignment &other);
  void HelpClean(void);

  mtlIdentifier *identifier_p;
};


class mtlIndexAssignment : public mtlBinaryOperand
{
public:
  mtlIndexAssignment(void) {operation = OPASSIGNMENT; identifier_p = NULL; 
  			    index = dsDimensions(); index_set = 0; oneD_index = 0; };

  mtlIndexAssignment(const mtlIndexAssignment &copy) : mtlBinaryOperand(copy) {identifier_p = NULL;
    HelpIAssignCopy(copy);};

  mtlIndexAssignment &operator= (const mtlIndexAssignment &right) {
    if(&right != this) 
      {mtlOperand::operator=(right); HelpIAssignCopy(right);}
    return *this;};

  ~mtlIndexAssignment(void) {HelpClean();};

  mtlExpression *Clone(void) {return new mtlIndexAssignment(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e);

  virtual dsErrBool CheckDataDependency(void);

  virtual dsAllDataTypes Evaluate(void);

  void SetIdentifier(mtlIdentifier *new_identifier_p);

  void SetIndex(dsDimensions new_index);

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions);

protected:
  void HelpIAssignCopy(const mtlIndexAssignment &other);
  void HelpClean(void);

  dsDimensions index;
  short index_set;
  unsigned long oneD_index;
  mtlIdentifier *identifier_p;
};



class mtlBitAssignment : public mtlBinaryOperand
{
public:

  mtlBitAssignment(void) {operation = OPBITASSIGNMENT; identifier_p = NULL; bit_rule_p = NULL;};

  mtlBitAssignment(const mtlBitAssignment &copy) : mtlBinaryOperand(copy) {identifier_p = NULL;
    bit_rule_p = NULL; HelpBAssignCopy(copy);};

  mtlBitAssignment &operator= (const mtlBitAssignment &right) {
    if(&right != this) 
      {mtlOperand::operator=(right); HelpBAssignCopy(right);}
    return *this;};

  ~mtlBitAssignment(void) {HelpClean();};

  mtlExpression *Clone(void) {return new mtlBitAssignment(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e);

  virtual dsErrBool CheckDataDependency(void);

  virtual dsAllDataTypes Evaluate(void);

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions);

  void SetIdentifier(mtlIdentifier *new_identifier_p);

  void SetBitRule(mtlParseBitNode *new_bit_rule_p);

  long GetBitRuleLength(void);
protected:
  void HelpBAssignCopy(const mtlBitAssignment &other);
  void HelpClean(void);
  void FreeBitRule(void);
  void CopyBitRule(mtlParseBitNode *rule_2_copy_p);
  void FreeIdentifier(void);

  mtlIdentifier *identifier_p;
  mtlParseBitNode *bit_rule_p;
};


class mtlLess : public mtlBinaryOperand
{
public:

  mtlLess(void) {operation = OPLESS;};

  ~mtlLess(void) {};

  mtlExpression *Clone(void) {return new mtlLess(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e) {return BoolCheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return BoolCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void)
    {
      return ((expressions[0]->Evaluate()).operator <(expressions[1]->Evaluate()));
    };
};



class mtlLessEqual : public mtlBinaryOperand
{
public:

  mtlLessEqual(void) {operation = OPLESSEQUAL;};

  ~mtlLessEqual(void) {};

  mtlExpression *Clone(void) {return new mtlLessEqual(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){return BoolCheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return BoolCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void)
    {
      return ((expressions[0]->Evaluate()).operator <=(expressions[1]->Evaluate()));
    };
};



class mtlGreater : public mtlBinaryOperand
{
public:

  mtlGreater(void) {operation = OPGREAT;};

  ~mtlGreater(void) {};

  mtlExpression *Clone(void) {return new mtlGreater(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){return BoolCheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return BoolCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void)
    {
      return ((expressions[0]->Evaluate()).operator >(expressions[1]->Evaluate()));
    };
};



class mtlGreaterEqual : public mtlBinaryOperand
{
public:

  mtlGreaterEqual(void) {operation = OPGREATEQUAL;};

  ~mtlGreaterEqual(void) {};

  mtlExpression *Clone(void) {return new mtlGreaterEqual(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){return BoolCheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return BoolCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void)
    {
      return ((expressions[0]->Evaluate()).operator >=(expressions[1]->Evaluate()));
    };
};



class mtlEqual : public mtlBinaryOperand
{
public:

  mtlEqual(void) {operation = OPEQUAL;};

  ~mtlEqual(void) {};

  mtlExpression *Clone(void) {return new mtlEqual(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){return BoolCheckTypeConflict(data_type_e);};

  virtual dsAllDataTypes Evaluate(void);

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return BoolCheckDimensions(dimensions);};
};


class mtlSubstr : public mtlBinaryOperand
{
public:

  mtlSubstr(void) {operation = OPSUBSTR;};

  ~mtlSubstr(void) {};

  mtlExpression *Clone(void) {return new mtlSubstr(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){return BoolCheckTypeConflict(data_type_e);};

  virtual dsAllDataTypes Evaluate(void);

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return BoolCheckDimensions(dimensions);};
};




class mtlBitEqual : public mtlBinaryOperand
{
public:

  mtlBitEqual(void) {operation = OPBITEQUAL; bit_rule_p = NULL;};
  
  mtlBitEqual(const mtlBitEqual &copy): mtlBinaryOperand(copy) {bit_rule_p = NULL; HelpBEqualCopy(copy);};

  ~mtlBitEqual(void){HelpClean();};

  mtlBitEqual &operator= (const mtlBitEqual &right) {
    if(&right != this)
      {mtlOperand::operator=(right); HelpBEqualCopy(right);}
    return *this;};

  mtlExpression *Clone(void) {return new mtlBitEqual(*this);};

  virtual dsErrBool CheckDataDependency(void);

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e);

  virtual dsAllDataTypes Evaluate(void);

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions);

  void SetBitRule(mtlParseBitNode *new_bit_rule_p);
protected:
  void HelpClean(void);
  void HelpBEqualCopy(const mtlBitEqual &other);
  void FreeBitRule(void);
  void CopyBitRule(mtlParseBitNode *rule_2_copy_p);

  mtlParseBitNode *bit_rule_p;
};



class mtlNotEqual : public mtlBinaryOperand
{
public:

  mtlNotEqual(void) {operation = OPNOTEQUAL;};

  ~mtlNotEqual(void) {};

  mtlExpression *Clone(void) {return new mtlNotEqual(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){return BoolCheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return BoolCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void)
    {
      return ((expressions[0]->Evaluate()).operator !=(expressions[1]->Evaluate()));
    };
};


class mtlAnd : public mtlBinaryOperand
{
public:

  mtlAnd(void) {operation = OPAND;};

  ~mtlAnd(void) {};

  mtlExpression *Clone(void) {return new mtlAnd(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){return BoolCheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return BoolCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void)
    {
      return (is_expression_true(expressions[0]->Evaluate()) &&
	      is_expression_true(expressions[1]->Evaluate()));
    };
};



class mtlOr : public mtlBinaryOperand
{
public:

  mtlOr(void) {operation = OPOR;};

  ~mtlOr(void) {};

  mtlExpression *Clone(void) {return new mtlOr(*this);};

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e){return BoolCheckTypeConflict(data_type_e);};

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions){
    return BoolCheckDimensions(dimensions);};

  virtual dsAllDataTypes Evaluate(void)
    {
      return (is_expression_true(expressions[0]->Evaluate()) || 
	      is_expression_true(expressions[1]->Evaluate()));
    };
};



class mtlIfThenElse : public mtlOperand
{
public:

  mtlIfThenElse(void) {operation = OPIFELSE;};

  ~mtlIfThenElse(void) {};

  mtlExpression *Clone(void) {return new mtlIfThenElse(*this);};

  virtual dsErrBool CheckDataDependency(void);

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e);

  virtual dsAllDataTypes Evaluate(void);

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions);
};



class mtlIfThen : public mtlOperand
{
public:

  mtlIfThen(void) {operation = OPIF;};

  ~mtlIfThen(void) {};

  mtlExpression *Clone(void) {return new mtlIfThen(*this);};

  virtual dsErrBool CheckDataDependency(void);

  virtual dsErrBool CheckTypeConflict(mtlParseType &data_type_e);

  virtual dsAllDataTypes Evaluate(void);

  virtual dsErrBool CheckDimensions(dsDimensions &dimensions);
};



class mtlStatement 
{
public:

  /* constructors, destructors, etc. */
  mtlStatement(void){expression_p = NULL; statement = "";
  	dependency_checked = dimensions_checked = conflicts_checked = dsErrFalse;
	dependency_passed = dimensions_passed = conflicts_passed = dsErrFalse;};

  mtlStatement(string new_statement) {statement = new_statement;expression_p = NULL;
  	dependency_checked = dimensions_checked = conflicts_checked = dsErrFalse;
  	dependency_passed = dimensions_passed = conflicts_passed = dsErrFalse;};

  mtlStatement(const mtlStatement &copy) {HelpCopy(copy);};

  mtlStatement &operator= (const mtlStatement &right){
    if(&right != this)
      {	HelpCopy(right);}
    return *this;};

  ~mtlStatement(void) {identifiers.clear(); declarations.clear(); temp_id.clear(); bin_sizes.clear();delete expression_p;};



  dsAllDataTypes Evaluate(void);

  dsErrBool CheckDataDependency(void);

  dsErrBool CheckTypeConflict(void);

  dsErrBool CheckDimensions(void);

  void SetExpression(mtlExpression *new_expression_p);

  void AddIdentifier(string identifier,
		     long binsize);

  void AddTempIdentifier(string identifier);

  void AddDeclaration(string identifier);
  
  vector<string> *GetIdentifiers(void) {return &identifiers;};

  vector<string> *GetTempIdentifiers(void) {return &temp_id;};

  vector<string> *GetDeclarations(void) {return &declarations;};

  vector<long> *GetBinSizes(void) {return &bin_sizes;};

protected:
  void HelpClean(void);
  void HelpCopy(const mtlStatement &other);

  vector<string> identifiers;
  vector<string> temp_id;
  vector<string> declarations;
  vector<long> bin_sizes;
  mtlExpression *expression_p;
  string statement;
  
  dsErrBool dependency_checked, dimensions_checked, conflicts_checked;
  dsErrBool dependency_passed, dimensions_passed, conflicts_passed;
};



typedef struct parse_id_bin {
  long bin;
  mtlIdentifier *id_p;
} mtlParseIdBin;



extern mtlRegistry mtl_statement_map;
extern vector<mtlStatement *> mtl_statements;
extern mtlStatement *mtl_current_statement_p;

#endif








