#!/usr/bin/perl
# 
#  Copyright (C) 2004-2005,2007-2008,2016-2017,2021,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '002';
my $semantic = 'dmcopy';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmcopy"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
5,
6,
7,
8,
9,
10,
11,
12,
13,
14,
15,
16,
17,
18,
19,
20,
21,
22,
23,
24,
25,
26,
27,
28,
29,
30,
31,
32,
33,
);
my %test_blocks = 
( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
 "1" => { "image" => [ 1 ]},
 "2" => { "table" => [ 1, 2 ]},
 "3" => { "list" => [ ] },
 "4" => { "table" => [ 1, 2, 3 ]},
 "5" => { "image" => [ 1 ],
	  "table" => [ 2 ]},
 "6" => { "table" => [ 1, 2 ]},
 "7" => { "table" => [ 1, 2 ]},
 "8" => { "image" => [ 1 ],
	  "table" => [ 2, 3, 4, 5, 6 ]},
 "9" => { "table" => [ 1, 2, 3, 4, 5, 6, 7 ]},
 "10" => { "table" => [ 1, 2, 3, 4, 5, 6, 7 ]},
 "11" => { "table" => [ 1, 2, 3 ] },
 "12" => { "table" => [ 1, 2, 3, 4, 5, 6, 7 ] },
 "13" => { "table" => [ 2 ] },
 "14" => { "table" => [ 1, 2, 3, 4, 5, 6, 7 ]},
 "15" => { "table" => [ 1, 2, 3, 4, 5, 6, 7 ]},
 "16" => { "table" => [ 2 ],
           "image" => [ 1 ] },
 "17" => { "list" => [ ] },
 "18" => { "image" => [ 1 ]},
 "19" => { "table" => [ 1, 2, 3, 4 ] },
 "20" => { "table" => [1] },
 "21" => { "list" => [ ] },
 "22" => { "list" => [ ] },
 "23" => { "list" => [ ] },
 "24" => { "list" => [ ] },
 "25" => { "list" => [ ] },
 "26" => { "list" => [ ] },
 "27" => { "list" => [ ] },
 "28" => { "list" => [ ] },
 "29" => { "list" => [ ] },
 "30" => { "list" => [ ] },
 "31" => { "list" => [ ] },
 "32" => { "list" => [ ] },
 "33" => { "list" => [ ] },
);
		     
# Find Tool Executable in PATH
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;

my %exten = ("FITS" => "fits",
             "TEXT" => "dtf" );
my %copyOpts = ("TEXT" => "text/dtf");
my %links = ("8" => "indt002m007",
             "9" => "indt002m007",
             "10" => "indt002m007");
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#########################################################################

# Loop over DM kernels to test under.
my $kref = %infra{'kernels'};
my @kernels = @$kref;
my $kernelName;

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  # Loop over all tests
  foreach my $testid (@testlist)
  {
    # Init per-test variables.
    my $mismatch = 0;
    my $time_user;
    my $time_sys;
    $err_msg = "";
    my $skipDelete = 0;
    my $linkname = "";

    # change extension based on kernel
    $in_ext = $exten{$kernelName};

    # delete old outputs
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
    system("$RM -f $testout/${basename}*");

    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    my $infile = "$testin/in${basename}.$in_ext";
    my $filterfile = $testin;

    # if testing any kernel other than FITS, the FITS input files need to
    # be copied into the present kernel's format
    if ( $kernelName ne "FITS" )
    {
      my $sourceFile = "$testin/in${basename}.$source_ext";
      my $copyFile   = "$testout/in${basename}.$in_ext";
      my $options    = "opt=all";
      my $kernelOpts = $copyOpts{"$kernelName"};


      if (!  -e "$infile" )
      {
        my $linkbase = "";
        my $teststring = "$DMCOPY $sourceFile $copyFile\"[opt kernel=$kernelOpts]\" $options";

        # does a previous copy file exist?
        if ($testid == 8 || $testid == 9 || $testid == 10 )
        {
          $linkbase = $links{"$testid"};
          $linkname = "$testout/${linkbase}.$in_ext";
        }

        # if not, make one
        if ( ! -e $copyFile )
        {
          if ( $linkname ne "" )
          {
            if ( -e $linkname )
            {
              # create link
              $teststring = "ln -s $linkname $copyFile";
            }
          }

          # copy fits file to text/dtf file
          ($rc, $time_user, $time_sys) = system($teststring);
        }

        $infile = $copyFile;

         # Tests 7, 8, 9 and 10 use the same input file.
         # Don't delete generated input file yet
         if ( $testid == 7 )
         {
           $skipDelete = 1;
         }
      }
      else
      {
        # preserve inputs
        $skipDelete = 1;
      }
    }

    # PROGRAMMER - put output filename here
    my $outfile = "$testout/${basename}.$out_ext";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/${basename}.$out_ext";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/${basename}.diff";

    # --------------------------------------------------------------------------------
    # set test command
    my $test_string = "";
    if ( $testid == 1 )
    {
      $test_string = "$DMCOPY infile=$infile outfile=$outfile";
    }
    elsif ( $testid == 2 )
    {
      $test_string = "$DMCOPY infile=$infile outfile=$outfile" ;
    }
    elsif ( $testid == 3 )
    {
      $test_string = "$DMCOPY infile=$infile outfile=$outfile";
    }
    elsif ( $testid == 4 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[stdevt\]\" outfile=$outfile";
    }
    elsif ( $testid == 5 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[stdevt][bin x=32,y=32\]\" outfile=$outfile";
    }
    elsif ( $testid == 6 )
    {
      $test_string = "$DMCOPY infile=\"$infile" . '[(x,y)=circle(4183,5870,200)*!annulus(4222,5970,100,1000)]' . "\" outfile=$outfile";
    }
    elsif ( $testid == 7 )
    {
      $test_string = "$DMCOPY infile=\"$infile" . '[ccd_id=5,time=58153664.094645:58160019.294651]' . "\" outfile=$outfile";
    }
    elsif ( $testid == 8 )
    {
      $test_string = "$DMCOPY infile=\"$infile" . '[bin x=3900:4300:2,y=3900:4300:2]' . "\" outfile=$outfile option=\"\"";
    }
    elsif ( $testid == 9 )
    {
      $test_string = "$DMCOPY infile=\"$infile" . '[filter sky=region(' . "$testin/in${basename}.reg" . ')]' . "\" outfile=$outfile option=\"\"";
    }
    elsif ( $testid == 10 )
    {
      $test_string = "$DMCOPY infile=\"$infile" . '[exclude sky=circle(4057.5,4161.5,26)]' . "\" outfile=$outfile";
    }
    elsif ( $testid == 11 )
    {
      $test_string = "$DMCOPY infile=\"$infile" . '[status=0,grade=0,2,3,4,6]' . "\" outfile=$outfile";
    }
    elsif ( $testid == 12 )
    {
      $test_string = "$DMCOPY infile=\"$infile\" outfile=$outfile";
    }
    elsif ( $testid == 13 )
    {
     # check for Bug 9650
      $test_string = "$DMCOPY infile=\"$infile\" outfile=$outfile";
    }
    elsif ( $testid == 14 )
    {
     # test 11 from dmcopy.t
      $test_string = "$DMCOPY infile=$infile'[exclude pha=1:50][opt -ssupdate][newname]' outfile=$outfile";
    }
    elsif ( $testid == 15 )
    {
     # test 12 from dmcopy.t
      $test_string = "$DMCOPY infile=$infile'[exclude pha=1:50][newname]' outfile=$outfile";
    }
    elsif ( $testid == 16 )
    {
     # test 14 from dmcopy.t
      $test_string = "$DMCOPY infile=$infile'[stdevt][pha=100:300][bin x=32, y=32]' outfile=$outfile";
    }
    elsif ( $testid == 17 )
    {
     # test 15 from dmcopy.t
      $filterfile = "$testin/in${basename}.reg";
      $test_string = "$DMCOPY infile=$infile\'[(x,y)=region($filterfile)]\' outfile=$outfile";
    }
    elsif ( $testid == 18 )
    {
     # test 18 from dmcopy.t - check for Bug 8130 image rebinning
      $test_string = "$DMCOPY infile=$infile'[bin x=3826.5:4966.5:4.0,y=3516.5:4656.5:4.0]' outfile=$outfile";
    }
    elsif ( $testid == 19 )
    {
     # test 19 from dmcopy.t - check for Bug 8289 exclude + file subspace filter
      $filterfile = "${testin}/in${basename}_filter.fits[ccd_id=3]";
      $test_string = "$DMCOPY infile=$infile\'[exclude \@${filterfile}]\' outfile=$outfile";
    }
    elsif ( $testid == 20 )
    {
     # test 20 from dmcopy.t - check for Bug 9698 - losing filer info
     # if filter file and text used
      $filterfile = "${testin}/in${basename}_filter.txt,time=782478700,78250000";
      $test_string = "$DMCOPY infile=$infile\'[\@${filterfile}]\' outfile=$outfile";
    }
    elsif ( $testid == 21 )
    {
      # Test for Bug13670 - filtering an input file with no physical transform defined.
      # The resulting file should have the correct transform for the filter, as opposed
      # to a unit transform.
      $filterfile = "$testin/in${basename}.reg";
      $test_string = "$DMCOPY infile=$infile\'[(x,y)=rect(50,100,100,150)]\' outfile=$outfile";
    }
    elsif ( $testid == 22 )
    {
      # Test for Bug14267 - wherein the region filter applied to the file doesn't have any points in 
      # it because of non-overlapping shapes. The end result should be a file with no data in it (0x0 image or 0 row table).
      $test_string = "$DMCOPY infile=$infile\'[(#1,#2)=rect(10,10,20,20)&rect(30,30,40,40)]\' outfile=$outfile";
    }
    elsif ( $testid == 23 )
    {
      # SL-35 Test Plan - 1.2.1 straight dmcopy of 1D image
      $test_string = "$DMCOPY infile=$infile\'[1D_I2_BASIC]\' outfile=$outfile";
    }
    elsif ( $testid == 24 )
    {
      # SL-35 Test Plan - 1.2.2 straight dmcopy of 2D image
      $test_string = "$DMCOPY infile=$infile\'[2D_I2_BASIC]\' outfile=$outfile";
    }
    elsif ( $testid == 25 )
    {
      # SL-35 Test Plan - 1.2.3 straight dmcopy of 3D image
      $test_string = "$DMCOPY infile=$infile\'[3D_I2_BASIC]\' outfile=$outfile";
    }
    elsif ( $testid == 26)
    {
      # SL-35 Test Plan - 1.2.4 straight dmcopy of 4D image
      $test_string = "$DMCOPY infile=$infile\'[4D_I2_BASIC]\' outfile=$outfile";
    }
    elsif ( $testid == 27 )
    {
      # SL-35 Test Plan - 1.2.5 straight dmcopy of scaled 1D image
      $test_string = "$DMCOPY infile=$infile\'[1D_I2_SCALED]\' outfile=$outfile";
    }
    elsif ( $testid == 28 )
    {
      # SL-35 Test Plan - 1.2.6 straight dmcopy of scaled 2D image
      $test_string = "$DMCOPY infile=$infile\'[2D_I2_SCALED]\' outfile=$outfile";
    }
    elsif ( $testid == 29 )
    {
      # SL-35 Test Plan - 1.2.7 straight dmcopy of scaled 3D image
      $test_string = "$DMCOPY infile=$infile\'[3D_I2_SCALED]\' outfile=$outfile";
    }
    elsif ( $testid == 30)
    {
      # SL-35 Test Plan - 1.2.8 straight dmcopy of scaled 4D image
      $test_string = "$DMCOPY infile=$infile\'[4D_I2_SCALED]\' outfile=$outfile";
    }
    elsif ( $testid == 31 )
    {
      # SL-35 Test Plan - 5.2.1 straight dmcopy with externally set type [opt type=i2|r4|etc]
      $test_string = "$DMCOPY infile=$infile\'[2D_I2_SCALED][opt type=i4]\' outfile=$outfile";
    }
    elsif ( $testid == 32 )
    {
      # SL-35 Test Plan - 5.2.2 straight dmcopy with externally set type [opt type=i2|r4|etc]
      $test_string = "$DMCOPY infile=$infile\'[2D_I2_BASIC][opt type=r4]\' outfile=$outfile";
    }
    elsif ( $testid == 33 )
    {
      # SL-35 Test Plan - 6.1.2 float image with BLANK - straight copy
      $test_string = "$DMCOPY infile=$infile outfile=$outfile";
    }
    $test_string .= " >> $logfname 2>&1";
    
    # --------------------------------------------------------------------------------
    # run the tool
    dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
    close $logfile;
    
    ( $rc, $time_user, $time_sys ) = dt_run_test( $test_string );
    
    open $logfile, ">>$logfname";
    select($logfile); $| = 1; select($printfile);
    unless ( $rc == 0 )
    {
      $err_msg = "ERROR: $toolname failed to run\n";
    }
 
    # --------------------------------------------------------------------------------
    # validate results.
    if ( $block_types{"table"} )
    {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
        # do not compare null primary blocks for ascii kernel
        if ($block != 1 || ($kernelName ne "TEXT") )
        {
          $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
        }
      }
    }

    if ( $block_types{"image"} )
    {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmimgcalc_test( $outfile, $savfile, $block,"$testin/tolerance", $logfname, $test_diff );
      }
    }

    if ( $block_types{"list"} )
    {
      $err_msg .= dmlist_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}_savfilt", \@tolkeys, $logfname, $test_diff, "" );
    }

    if ( $block_types{"dtf"} )
    {
      $err_msg .= dtf_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}_savfilt", \@tolkeys, $logfname, $test_diff, "" );
    }

    # --------------------------------------------------------------------------------
    # report results
    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        push (@faillist, $kernelName);
        $errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
    $time_user_total += $time_user;
    $time_sys_total += $time_sys;

    # --------------------------------------------------------------------------------
    # delete any input files created just for this kernel
    if ( $kernelName ne "FITS" )
    {
      if ($skipDelete == 0 )
      {
        system("$RM -f $infile");
      }
      if ( $testid == 10 )
      {
        system("$RM -f $linkname");
      }        
    }
  }
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}

$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
