#!/usr/bin/perl
# 
#  Copyright (C) 2004-2005,2008,2015,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '003';
my $semantic = 'basicFilter';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $version = %infra{'version'};
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);


# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmlist"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
101,
102,
103,
104,
105,
106,
107,
108,
109,
    );

my %test_blocks = ( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
101 => { "text" => [ ] },
102 => { "text" => [ ] },
103 => { "text" => [ ] },
104 => { "text" => [ ] },
105 => { "text" => [ ] },
106 => { "text" => [ ] },
107 => { "text" => [ ] },
108 => { "text" => [ ] },
109 => { "text" => [ ] },
);

my %test_words = 
(
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
$version => "ASCDS_VERSION",
);
		     
# Find Tool Executable in PATH
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;

my %exten = ("FITS" => "fits",
             "TEXT" => "dtf" );
my $ext = "fits";

# This test executes a series of filter commands on columns of
# various data types.  We re-use the same command set for similar 
# data types.  Define commands here:

my @qfilters = ( # Boolean column                                              # Expected results
             "dmlist '{infile}[cols {c}][{c}=0]' counts",                      # 105 (match FALSE, single value)
             "dmlist '{infile}[cols {c}][{c}=1]' counts",                      #  11 (match TRUE, single value)
             "dmlist '{infile}[cols {c}][{c}=85]' counts",                     # Err (illegal numeric for boolean)
             "dmlist '{infile}[cols {c}][{c}=T]' counts",                      #  11 (match TRUE, single value)
             "dmlist '{infile}[cols {c}][{c}=F]' counts",                      # 105 (match FALSE, single value)
             "dmlist '{infile}[cols {c}][{c}=U]' counts",                      #  22 (match NULL, single value)
             "dmlist '{infile}[cols {c}][{c}=null]' counts",                   #  22 (match NULL, single value)
             "dmlist '{infile}[cols {c}][{c}=0,1]' counts",                    # 116 (match TRUE + FALSE, multiple values; excludes null ) 
             "dmlist '{infile}[cols {c}][{c}=T,F]' counts",                    # 116 (match TRUE + FALSE, multiple values; excludes null ) 
             "dmlist '{infile}[cols {c}][{c}=0,null]' counts",                 # 127 (match FALSE + null ) 
             "dmlist '{infile}[cols {c}][{c}=1,null]' counts",                 #  33 (match TRUE + null ) 
             "dmlist '{infile}[cols {c}][{c}=0:1]' counts",                    # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][{c}=0:]' counts",                     # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][{c}=:1]' counts",                     # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][{c}=0:1,null]' counts",               # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][{c}>0]' counts",                      # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][{c}>=0]' counts",                     # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][{c}<1]' counts",                      # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][{c}<=1]' counts",                     # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][{c}\!=0]' counts",                    # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][{c}\!=1]' counts",                    # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][{c}\!=null]' counts",                 # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][{c}\!=F]' counts",                    # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][{c}\!=T]' counts",                    # Err (bad syntax; no range support for boolean)
             "dmlist '{infile}[cols {c}][exclude {c}=0]' counts",              #  11 (match TRUE; exclude nulls)
             "dmlist '{infile}[cols {c}][exclude {c}=1]' counts",              # 105 (match FALSE; exclude nulls)
             "dmlist '{infile}[cols {c}][exclude {c}=T]' counts",              # 105 (match FALSE; exclude nulls)
             "dmlist '{infile}[cols {c}][exclude {c}=F]' counts",              #  11 (match TRUE; exclude nulls)
             "dmlist '{infile}[cols {c}][exclude {c}=null]' counts",           #   0 (empty filter)
             "dmlist '{infile}[cols {c}][exclude {c}=0:1]' counts",            # Err (bad syntax; no range support for boolean)
            );

my @sfilters = ( # String columns                                              # Expected results
             "dmlist '{infile}[cols {c}][{c}=TRUE]' counts",                   #   3 (exact match, single value)
             "dmlist '{infile}[cols {c}][{c}=of]' counts",                     #   1 (exact match, single value.. "of" not "off")
             "dmlist '{infile}[cols {c}][{c}=GRB\ 980425]' counts",            #   1 (value containing space)
             "dmlist '{infile}[cols {c}][{c}=\!Box]' counts",                  #   1 (value starts with '!' char)
             "dmlist '{infile}[cols {c}][{c}=null]' counts",                   #  19 (only null records)
             "dmlist '{infile}[cols {c}][{c}=GALAXY,STAR]' counts",            #   9 (exact match, multiple values)
             "dmlist '{infile}[cols {c}][{c}=Pie,\!Pie]' counts",              #   2 (exact match and value with '!' is second item.)
             "dmlist '{infile}[cols {c}][{c}=M31,null]' counts",               #  20 (exact value + null records)
             "dmlist '{infile}[cols {c}][{c}=a:z]' counts",                    #  70 (lc records 'a*' => 'z'; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}=on:under]' counts",               #  18 (range includes matching edges; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}=A:]' counts",                     # 105 (uc + lc; not numerical, not !<shape>; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}=:]' counts",                      # 119 (all not null)
             "dmlist '{infile}[cols {c}][{c}=:A]' counts",                     #  14 (numerical and !<shape> records; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}=a:z,null]' counts",               #  89 (lc records 'a*' => 'z' + null)
             "dmlist '{infile}[cols {c}][{c}>Polygon]' counts",                #  81 (uc > Polygon + all lc; not numeric, not !<shape>; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}>=Polygon]' counts",               #  82 (above + exact match "Polygon")
             "dmlist '{infile}[cols {c}][{c}<con]' counts",                    #  72 (all uc + lc < con + 3C273 (numeric) + !<shape>; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}<=con]' counts",                   #  72 (same as above.. no exact match for 'con')
             "dmlist '{infile}[cols {c}][{c}\!=Annulus]' counts",              # 118 (all but null and Annulus)
             "dmlist '{infile}[cols {c}][{c}\!=\!Annulus]' counts",            # 118 (all but null and !Annulus)
             "dmlist '{infile}[cols {c}][{c}\!=null]' counts",                 # 119 (all but null)
             "dmlist '{infile}[cols {c}][{c}\!=GALAXY]' counts",               # 113 (all but null and matching GALAXY records)
             "dmlist '{infile}[cols {c}][{c}\!=GALAXY:STAR]' counts",          # err (illegal syntax)
             "dmlist '{infile}[cols {c}][exclude {c}=Annulus]' counts",        # 118 (all but null and Annulus)
             "dmlist '{infile}[cols {c}][exclude {c}=\!Annulus]' counts",      # 118 (all but null and !Annulus)
             "dmlist '{infile}[cols {c}][exclude {c}=null]' counts",           #   0 (empty - no included values)
             "dmlist '{infile}[cols {c}][exclude {c}=between:inside]' counts", # 102 (all but null and 17 matching rows within range, excludes matching edges)
            );

my @ffilters = ( # Floating point columns                                           # Expected results
             "dmlist '{infile}[cols {c}][{c}=4890.33984]' counts",                  #   1 (exact match)
             "dmlist '{infile}[cols {c}][{c}=null]' counts",                        #  22 (null and nan records)
             "dmlist '{infile}[cols {c}][{c}=3622.2099,4166.91015]' counts",        #   2 (exact match, multiple values)
             "dmlist '{infile}[cols {c}][{c}=4395.93994,null]' counts",             #  23 (exact value and null records)
             "dmlist '{infile}[cols {c}][{c}=1000:2000]' counts",                   #   5 (general range; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}=4891.1098:4891.1099]' counts",         #   1 (fine range; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}=3052.94995:]' counts",                 #  92 (open ended max; incl start; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}=:]' counts",                           # 116 (excludes nulls)
             "dmlist '{infile}[cols {c}][{c}=:3052.94995]' counts",                 #  24 (open ended min; excl end; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}=1000:2000,null]' counts",              #  27 (general range + nulls)
             "dmlist '{infile}[cols {c}][{c}>3052.94995]' counts",                  #  91 (open ended max; excl start; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}>=3052.94995]' counts",                 #  92 (open ended max; incl start; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}<2265.7600]' counts",                   #  16 (open ended min; excl end; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}<=2265.7600]' counts",                  #  17 (open ended min; incl end; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}\!=3622.2099]' counts",                 # 115 (all but matching record; excludes nulls)
             "dmlist '{infile}[cols {c}][{c}\!=null]' counts",                      # 116 (all but null)
             "dmlist '{infile}[cols {c}][{c}\!=1000:2000]' counts",                 # err (illegal syntax)
             "dmlist '{infile}[cols {c}][exclude {c}=4000.7199]' counts",           # 115 (all but matching record; excludes nulls)
             "dmlist '{infile}[cols {c}][exclude {c}=null]' counts",                #   0 (empty - no included values)
             "dmlist '{infile}[cols {c}][exclude {c}=3622.2099:4166.91015]' counts",#  93 (inverted range, incl end, excl start; exclude nulls)
             #"dmlist '{infile}[cols {c}][{c}=NaN]' counts",                        #   2 (NaN records only)
            );

my @ifilters = ( # Integer columns                                             # Expected results (byte, int, byte|int_wtn)
             "dmlist '{infile}[cols {c}][{c}=23]' counts",                     #   1,  1,  1 (exact match)
             "dmlist '{infile}[cols {c}][{c}=44]' counts",                     #  11, 11, 11 (exact match multiple hits)
             "dmlist '{infile}[cols {c}][{c}=null]' counts",                   #   0,  0, 20 (null records if TNULL defined)
             "dmlist '{infile}[cols {c}][{c}=22,35]' counts",                  #  10, 10, 10 (exact match, multiple values)
             "dmlist '{infile}[cols {c}][{c}=23,null]' counts",                #   1,  1, 21 (exact value + null records w/TNULL)
             "dmlist '{infile}[cols {c}][{c}=10:20]' counts",                  #  12, 12, 12 (range; incl both ends; excludes nulls w/TNULL)
             "dmlist '{infile}[cols {c}][{c}=33:]' counts",                    # 105, 85, 85 (open ended max; incl start; excludes nulls w/TNULL)
             "dmlist '{infile}[cols {c}][{c}=:]' counts",                      # 138,138,118 (exclude nulls)
             "dmlist '{infile}[cols {c}][{c}=:33]' counts",                    #  36, 56, 36 (open ended min; incl end; excludes nulls w/TNULL)
             "dmlist '{infile}[cols {c}][{c}=10:20,null]' counts",             #  12, 12, 32 (range incl both ends + includes nulls w/TNULL)
             "dmlist '{infile}[cols {c}][{c}>31]' counts",                     # 108, 88, 88 (open ended max; excl start; excludes nulls w/TNULL)
             "dmlist '{infile}[cols {c}][{c}>=31]' counts",                    # 111, 91, 91 (open ended max; incl start; excludes nulls w/TNULL)
             "dmlist '{infile}[cols {c}][{c}<26]' counts",                     #  26, 46, 26 (open ended min; excl end; excludes nulls w/TNULL)
             "dmlist '{infile}[cols {c}][{c}<=26]' counts",                    #  26, 46, 26 (open ended min; incl end; excludes nulls w/TNULL)
             "dmlist '{infile}[cols {c}][{c}\!=36]' counts",                   # 137,137,117 (all but matching record; excludes nulls w/TNULL)
             "dmlist '{infile}[cols {c}][{c}\!=null]' counts",                 # 138,138,118 (all but null)
             "dmlist '{infile}[cols {c}][{c}\!=10:20]' counts",                # err         (illegal syntax)
             "dmlist '{infile}[cols {c}][exclude {c}=40]' counts",             # 131,131,111 (all but matching record; excludes nulls w/TNULL)
             "dmlist '{infile}[cols {c}][exclude {c}=null]' counts",           #   0,  0,  0 (empty - no included values)
             "dmlist '{infile}[cols {c}][exclude {c}=36:41]' counts",          # 113,113, 93 (inverted range, excl start, incl end; exclude nulls w/TNULL)
            );



#########################################################################

# Loop over DM kernels to test under.
my $kref = %infra{'kernels'};
my @kernels = @$kref;
my $kernelName;

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  # Loop over all tests
  foreach my $testid (@testlist)
  {
    # Init per-test variables.
    my $mismatch = 0;
    $err_msg = "";
    my $time_user = 0;
    my $time_sys = 0;

    # change extension based on kernel
    $ext = $exten{$kernelName};

    # delete old outputs
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
    system("$RM -f $testout/${basename}*");

    # get verification set.
    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    # PROGRAMMER - put input filename here
    my $infile = "$testin/indt003m101.$ext";  #all tests use same input.
    # PROGRAMMER - put output filename here
    my $outfile = "$testout/$basename.txt";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/$basename.txt";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/$basename.diff";

    # --------------------------------------------------------------------------------
    # set test command
    my $test_string = "";
    my $col;
    my @filters;

    # Select column and test command set
    if ( $testid == 101 )
    {
      $col = "bcol";
      @filters = @ifilters;
    }
    elsif ( $testid == 102 )
    {
      $col = "bcol_wtn";
      @filters = @ifilters;
    }
    elsif ( $testid == 103 )
    {
      $col = "fcol";
      @filters = @ffilters;
    }
    elsif ( $testid == 104 )
    {
      $col = "fcol_wtn";
      @filters = @ffilters;
    }
    elsif ( $testid == 105 )
    {
      $col = "icol";
      @filters = @ifilters;
    }
    elsif ( $testid == 106 )
    {
      $col = "icol_wtn";
      @filters = @ifilters;
    }
    elsif ( $testid == 107 )
    {
      $col = "scol";
      @filters = @sfilters;
    }
    elsif ( $testid == 108 )
    {
      $col = "scol_wtn";
      @filters = @sfilters;
    }
    elsif ( $testid == 109 )
    {
      $col = "qcol";
      @filters = @qfilters;
    }

    # --------------------------------------------------------------------------------
    # run the test
    #dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );

    open(OUTHND, "> $outfile");
    printf(OUTHND "%30s %60s\n", "Command", $col);
    my $template;
    my $result;
    foreach $template (@filters)
    {
      $result = "";
    
      $test_string = $template;
      $test_string =~ s/{infile}/$infile/g;
      $test_string =~ s/{c}/$col/g;
      open(TMPHND, "$test_string 2>&1 |");
      while ( $line = <TMPHND> )
      {
        chomp($line);
        $result .= $line;
      }
      close(TMPHND);
      if ($result =~ /Failed/){ $result = "Error";}
      $result =~ s/\s//g;
  
      printf(OUTHND "%-80s %10s\n", $template, $result );
    }
    printf(OUTHND "\n\n");
    close(OUTHND);

    # --------------------------------------------------------------------------------
    # validate results
    if ( $block_types{"text"} )
    {
      $err_msg .= diff_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}.sav_filt", \%test_words, $logfname, $test_diff );
    }

    # --------------------------------------------------------------------------------
    # report results
    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        push (@faillist, $kernelName);
        $errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
    $time_user_total += $time_user;
    $time_sys_total  += $time_sys;
    
  } # end test loop
} # end kernel loop

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
