#!/usr/bin/perl
# 
#  Copyright (C) 2004-2005,2008,2017,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '010';
my $semantic = 'bug8522vecfilt';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $version = %infra{'version'};
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmcopy"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
5,
6,
7,
8,
9,
10,
11,
12,
13,
14,
15,
16
);

my %test_blocks = 
( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
1 => { "table" => [ 2, 3, 4, 5, 6 ] },
2 => { "table" => [ 2, 3, 4, 5, 6 ] },
3 => { "table" => [ 2, 3, 4, 5, 6 ] },
4 => { "table" => [ 2, 3, 4, 5, 6 ] },
5 => { "table" => [ 2, 3, 4, 5, 6 ] },
6 => { "table" => [ 2, 3, 4, 5, 6 ] },
7 => { "table" => [ 2, 3, 4, 5, 6 ] },
8 => { "table" => [ 2, 3, 4, 5, 6 ] },
9 => { "table" => [ 2, 3, 4, 5, 6 ] },
10 => { "table" => [ 2, 3, 4, 5, 6 ] },
11 => { "table" => [ 2, 3, 4, 5, 6 ] },
12 => { "table" => [ 2, 3, 4, 5, 6 ] },
13 => { "table" => [ 2, 3, 4, 5, 6 ] },
14 => { "table" => [ 2, 3, 4, 5, 6 ] },
15 => { "table" => [ 2, 3, 4, 5, 6 ] },
16 => { "text" => [ ] },
);

my %test_words = 
(
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
# $version => "ASCDS_VERSION"
 $version => "ASCDS_VERSION",
 $testin  => "<TESTIN>"
);

		     
# Find Tool Executable in PATH
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;

my %exten = ("FITS" => "fits",
             "TEXT" => "dtf" );
my %copyOpts = ("TEXT" => "text/dtf");
my %links = ("1" => "indt010m001");
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#########################################################################

# Loop over DM kernels to test under.
my $kref = %infra{'kernels'};
my @kernels = @$kref;
my $kernelName;

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  # Loop over all tests
  foreach my $testid (@testlist)
  {  
    # Init per-test variables.
    my $mismatch = 0;
    my $time_user;
    my $time_sys;
    $err_msg = "";
    my $skipDelete = 0;
    my $linkname = "";

    # change extension based on kernel
    $in_ext = $exten{$kernelName};

    # delete old outputs
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
    system("$RM -f $testout/${basename}*");

    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    my $infile = "$testin/in${basename}.$in_ext";

    # if testing any kernel other than FITS, the FITS input files need to
    # be copied into the present kernel's format
    if ( $kernelName ne "FITS" )
    {
      my $sourceFile = "$testin/in${basename}.$source_ext";
      my $copyFile   = "$testout/in${basename}.$in_ext";
      my $options    = "opt=all";
      my $kernelOpts = $copyOpts{"$kernelName"};

      if ( ! -e "$infile" )
      {
         my $linkbase = "";
         my $teststring = "$DMCOPY $sourceFile $copyFile\"[opt kernel=$kernelOpts]\" $options";

         # does a previous copy file exist?
         if ($testid > 1 )
         {
           $linkbase = $links{"1"};
           $linkname = "$testout/${linkbase}.$in_ext";
         }

         # if not, make one
         if ( ! -e $copyFile )
         {
           if ( $linkname ne "" )
           {
             if ( -e $linkname )
             {
               # create link
               $teststring = "ln -s $linkname $copyFile";
             }
           }

           # copy fits file to text/dtf file
           ($rc, $time_user, $time_sys) = system($teststring);
         }

         $infile = $copyFile;

         # Tests 1 through 15 use the same input file.
          # Don't delete generated input file
         if ( $testid == 1 )
         {
           $skipDelete = 1;
         }
      }
      else
      {
        $skipDelete = 1;
      }
    }

    # PROGRAMMER - put output filename here
    my $outfile = "$testout/${basename}.fits";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/${basename}.fits.gz";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/${basename}.diff";

    # --------------------------------------------------------------------------------
    # set test command
    my $test_string = "";

    if ( $testid == 1 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[exclude chipx=3,chipy=301\]\" outfile=$outfile";
    }
    elsif ( $testid == 2 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[chipx=3,chipy=301\]\" outfile=$outfile";
    }
    elsif ( $testid == 3 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[x=4400:4430,y=2902:2903\]\" outfile=$outfile";
    }
    elsif ( $testid == 4 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[exclude x=4400:4430,y=2902:2903\]\" outfile=$outfile";
    }
    elsif ( $testid == 5 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[y=2902:2903\]\" outfile=$outfile";
    }
    elsif ( $testid == 6 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[chip=circle(100,300,100)]\" outfile=$outfile";
    }
    elsif ( $testid == 7 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[sky=circle(4300,2900,50)\]\" outfile=$outfile";
    }
    elsif ( $testid == 8 )
    {
      # Bug #8521 - exclude filtering cpts - integer region
      $test_string = "$DMCOPY infile=$infile'[exclude tdet=circle(3300,5000,100)]' outfile=$outfile";
    }
    elsif ( $testid == 9 )
    {
      # Bug #8521 integer x cpt
      $test_string = "dmcopy infile=$infile'[exclude tdetx=3350:3400,3450:3500]' outfile=$outfile";
    }
    elsif ( $testid == 10 )
    {
      # Bug #8521 integer y cpt, off range
      $test_string = "$DMCOPY infile=$infile'[exclude tdety=5000:5100]' outfile=$outfile";
    }
    elsif ( $testid == 11 )
    {
      # Bug #8521 real region
      $test_string = "$DMCOPY infile=$infile'[exclude sky=circle(4300,2900,100)]' outfile=$outfile";
    }
    elsif ( $testid == 12 )
    {
      # Bug #8521 real x cpt
      $test_string = "$DMCOPY infile=$infile'[exclude x=4096:4300,4350:4400]' outfile=$outfile";
    }
    elsif ( $testid == 13 )
    {
      # Bug #8521 real y cpt
      $test_string = "$DMCOPY infile=$infile'[exclude y=2096:2900,2950:3000]' outfile=$outfile";
    }
    elsif ( $testid == 14 )
    { 
      # Bug #8521 ahelp example
      $test_string = "$DMCOPY infile=$infile'[exclude (ccd_id=1:6,8:9,chipx=0:100,fltgrade=10:20)||(ccd_id=0,chipx=50:100)]' outfile=$outfile";
    }
    elsif ( $testid == 15 )
    {
      # Bug #8521: This test is broken!  Need Bug #!
      $test_string = "$DMCOPY infile=$infile'[exclude tdet=circle(3300,5000,100),tdety=4900:4950,5000:5100]' outfile=$outfile";
    }
    elsif ( $testid == 16 )
    {
      # Bug #SL92: exclude field = empty table with NULL subspace
      $outfile = "$testout/${basename}.txt";
      $savfile = "$testsav/${basename}.txt";
      $test_string = "$DMLIST infile=$infile'[exclude chip=field()]' outfile=$outfile opt=blocks,subspace";
    }
    $test_string .= " >> $logfname 2>&1";

    # --------------------------------------------------------------------------------
    # run the test
    dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
    close $logfile;
    
    ($rc, $time_user, $time_sys) = dt_run_test($test_string);
    
    open $logfile, ">>$logfname";
    select($logfile); $| = 1; select($printfile);
    unless ( $rc == 0 )
    {
      $err_msg = "ERROR: $toolname failed to run\n";
    }
   
    # --------------------------------------------------------------------------------
    # validate results
    if ( $block_types{"table"} )
    {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
    }
    if ( $block_types{"image"} )
    {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
    }
    if ( $block_types{"list"} )
    {
      $err_msg .= dmlist_test( $outfile, "${outfile}_filt", $savfile, "${savfile}_filt", \@tolkeys, $logfname, $test_diff, "" );
    }
    if ( $block_types{"text"} )
    {
      $err_msg .= diff_test( $outfile, "${outfile}_outfilt", $savfile, "${outfile}_savfilt", \%test_words, $logfname, $test_diff );
    }

    # --------------------------------------------------------------------------------
    # report results
    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        push (@faillist, $kernelName);
        $errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
    $time_user_total += $time_user;
    $time_sys_total  += $time_sys;
    
    # --------------------------------------------------------------------------------
    # delete any input files created just for this kernel
    if ( $kernelName ne "FITS" )
    {
      if ( $skipDelete == 0 )
      {
        system("$RM -f $infile");
      } 
      if ( $testid == 15 )
      {
        system("$RM -f $linkname");
      }               
    }
  }
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
