#!/usr/bin/perl
# 
#  Copyright (C) 2004-2005,2008,2015,2021,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '012';
my $semantic = 'rebin';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $version = %infra{'version'};
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmcopy"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
5,
6,
7,
8,
9,
10,
11,
12,
13,
14,
15,
16,
17,
18,
19,
20,
21,
22,
23,
24,
25,
26,
27,
28
);

my %test_blocks = 
( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
1 => { "image" => [1] },
2 => { "image" => [1] },
3 => { "text" => [1] },
4 => { "text" => [1] },
5 => { "text" => [1] },
6 => { "image" => [1] },
7 => { "image" => [1] },
8 => { "image" => [1] },
9 => { "image" => [1] },
10 => { "image" => [1] },
11 => { "image" => [1] },
12 => { "image" => [1] },
13 => { "image" => [1] },
14 => { "image" => [1] },
15 => { "image" => [1],
	"table" => [2, 3, 4, 5, 6, 7] },
16 => { "image" => [1],
	"table" => [2, 3, 4, 5, 6, 7] },
17 => { "image" => [1],
	"table" => [2, 3, 4, 5, 6, 7] },
18 => { "image" => [1] },
19 => { "image" => [1] },
20 => { "image" => [1] },
21 => { "list"  => [ ] },
22 => { "list"  => [ ] },
23 => { "list"  => [ ] },
24 => { "list"  => [ ] },
25 => { "list"  => [ ] },
26 => { "list"  => [ ] },
27 => { "list"  => [ ] },
28 => { "list"  => [ ] },
);

my %test_words = 
(
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
  $testin  => "TESTIN",
  $version => "ASCDS_VERSION",
  '#\s\d+:' => "# <PID>:"
);

		     
# Find Tool Executable in PATH
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;

my %exten = ("FITS" => "fits",
             "TEXT" => "dtf" );
my %copyOpts = ("TEXT" => "text/dtf");
my %links = ("1"  => "indt012m001",
             "12" => "indt012m012",
             "15" => "indt012m015",
             "18" => "indt012m018"); 
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#########################################################################

# Loop over DM kernels to test under.
my $kref = %infra{'kernels'};
my @kernels = @$kref;
my $kernelName;

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  # Loop over all tests
  foreach my $testid (@testlist)
  {
    # Init per-test variables.
    my $mismatch = 0;
    my $time_user;
    my $time_sys;
    $err_msg = "";
    my $skipDelete = 0;
    my $linkname = "";

    # change extension based on kernel
    $in_ext = $exten{$kernelName};

    # delete old outputs
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
    system("$RM -f $testout/${basename}*");

    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    my $infile = "$testin/in${basename}.$in_ext";

    # if testing any kernel other than FITS, the FITS input files need to
    # be copied into the present kernel's format
    if ( $kernelName ne "FITS" )
    {
      my $sourceFile = "$testin/in${basename}.$source_ext";
      my $copyFile   = "$testout/in${basename}.$in_ext";
      my $options    = "";
      my $kernelOpts = $copyOpts{"$kernelName"};

      if ( ! -e "$infile" )
      {
         my $linkbase = "";
         my $teststring = "$DMCOPY $sourceFile $copyFile\"[opt kernel=$kernelOpts]\" $options";

         # does a previous copy file exist?
         if ( $testid >= 2 && $testid <= 11)
         {
           $linkbase = $links{"1"};
           $linkname = "$testout/${linkbase}.$in_ext";
         }
         elsif ( $testid >= 13 && $testid <= 14 )
         {
           $linkbase = $links{"12"};
           $linkname = "$testout/${linkbase}.$in_ext";
         }
         elsif ( $testid == 16 )
         {
           $linkbase = $links{"15"};
           $linkname = "$testout/${linkbase}.$in_ext";
         }
         elsif ( $testid == 19 )
         {
           $linkbase = $links{"18"};
           $linkname = "$testout/${linkbase}.$in_ext";
         }

         # if not, make one
         if ( ! -e $copyFile )
         {
           if ( $linkname ne "" )
           {
             if ( -e $linkname )
             {
               # create link
               $teststring = "ln -s $linkname $copyFile";
             }
           }

           # copy fits file to text/dtf file
           ($rc, $time_user, $time_sys) = system($teststring);
         }

        $infile = $copyFile;

         # Tests 1 - 11 use the same input file (links exist).
         # Tests 12 - 14 use the same input file
         # Tests 15 and 16 use the same input file
         # Tests 17 and 18 use the same input file (links exist)
         # Don't delete generated input file
         if ( $testid == 1 || $testid == 12 || $testid == 15 || $testid == 18 )
         {
           $skipDelete = 1;
         }
      }
      else
      {
        $skipDelete = 1;
      }
    }

    # PROGRAMMER - put output filename here
    my $outfile = "$testout/${basename}.fits";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/${basename}.fits.gz";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/${basename}.diff";

    # --------------------------------------------------------------------------------
    # set test command
    my $test_string = "";

    if ( $testid == 1 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[bin #1=::4,#2=::4\]\" outfile=$outfile  >> $logfname 2>&1";
    }
    elsif ( $testid == 2 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[bin sky=16\]\" outfile=$outfile  >> $logfname 2>&1";
    }
    elsif ( $testid == 3 )
    {
      $outfile = "$testout/${basename}.txt";
      $savfile = "$testsav/${basename}.txt";
      $test_string = "$DMCOPY infile=\"$infile\[bin sky=20\]\" outfile=none.fits > $outfile 2>&1";
    }
    elsif ( $testid == 4 )
    {
      $outfile = "$testout/${basename}.txt";
      $savfile = "$testsav/${basename}.txt";
      $test_string = "$DMCOPY infile=\"$infile\[bin sky=4\]\" outfile=none.fits > $outfile 2>&1";
    }
    elsif ( $testid == 5 )
    {
      $outfile = "$testout/${basename}.txt";
      $savfile = "$testsav/${basename}.txt";
      $test_string = "$DMCOPY infile=\"$infile\[bin #1=1.5,#2=1.5\]\" outfile=none.fits > $outfile 2>&1";
    }
    elsif ( $testid == 6 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[bin #1=2,#2=4\]\" outfile=$outfile  >> $logfname 2>&1";
    }
    elsif ( $testid == 7 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[bin x=16,y=32\]\" outfile=$outfile  >> $logfname 2>&1";
    }
    elsif ( $testid == 8 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[bin #1=0.5:1024.5:2,#2=0.5:1024.5:2\]\" outfile=$outfile  >> $logfname 2>&1";
    }
    elsif ( $testid == 9 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[bin sky=500:5000:24\]\" outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 10 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[bin #1=11:1000:2,#2=359:900:2\]\" outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 11 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[bin x=80.5:8000.5:16,y=2864.5:7200.5:16\]\" outfile=$outfile >> $logfname 2>&1";      
    }
    elsif ( $testid == 12 )
    {
      # Test default bin factor for physical col sky
      $test_string = "$DMCOPY infile=\"$infile\[bin sky\]\" outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 13 )
    {
      # Test default bin factor for physical cols x,y
      $test_string = "$DMCOPY infile=\"$infile\[bin x,y\]\" outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 14 )
    {
      # rebin outside of image boundaries
      $test_string = "$DMCOPY infile=\"$infile\[bin x=0:9000:64,y=0:10000:64\]\" outfile=$outfile >> $logfname 2>&1";      
    }
    elsif ( $testid == 15 )
    {
      # real event data
      $test_string = "$DMCOPY infile=$infile'[sky=circle(4000,4000,150)][bin sky=16]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 16 )
    {
      # binning on logical w/ real data
      $test_string = "$DMCOPY infile=$infile'[bin #1=500:550:2,#2=500:550:2]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 17 )
    {
      # rebinning i2 image
      $test_string = "$DMCOPY infile=$infile'[bin x=3950:4050:2,y=3950:4050:2]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 18 )
    {
      # i2 rectangular image, rebin by 4
      $test_string = "$DMCOPY infile=$infile'[bin pos=4]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 19 )
    {
      # i2 rectangular image, rebin by 16
      # Need to catch rolling over of values! Bug! (Not sure which Bug#...)
      $test_string = "$DMCOPY infile=$infile'[bin pos=16]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 20 )
    {
      # Need to test copy onto bigger grid once that is fixed.
      # Bug #8904
      $test_string = "$DMCOPY infile=$infile'[bin #1=-9:500:1,#2=-9:500:1][opt null=-99]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 21 )
    {
      # SL-35 Test Plan - 2.3.1 bin - straight copy of 2D image
      $test_string = "$DMCOPY infile=$infile'[2D_I2_BASIC][bin x=40,y=40]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 22 )
    {
      # SL-35 Test Plan - 2.3.2 bin - straight copy of scaled 2D image
      $test_string = "$DMCOPY infile=$infile'[2D_I2_SCALED][bin x=40,y=40]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 23 )
    {
      # SL-35 Test Plan - 3.1.1 bin - [bin x,y,z] on 3D image
      $test_string = "$DMCOPY infile=$infile'[3D_I2_BASIC][bin x,y,z=100]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 24 )
    {
      # SL-35 Test Plan - 3.1.2 bin - [bin x,y,z] on scaled 3D image
      $test_string = "$DMCOPY infile=$infile'[3D_I2_SCALED][bin x,y,z=100]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 25 )
    {
      # SL-35 Test Plan - 3.2.1 bin - [bin x,y] (compact 3D->2D) on 3D image
      $test_string = "$DMCOPY infile=$infile'[3D_I2_BASIC][bin x,y]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 26 )
    {
      # SL-35 Test Plan - 3.2.2 bin - [bin x,y] (compact 3D->2D) on scaled 3D image
      $test_string = "$DMCOPY infile=$infile'[3D_I2_SCALED][bin x,y]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 27 )
    {
      # SL-35 Test Plan - 4.1.1 bin - [bin x,y,z] on 4D image 
      $test_string = "$DMCOPY infile=$infile'[4D_I2_BASIC][bin x,y,z]' outfile=$outfile >> $logfname 2>&1";
    }
    elsif ( $testid == 28 )
    {
      # SL-35 Test Plan - 4.1.2 bin - [bin x,y,z] on 4D scaled image
      $test_string = "$DMCOPY infile=$infile'[4D_I2_SCALED][bin x,y,z]' outfile=$outfile >> $logfname 2>&1";
    }

    # --------------------------------------------------------------------------------
    # run the test
    dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
    close $logfile;
    
    ($rc, $time_user, $time_sys) = dt_run_test($test_string);
    
    open $logfile, ">>$logfname";
    select($logfile); $| = 1; select($printfile);
    unless ( $rc == 0 || $testid == 3 || $testid == 4 || $testid == 5 )
    {
      $err_msg = "ERROR: $toolname failed to run\n";
    }
   
    # --------------------------------------------------------------------------------
    # validate results
    if ( $block_types{"table"} )
    {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
    }
    if ( $block_types{"image"} )
    {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
    }
    if ( $block_types{"list"} )
    {
      $err_msg .= dmlist_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}_savfilt", \@tolkeys, $logfname, $test_diff, "" );
    }
    if ( $block_types{"text"} )
    {
      $err_msg .= diff_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}_savfilt", \%test_words, $logfname, $test_diff );
    }
    if ( $block_types{"dtf"} )
    {
      $err_msg .= dtf_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}_savfilt", \@tolkeys, $logfname, $test_diff, "" );
    }

    # --------------------------------------------------------------------------------
    # report results
    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        push (@faillist, $kernelName);
        $errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
    $time_user_total += $time_user;
    $time_sys_total  += $time_sys;

    # --------------------------------------------------------------------------------
    # delete any input files created just for this kernel
    if ( $kernelName ne "FITS" )
    {
      if ( $skipDelete == 0 )
      {
        system("$RM -f $infile");
      }
      if ( $testid==11 || $testid==14 || $testid==16 || $testid==19)
      {
        system("$RM -f $linkname");
      }        
    }
  }
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
