#!/usr/bin/perl
# 
#  Copyright (C) 2004-2009,2013,2015,2019,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '022';
my $semantic = 'asciiRead';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmlist"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
5,
6,
7,
8,
9,
10,
11,
12,
13,
14,
15,
16,
17,
18,
19,
20,
21,
22, # SL-79: TSV file with dupe col names, missing, colspec, collist order different from colspec
23, # SL-79: TSV file with dupe col names, missing, colspec, collist order different from colspec. Filter col order
24, #SL-91: DTF file with blank CONTINUE and comment
25, #SL-91: DTF file with non-blank CONTINUE and comment
26, #SL-99: check interpretation of dash as NaN for column with no type defined
);

my %test_blocks = ( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
 1 => { "text" => [] },
 2 => { "text" => [] },
 3 => { "text" => [] },
 4 => { "text" => [] },
 5 => { "text" => [] },
 6 => { "text" => [] },
 7 => { "text" => [] },
 8 => { "text" => [] },
 9 => { "text" => [] },
 10 => { "text" => [] },
 11 => { "text" => [] },
 12 => { "text" => [] },
 13 => { "text" => [] },
 14 => { "text" => [] },
 15 => { "text" => [] },
 16 => { "text" => [] },
 17 => { "text" => [] },
 18 => { "text" => [] },
 19 => { "text" => [] },
 20 => { "text" => [] },
 21 => { "text" => [] },
 22 => { "text" => [] },
 23 => { "text" => [] },
 24 => { "text" => [] },
 25 => { "text" => [] },
 26 => { "text" => [] }
);

my %test_words = (
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
# $version => "ASCDS_VERSION"
#
# Linux and Solaris output different "nan" values, so make
# them identical for comparison purposes
  $testin  => "TESTIN",
  'NaN' => "NAN",
  'nan\s' => "NAN "
);

		     
# Find Tool Executable in PATH
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;
#########################################################################

# Loop over all tests
foreach my $testid (@testlist)
{
   # Init per-test variables.
   my $mismatch = 0;
   my $time_user;
   my $time_sys;
   $err_msg = "";

   # delete old outputs
   my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
   system("$RM -f $testout/${basename}*");

   my $infile = "$testin/in${basename}";
   # PROGRAMMER - put output filename here
   my $outfile = "$testout/${basename}.txt";
   # PROGRAMMER - put saved output filename here
   my $savfile = "$testsav/${basename}.txt";
   # PROGRAMMER - put name of diff file here, if needed
   my $test_diff = "$testout/${basename}.diff";

   # --------------------------------------------------------------------------------
   # set test command
   my $test_string = "";
   
   if ( $testid == 1 )
   {
      # raw file with blank line header test
      $infile = "${infile}.raw";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";
   }
   elsif ( $testid == 2 )
   {
      # raw with descriptive type header
      $infile = "${infile}.raw";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";
   }
   elsif ( $testid == 3 )
   {
      # raw file with no header
      $infile = "${infile}.raw";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";
   }
   elsif ( $testid == 4 )
   {
      # raw file with lines to skip
      $infile = "${infile}.raw";
      $test_string = "$DMLIST infile=$infile'[opt skip=5]' opt=full,all outfile=$outfile";
   }
   elsif ( $testid == 5 )
   {
      # simple with descriptive type header
      $infile = "${infile}.simple";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";   
   }
   elsif ( $testid == 6 )
   {
      # simple without descriptive type header, just column names
      $infile = "${infile}.simple";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";   
   }
   elsif ( $testid == 7 )
   {
      # simple with lines to skip
      $infile = "${infile}.simple";
      $test_string = "$DMLIST infile=$infile'[opt skip=5]' opt=full,all outfile=$outfile opt=skip=5";   
   }
   elsif ( $testid == 8 )
   {
     # simple - Bug 9531 - column header in unusual format, variation 1
      $infile = "${infile}.simple";
      $test_string = "$DMLIST infile=$infile opt=data outfile=$outfile";
   }
   elsif ( $testid == 9 )
   {
     # simple - Bug 9531 - column header in unusual format, variation 2,
     #                     with lines to skip
     $infile = "${infile}.simple"; 
     $test_string = "$DMLIST infile=$infile'[opt skip=6]' opt=data outfile=$outfile";   
   }
   elsif ( $testid == 10 )
   {
      #dtf with descriptive type header
      # Ryan, this is printing the invisible axis on the array
      $infile = "${infile}.dtf";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";
   }
   elsif ( $testid == 11 )
   {
      #dtf without descriptive type header
      $infile = "${infile}.dtf";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";
   }
   elsif ( $testid == 12 )
   {
      #dtf with descriptive type header and lines to skip
      $infile = "${infile}.dtf";
      $test_string = "$DMLIST infile=$infile'[opt skip=5]' opt=full,all outfile=$outfile opt=skip=5";
   }
   elsif ( $testid == 13 )
   {
      #dtf fixed file with descriptive type header
      $infile = "${infile}.dtf";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";
   }
   elsif ( $testid == 14 )
   {
      #dtf fixed without descriptive type header
      $infile = "${infile}.dtf";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";
   }
   elsif ( $testid == 15 )
   {
      # raw file with comma used as separator
      $infile = "${infile}.raw";
      $test_string = "$DMLIST infile=$infile'[opt sep=\",\"]' opt=full,all outfile=$outfile";
   }
   elsif ( $testid == 16 )
   {
      # raw file with NULLSTR=999
      $infile = "${infile}.raw";
      $test_string = "$DMLIST infile=$infile'[opt NULLSTR=999]' opt=data outfile=$outfile";
   }
   elsif ( $testid == 17 )
   {
      # DTF file with TNULL=999
      $infile = "${infile}.dtf";
      $test_string = "$DMLIST infile=$infile opt=data outfile=$outfile";
   }
   elsif ( $testid == 18 )
   {
      # CSCRDB file with column names
      $infile = "${infile}.cscrdb";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";   
   }
   elsif ( $testid == 19 )
   {
      # CSCRDB file with column names and units 
      $infile = "${infile}.cscrdb";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";   
   }
   elsif ( $testid == 20 )
   {
      # CSCRDB file with no column names
      $infile = "${infile}.cscrdb";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";   
   }
   elsif ( $testid == 21 )
   {
      # TSV file with column disp code as "G', no data in column 1
      $infile = "${infile}.tsv'[opt kernel=text/tsv]'";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";   
   }
   elsif ( $testid == 22 )
   {
      # TSV file with 2 columns having the same name
      $infile = "${infile}.tsv'[opt kernel=text/tsv]'";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";   
   }
   elsif ( $testid == 23 )
   {
      # TSV file with 2 columns having the same name + shuffle cols around
      $infile = "${infile}.tsv'[opt kernel=text/tsv][cols #6, #1, #3, #7, #2, #4, #5]'";
      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";   
   }
   elsif ( $testid == 24 )
   {
      #dtf file with blank Continue and comment
      $infile = "${infile}.dtf";
      $test_string = "$DMLIST infile=$infile opt=head outfile=$outfile";
   }
   elsif ( $testid == 25 )
   {
      #dtf file with non-blank Continue and comment
      $infile = "${infile}.dtf";
      $test_string = "$DMLIST infile=$infile opt=head outfile=$outfile";
   }
   elsif ( $testid == 26 )
   {
      # Check interpretation of dash as NaN for column with no type defined
      $infile = "${infile}.simple";
      $test_string = "$DMLIST infile=$infile opt=data outfile=$outfile";
   }
#   elsif ( $testid == 22 )
#   {
      # This test is expected to fail
      #cvs file - to test unrecognized kernel
#      $infile = "${infile}.cvs";
#      $test_string = "$DMLIST infile=$infile opt=full,all outfile=$outfile";
#   }

   $test_string .= " >> $logfname 2>&1";

   # --------------------------------------------------------------------------------
   # run the test
   dt_print_test_header( $testid, $test_string, $logfile, $printfile );
   close $logfile;
   
   ($rc, $time_user, $time_sys) = dt_run_test($test_string);
   
   open $logfile, ">>$logfname";
   select($logfile); $| = 1; select($printfile);
   unless ( $rc == 0 )
   {
      $err_msg = "ERROR: $toolname failed to run\n";
   }
   
   # --------------------------------------------------------------------------------
   # validate results
   my $block_typesref = $test_blocks{$testid};
   my %block_types = %$block_typesref;
   if ( $block_types{"table"} )
   {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
	 $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
   }
   if ( $block_types{"image"} )
   {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
	 $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
   }
   if ( $block_types{"list"} )
   {
      $err_msg .= dmlist_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}_savfilt", \@tolkeys, $logfname, $test_diff, "" );
   }
   if ( $block_types{"text"} )
   {
      $err_msg .= diff_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}_savfilt", \%test_words, $logfname, $test_diff );
   }

   # --------------------------------------------------------------------------------
   # report results
   unless ( $err_msg eq "" )
   {
     push( @faillist, $testid );
     $mismatch = 1;
   }
   dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
   $time_user_total += $time_user;
   $time_sys_total  += $time_sys;

}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
