#!/usr/bin/perl
# 
#  Copyright (C) 2004-2008,2012,2015,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '026';
my $semantic = 'ParseErrors';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $version = %infra{'version'};
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my @tolkeys;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmlist"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
 "1",
 "2",
 "3",
 "4",
 "5",
 "6",
 "7",
 "8",
 "9",
 "10",
 "11",
 "12",
 "13",
 "14",
 "15",
 "16",
 "17",
 "18",
 "19"
);

my %test_blocks = ( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
 "1" => { "text" => [ ] },
 "2" => { "text" => [ ] },
 "3" => { "text" => [ ] },
 "4" => { "text" => [ ] },
 "5" => { "text" => [ ] },
 "6" => { "text" => [ ] },
 "7" => { "text" => [ ] },
 "8" => { "text" => [ ] },
 "9" => { "text" => [ ] },
 "10" => { "text" => [ ] },
 "11" => { "text" => [ ] },
 "12" => { "text" => [ ] },
 "13" => { "text" => [ ] },
 "14" => { "text" => [ ] },
 "15" => { "text" => [ ] },
 "16" => { "text" => [ ] },
 "17" => { "text" => [ ] },
 "18" => { "text" => [ ] },
 "19" => { "text" => [ ] }
);

my %test_words = (
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
  $testin  => "TESTIN",
  $version => "ASCDS_VERSION",
  '#\s\d+:' => "# <PID>:"  
);

		     
# Find Tool Executable in PATH
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my $err_msg = "";
my $rc;
my @faillist = ( );
my $time_user_total = 0;
my $time_sys_total = 0;

my %exten = ("FITS" => "fits",
             "TEXT" => "dtf" );
my %copyOpts = ("TEXT" => "text/dtf");
my %links = ("1" => "indt026m001",
             "3" => "indt026m003",
             "5" => "indt026m005");
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#########################################################################

# Loop over DM kernels to test under.
my $kref = %infra{'kernels'};
my @kernels = @$kref;
my $kernelName;

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  # Loop over all tests
  foreach my $testid (@testlist)
  {
    # Init per-test variables.
    my $mismatch = 0;
    my $time_user;
    my $time_sys;
    $err_msg = "";
    my $skipDelete = 0;
    my $linkname = "";

    # change extension based on kernel
    $in_ext = $exten{$kernelName};

    # delete old outputs
    my $base     = sprintf "dt%.3d", $id;
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
    system("$RM -f $testout/${basename}*");

    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    my $inbase = "$testin/in${basename}";
    my $infile = "$testin/in${basename}.$in_ext";
    my $inbase_filt = "$testin/in${base}";

    # if testing any kernel other than FITS, the FITS input files need to
    # be copied into the present kernel's format
    if ( $kernelName ne "FITS" )
    {
      my $sourceFile = "$testin/in${basename}.$source_ext";
      my $copyFile   = "$testin/in${basename}.$in_ext";
      my $options    = "opt=all";
      my $kernelOpts = $copyOpts{"$kernelName"};

      if ( $testid == 16 || $testid == 17 || $testid ==18 )
      {
        # these files because they are simple TEXT files
         $infile = "${inbase}.simple";
         $skipDelete = 1;
      }
      elsif ( ! -e "$infile" )
      {
         my $linkbase = "";
         my $teststring = "$DMCOPY $sourceFile $copyFile\"[opt kernel=$kernelOpts]\" $options";

         # does a previous copy file exist?
         if (($testid >= 12 && $testid <= 15) || $testid == 1 || $testid == 2 )
         {
           $linkbase = $links{"1"};
           $linkname = "$testout/${linkbase}.$in_ext";
         }
         elsif ($testid == 3 || $testid == 4 || $testid == 9 || $testid == 10 || $testid == 19 )
         {
           $linkbase = $links{"3"};
           $linkname = "$testout/${linkbase}.$in_ext";
         }
         elsif ($testid == 5 || $testid == 7 || $testid == 8 || $testid == 11 )
         {
           $linkbase = $links{"5"};
           $linkname = "$testout/${linkbase}.$in_ext";
         }

         # if not, make one
         if ( ! -e $copyFile )
         {
           if ( $linkname ne "" )
           {
             if ( -e $linkname )
             {
               # create link
               $teststring = "ln -s $linkname $copyFile";
             }
           }

           # copy fits file to text/dtf file or create link
           ($rc, $time_user, $time_sys) = system($teststring);
         }

         $infile = $copyFile;

         # Tests 1, 2 , 12, 13, 14, and 15 use the same input file.
         # Tests 3, 4, 9 10, and 19 use the same input file
         # Tests 5, 7, 8 and 11 use the same input file
         # Don't delete generated input file
         if ( $testid == 1 || $testid == 3 || $testid == 5 )
         {
           $skipDelete = 1;
         }
      }
      else
      {
        $skipDelete = 1;
      }
    }

    # PROGRAMMER - put output filename here
    my $outfile = "$testout/${basename}.txt";
    my $outfilefilt = "$testout/${basename}.txt_filt";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/${basename}.txt";
    my $savfilefilt = "$testout/${basename}.txt_std";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/${basename}.diff";

    # --------------------------------------------------------------------------------
    # set test command
    my $test_string;

    if ( $testid == 1 )
    {
      # Bug 6560:
      #   This should produce an error (missing ')')
      $test_string = "$DMLIST $infile\"" . '[(x,y)=region(' . "${inbase_filt}srclist.fits" . '[component=lgrid\(1:2:1])][opt null=nan]' . "\" blocks outfile=$outfile,$outfile";
    }
    elsif ( $testid == 2 )
    {
      # Bug 6560:
      #   This should run
      $test_string = "$DMLIST $infile\"" . '[(x,y)=region(' . "${inbase_filt}srclist.fits" . '[component=lgrid(1:2:1)])][opt null=nan]' . "\" blocks outfile=$outfile,$outfile";
    }
    elsif ( $testid == 3 )
    {
      # Bug 7743:
      #   This should produce an error (should be "region(fov.fits)")
      $test_string = "$DMLIST \"$infile" . '[sky=bounds(' . "${inbase_filt}fov.fits" . ')][bin sky=1]' . "\" blocks outfile=$outfile";
    }
    elsif ( $testid == 4 )
    {
      # Bug 7743:
      #   This should run
      $test_string = "$DMLIST \"$infile" . '[sky=bounds(region(' . "${inbase_filt}fov.fits" . '))][bin sky=1]' . "\" blocks outfile=$outfile";
    }
    elsif ( $testid == 5 )
    {
      # Bug 8618:
      #   This should produce an error (extra ']')
      $test_string = "$DMLIST $infile\'[channel=160:210]][cols counts]\' cols";
    }
    elsif ( $testid == 6 )
    {
      # Bug 8618:
      #   This should run
      $test_string = "$DMLIST $infile\'[channel=160:210][cols counts]' cols outfile=$outfile";
    }
    #
    # Various bad-syntax cases (all should produce errors)
    #
    elsif ( $testid == 7 )
    {
      # space in block name
      # NB:  Code change for Bug 10558 allows for spaces in block names
      #      so this is no longer an error. However since there is no
      #      column named "spectrum foo", this test generates a block not
      #      found error. Run this test and check for that error.
      #      MW 7/5/2007
     
      $test_string = "$DMLIST $infile\"[spectrum foo]\" blocks outfile=$outfile";
    }
    elsif ( $testid == 8 )
    {
      # too many block names
      $test_string = "$DMLIST $infile\"[spectrum][myblock][foo]\" blocks outfile=$outfile";
    }
    elsif ( $testid == 9 )
    {
      # multiple bin specs
      $test_string = "$DMLIST $infile\"[bin x=1:8192:4,y=1:8192:4][bin x=1:8192:8,y=1:8192:8]\" blocks outfile=$outfile";
    }
    elsif ( $testid == 10 )
    {
      # missing ')'
      $test_string = "$DMLIST \"$infile" . '[sky=bounds\(region(' . "${inbase_filt}fov.fits" . ')]' . "\" blocks outfile=$outfile";
    }
    elsif ( $testid == 11 )
    {
      # exclude and filter
      $test_string = "$DMLIST $infile\'[exclude counts>2][channel=160:210]' blocks outfile=$outfile";
    }
    elsif ( $testid == 12 )
    {
      # ')' where there should be ']'
      $test_string = "$DMLIST $infile\"" . '[(x,y)=region(' . "${inbase_filt}srclist.fits" . '[component=1\))]' . "\" blocks outfile=$outfile";
    }
    #
    # These test the limit of 32 levels of () and [] nesting that is built into
    # cs_parse_paren_token_validate()
    #
    elsif ( $testid == 13 )
    {
      # 32 levels of nesting -- this is bad region syntax but shouldn't generate a parse error
      $test_string = "$DMLIST \"$infile" . '[sky=bounds(region((((((((((((((((((((((((((((((' . "${inbase_filt}fov.fits" . ')))))))))))))))))))))))))))))))]' . "\" blocks outfile=$outfile";
    }
    elsif ( $testid == 14 )
    {
      # 65 levels of nesting -- this should generate a parse error
      # should fail in both the 32-bit and 64-bit builds
      $test_string = "dmlist \"$infile" . '[sky=bounds(region(((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((' . "${inbase_filt}fov.fits" . '))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))]' . "\" blocks outfile=$outfile";
    }
    elsif ( $testid == 15 )
    {
      # Bug 8953: too many filter ranges
      $test_string = "$DMLIST $infile\'[energy=300:10000:10]' blocks outfile=$outfile";
    }
    # tests added May 2, 2006 for SIMPLE header parsing for vector arrays
    if ( $testid == 16 )
    {
      # check for vector array in SIMPLE header
      $test_string = "$DMLIST \"${inbase}.simple\" opt=cols outfile=$outfile";
    }
    elsif ( $testid == 17 )
    {
      # check for vector array in SIMPLE header
      $test_string = "$DMLIST \"${inbase}.simple\" opt=cols outfile=$outfile";
    }
    elsif ( $testid == 18 )
    {
      # check for vector array in SIMPLE header and disregards extra spaces
      # around column name or in array brackets
      $test_string = "$DMLIST \"${inbase}.simple\" opt=cols outfile=$outfile";
    }
    elsif ( $testid == 19 )
    {
      # Bug 13286 - no operator following name in second filter clause
      #  DM interprets new clause as part of value following first op
      #  e.g. "ccd_id>1,node_id", value is assumed to be "1,node_id"
      #  which is invalid numeric value "node_id" alone is not valid filter.
      $test_string = "$DMLIST $infile\'[ccd_id>1,node_id]' opt=data outfile=$outfile";
    }
    $test_string .= " >> $logfname 2>$outfile";

    # --------------------------------------------------------------------------------
    # run the test
    dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
    close $logfile;
    
    ($rc, $time_user, $time_sys) = dt_run_test($test_string);
    
    open $logfile, ">>$logfname";
    select($logfile); $| = 1; select($printfile);
    #unless ( $rc == 0 )
    #{
    #   $err_msg = "ERROR: $toolname failed to run\n";
    #}
   
    # --------------------------------------------------------------------------------
    # validate results
    if ( $block_types{"table"} )
    {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
    }
    if ( $block_types{"image"} )
    {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
    }
    if ( $block_types{"list"} )
    {
      $err_msg .= dmlist_test( $outfile, "${outfile}_filt", $savfile, "${savfile}_filt", \@tolkeys, $logfname, $test_diff, "" );
    }
    if ( $block_types{"text"} )
    {
      $err_msg .= diff_test( $outfile, $outfilefilt, $savfile, $savfilefilt, \%test_words, $logfname, $test_diff );
    }

    # --------------------------------------------------------------------------------
    # report results
    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        push (@faillist, $kernelName);
        $errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
    $time_user_total += $time_user;
    $time_sys_total  += $time_sys;
    
    # --------------------------------------------------------------------------------
    # delete any input files created just for this kernel
    if ( $kernelName ne "FITS" )
    {
      if ( $skipDelete == 0 )
      {
        system("$RM -f $infile");
      }
      if ($testid == 10 || $testid == 11 || $testid == 17)
      {
          system("$RM -f $linkname");
      }        
    }
  }
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;

