#!/usr/bin/perl
# 
#  Copyright (C) 2015-2017,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '047';
my $semantic = 'boolean3stat';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";
$ENV{'LC_ALL'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $version = %infra{'version'};
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmcopy"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
5,
6,
7,
8,
9,
10,
21,
22,
23,
31,
32,
33,
41,
42,
43,
44,
45,
51,
52,
53,
54,
55,
56,
57,
65,
66,
67,
68,
69,
71,
72,
73,
);

my %test_blocks = ( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],    <<= dmimgcalc test
#          "table" => [ 2, 4 ] },  <<= dmdiff test
# "2" => { "list" => [ ] },...
1 => { "table" => [1,2] },
2 => { "table" => [1,2] },
3 => { "table" => [1,2] },
4 => { "table" => [1,2] },
5 => { "table" => [1,2] },
6 => { "table" => [1,2] },
7 => { "table" => [1,2] },
8 => { "table" => [1,2] },
9 => { "table" => [1,2] },
10 => { "table" => [1,2] },
21 => { "table" => [1,2] },
22 => { "table" => [1,2] },
23 => { "table" => [1,2] },
31 => { "table" => [1,2]},
32 => { "table" => [1,2]},
33 => { "table" => [1,2]},
41 => { "text2" => [] },
42 => { "text" => [] },
43 => { "text" => [] },
44 => { "text" => [] },
45 => { "text" => [] },
51 => { "fail" => []},       # expected failure.
52 => { "fail" => []},       # expected failure.
53 => { "fail" => []},       # expected failure.
54 => { "fail" => []},       # expected failure.
55 => { "fail" => []},       # expected failure.
56 => { "fail" => []},       # expected failure.
57 => { "fail" => []},       # expected failure.
65 => { "text" => [] },
66 => { "text2" => [] },
67 => { "text" => [] },
68 => { "table" => [2] },
69 => { "text2"  => [] },
71 => { "table" => [2]},
72 => { "table" => [2]},
73 => { "table" => [2]},
);

my %test_words = (
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# ** DTF HISTORY format **
$version => "ASCDS_VERSION",
'HISTORY.*\n' => "",
'HISTNUM.*\n' => ""
);

my %test_tsv_words = (
# ** TSV HISTORY format **
$version => "ASCDS_VERSION",
'#HISTORY.*\n' => "",
'HISTNUM.*\n' => ""
);

# Set parameters to defaults
system( "$PUNLEARN dmcopy" );
system( "$PUNLEARN dmlist" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;

my %exten = ("FITS" => "fits");
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#######################################################################

# Loop over DM kernels to test under.
my $kref = %infra{'kernels'};
my @kernels = @$kref;
my $kernelName;

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  # Loop over all tests
  foreach my $testid (@testlist)
  {
    if ( $test_blocks{$testid} == "" ){ next; }

    # Init per-test variables.
    my $mismatch = 0;
    my $time_user;
    my $time_sys;
    $err_msg = "";
    my $skipDelete = 0;
    my $linkname = "";

    # change extension based on kernel
    $in_ext = $exten{$kernelName};

    # delete old outputs
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;

    system("$RM -f $testout/${basename}*");

    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    my $infile = "$testin/in${basename}.$in_ext";

    # PROGRAMMER - put output filename here
    my $outfile = "$testout/${basename}.fits";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/${basename}.fits";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/${basename}.diff";

    # --------------------------------------------------------------------------------
    # set test command
    my $test_string = "";
    
    # -------------------------------------------------------
    # Basic tests: 3-state boolean values
    # -------------------------------------------------------
    if ( $testid == 1 )
    {
	$test_string = "$DMCOPY infile=\"${infile}[double=T]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 2 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=y]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 1;
	$savfile = "$testsav/${basename_ref}.fits";
   }
    if ( $testid == 3 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=1]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 1;
	$savfile = "$testsav/${basename_ref}.fits";
    }
    if ( $testid == 4 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=f]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 5 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=N]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 4;
	$savfile = "$testsav/${basename_ref}.fits";
    }
    if ( $testid == 6 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=0]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 4;
	$savfile = "$testsav/${basename_ref}.fits";
    }
    if ( $testid == 7 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=U]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 8 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=u]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 7;
	$savfile = "$testsav/${basename_ref}.fits";

    }
    if ( $testid == 9 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=2]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 7;
	$savfile = "$testsav/${basename_ref}.fits";
    }
    if ( $testid == 10 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=null]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 7;
	$savfile = "$testsav/${basename_ref}.fits";
    }
    # -------------------------------------------------------
    # Basic tests: list filter
    # test-21: copy rows containing TRUE and NULL value
    # test-22: copy rows containing FALSE and NULL value
    # test-23: copy rows containing FALSE, TRUE and NULL value
    # -------------------------------------------------------
    if ( $testid == 21 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=1,2]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 22 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=n,u]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 23 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=0,Y,U]\" outfile=$outfile verbose=0";
    }

    # -------------------------------------------------------
    # Basic tests: with filter in row range
    # test-31: infile with nulll value as B
    # test-32: infile with nulll value as N
    # test-33: infile with nulll value as U
    # -------------------------------------------------------
    if ( $testid == 31 )
    {
 	#-- Note:  infile with null value as 'B' 
	$test_string = "$DMCOPY infile=\"${infile}[#row=1:106]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 32 )
    {
 	#-- Note:  infile with null value as 'N'
	$test_string = "$DMCOPY infile=\"${infile}[#row=1:106]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 33 )
    {
	#-- Note:  infile with null value as 'U'
 	$test_string = "$DMCOPY infile=\"${infile}[#row=1:106]\" outfile=$outfile verbose=0";
    }

    # -------------------------------------------------------
    # Basic tests: text/raw and text/simple format with NULLSTR option
    # test-41: infile with nulll value as B
    # test-42: infile with nulll value as N
    # test-43: infile with nulll value as U
    # -------------------------------------------------------
     if ( $testid == 41 )
    {	
	$infile  = "$testin/in${basename}.dtf";
	$outfile = "$testout/${basename}.tsv";
	$savfile = "$testsav/${basename}.tsv";

 	# test read/write dtf/tsv logical column for infile with NULLSTR=- and output file with NULLSTR=NULL
	$test_string = "$DMCOPY infile=\"${infile}[opt NULLSTR=-]\" outfile=$outfile\"[opt kernel=text/tsv,NULLSTR=NULL]\" verbose=0";
    }
    if ( $testid == 42 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 41;
	$infile  = "$testin/in${basename_in}.dtf";
	$outfile = "$testout/${basename}.raw";
	$savfile = "$testsav/${basename}.raw";

 	# test read/write dtf/raw logical column for infile with NULLSTR=- and output file with NULLSTR=NULL
	$test_string = "$DMCOPY infile=\"${infile}[opt NULLSTR=-]\" outfile=$outfile\"[opt kernel=text/raw,NULLSTR=NULL]\" verbose=0";
    }
    if ( $testid == 43 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 41;
	$infile  = "$testin/in${basename_in}.dtf";
	$outfile = "$testout/${basename}.dtf";
	$savfile = "$testsav/${basename}.dtf";

 	# test read/write dtf/dtf logical column for infile with NULLSTR=- and output file with NULLSTR=NULL
	$test_string = "$DMCOPY infile=\"${infile}[opt NULLSTR=-]\" outfile=$outfile\"[opt kernel=text/dtf,NULLSTR=NULL]\" verbose=0";
    }
    if ( $testid == 44 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 41;
	$infile  = "$testin/in${basename_in}.dtf";
	$outfile = "$testout/${basename}.txt";
	$savfile = "$testsav/${basename}.txt";

 	# test read/write dtf/txt logical column for infile with NULLSTR=- and output file with NULLSTR=NULL
	$test_string = "$DMCOPY infile=\"${infile}[opt NULLSTR=-]\" outfile=$outfile\"[opt kernel=text/sm,NULLSTR=NULL]\" verbose=0";
    }
    if ( $testid == 45 )
    {
	$infile  = "$testin/in${basename}.raw";
	$outfile = "$testout/${basename}.txt";
	$savfile = "$testsav/${basename}.txt";

 	# test read/write raw/txt logical column for infile with NULLSTR=NULL and output file with NULLSTR=H
	$test_string = "$DMCOPY infile=\"${infile}[opt NULLSTR=NULL]\" outfile=$outfile\"[opt kernel=text/simple,NULLSTR=H]\" verbose=0";
    }

   # -------------------------------------------------------
    # Basic tests: expected bad returns
    # -------------------------------------------------------
    if ( $testid == 51 )
    {
	$test_string  = "$DMLIST infile=\"${infile}[double=0:4]\" opt=counts outfile=$outfile verbose=0";
    }
    if ( $testid == 52 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string  = "$DMLIST infile=\"${infile}[double=0:1]\" opt=counts outfile=$outfile verbose=0";
    }
    if ( $testid == 53 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string  = "$DMLIST infile=\"${infile}[double=2:]\" opt=counts outfile=$outfile verbose=0";
    }

    if ( $testid == 54 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string  = "$DMLIST infile=\"${infile}[double >= Y]\" opt=counts outfile=$outfile  verbose=0";
    }

    if ( $testid == 55 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string  = "$DMLIST infile=\"${infile}[double <= U]\" opt=counts outfile=$outfile  verbose=0";
    }

    if ( $testid == 56 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string  = "$DMLIST infile=\"${infile}[double > n]\" opt=counts outfile=$outfile verbose=0";
    }
 
    if ( $testid == 57 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string  = "$DMLIST infile=\"${infile}[double < 2]\" opt=counts outfile=$outfile  verbose=0";
    }
    if ( $testid == 65 )
    {   
	$infile  = "$testin/in${basename}.tsv\"[opt kernel=text/tsv]\"";
	$outfile = "$testout/${basename}.txt";
	$savfile = "$testsav/${basename}.txt";
	$test_string = "$DMCOPY infile=${infile} outfile=$outfile\"[opt kernel=text/simple]\" verbose=0 ";
    }
    if ( $testid == 66 )
    {   
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 65;
	$infile  = "$testin/in${basename_in}.tsv\"[opt kernel=text/tsv]\"";
	$outfile = "$testout/${basename}.tsv";
	$savfile = "$testsav/${basename}.tsv";
	$test_string = "$DMCOPY infile=${infile} outfile=$outfile\"[opt kernel=text/tsv]\" verbose=0 ";
    }
    if ( $testid == 67 )
    {   
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 65;
	$infile  = "$testin/in${basename_in}.tsv\"[opt kernel=text/tsv]\"";
	$outfile = "$testout/${basename}.dtf";
	$savfile = "$testsav/${basename}.dtf";
	$test_string  = "$DMCOPY infile=${infile} outfile=$outfile\"[opt kernel=text/dtf]\" verbose=0 ";
    }

    if ( $testid == 68 )
    {   
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 65;
	$infile  = "$testin/in${basename_in}.tsv\"[opt kernel=text/tsv]\"";
	$test_string  = "$DMCOPY infile=${infile} outfile=$outfile verbose=0 ";
    }

    if ( $testid == 69 )
    {   
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.$in_ext";
	$outfile = "$testout/${basename}.tsv";
	$savfile = "$testsav/${basename}.tsv";
	$test_string  = "$DMCOPY infile=${infile} outfile=$outfile\"[opt kernel=text/tsv]\" verbose=0 ";
    }
 
   if ( $testid == 71 )
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 1;
	$infile   = "$testin/in${basename}.$in_ext";
	
	$test_string = "$DMCOPY infile=\"${infile}[exclude double=t]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 72 )
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 1;
	$infile   = "$testin/in${basename}.$in_ext";
	$test_string = "$DMCOPY infile=\"${infile}[exclude double=0]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 73 )
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 1;
	$infile   = "$testin/in${basename}.$in_ext";
	$test_string = "$DMCOPY infile=\"${infile}[exclude double=U]\" outfile=$outfile verbose=0";
    }
    $test_string .= " >> $logfname 2>&1";
 
    # --------------------------------------------------------------------------------
    # run the test
    dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
    close $logfile;
    
    ($rc, $time_user, $time_sys) = dt_run_test($test_string);
    
    if ( $block_types{"fail"} )
    { # Expected Failure test check.
        if ( $rc == 0 )
	{
	    $err_msg = "  ERROR: Expected failure did not occur.\n";
	}
	$rc = 0;
    }

    open $logfile, ">>$logfname";
    select($logfile); $| = 1; select($printfile);
    unless ( $rc == 0 )
    {
      $err_msg = "ERROR: $toolname failed to run\n";
    }
   
    # --------------------------------------------------------------------------------
    # validate results
    if ( $block_types{"table"} )
    {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff ); 
      }
    }
    if ( $block_types{"image"} )
    {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
    }
    if ( $block_types{"list"} )
    {
      $err_msg .= dmlist_test( $outfile, "${outfile}_filt_list", $savfile, "$testout/${basename}_savfilt_list", \@tolkeys, $logfname, $test_diff, "" );
    }
    if ( $block_types{"text"} )
    {
      $err_msg .= diff_test( $outfile, "${outfile}_filt_text", $savfile, "$testout/${basename}_savfilt_text", \%test_words, $logfname, $test_diff );
    }
    if ( $block_types{"text2"} )
    {
      $err_msg .= diff_test( $outfile, "${outfile}_filt_text", $savfile, "$testout/${basename}_savfilt_text", \%test_tsv_words, $logfname, $test_diff );
    }

    # --------------------------------------------------------------------------------
    # report results
    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        #push (@faillist, $kernelName);
        #$errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
    $time_user_total += $time_user;
    $time_sys_total  += $time_sys;

    # --------------------------------------------------------------------------------
    # delete any input files created just for this kernel
    if ( $kernelName ne "FITS" )
    {
      if ( $skipDelete == 0 )
      {
        system("$RM -f $infile");
      }
    }

  }
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
