#!/usr/bin/perl
# 
#  Copyright (C) 2019,2021  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

#
#############################################################
# Regression Test Script Template
#############################################################
# Turn off buffering
select(STDOUT); $| = 1;

my $tool = dmdiff; # Name of tool to be tested

#----------------------------------------------------------------------
# Find Test Utilities File
#----------------------------------------------------------------------
use File::Basename;
$cwd = dirname($0); # relative to where this script is run from (cwd).
$tutils = "${cwd}/regtest_utils.pl";
if ( ! -e "$tutils" )
{
   print "ERROR: Test Utilities file not found. '${tutils}' \n";
   exit(1);
}
else
{
   require "$tutils";
}

#----------------------------------------------------------------------
# Define base directories
#----------------------------------------------------------------------
#Get test paths with tool specific extensions
($indir, $outdir, $savdir, $logdir) = &get_test_dirs( $tool );

#Create output directories as needed
&gen_test_dirs( $outdir, $logdir );

#Add paths to find test executables
foreach  $dir ( split(/\s+/,$ENV{'ASCDS_INSTALL'} ) )
{
  $ENV{'PATH'} .= ":${dir}/test/bin/${tool}";
}
  
#----------------------------------------------------------------------
# Define Test Suite.
#----------------------------------------------------------------------
my @testlist = define_test_suite();

#----------------------------------------------------------------------
# Standard Test Run.
#----------------------------------------------------------------------
do_standard_test( $tool, $outdir, $logdir, $savdir, \@testlist );



#----------------------------------------------------------------------
# SUBROUTINES
#----------------------------------------------------------------------
# ===================================================================
# define_test_suite( void )
#   Create TestCase objects defining each test.
#
#   Arguments
#     none
#
#   Globals
#     tool    = Tool being tested
#     indir   = Test input directory
#     outdir  = Test output directory
#
#   Returns
#     tests: array of TestCase objects
# --------------------------------------------------------------------
sub define_test_suite
{
  my @testlist = [];
  my $ntests   = 53;   #!! how many test cases

  # pre-size the testslist array
  $endndx = $ntests - 1;
  $#testlist = $endndx;  # this sizes the array

  #!! Find test executable - adjust as needed to support command generation.
  my $toolexe = "";
  $toolexe = find_in_path( $tool, $ENV{'PATH'}, ':');
  if ( $toolexe eq "" )
  {
    print "ERROR: Tool executable not found. \n";
  }
  my $unitexe = "";
  $unitexe = find_in_path( 'unittest_tolerance', $ENV{'PATH'}, ':');
  if ( $unitexe eq "" )
  {
    print "ERROR: Unit Test executable not found. \n";
  }

  if ( ($toolexe eq "") || ($unitexe eq "") )
  {
     print STDOUT " ${tool} : FAIL \n";
     exit(1);
  }

  # Find system utilities used
  #!! Add any system utilities used in test command strings
  #my $CP = find_in_path("cp", $ENV{'PATH'}, ':');

  my $comstr = ""; # command string
  $indir  = "${indir}2";   # MCD TEMP: TEMPORARY SPACE 
  $outdir = "${outdir}2";   # MCD TEMP: TEMPORARY SPACE 
  $savdir = "${savdir}2";  # MCD TEMP: TEMPORARY SPACE 

  for $ii (1 .. $ntests) 
  {
    my $testid = sprintf("test%02d", $ii);
    my $t = TestCase->new( id => $testid, mksubdir => 0 );

    #Setup each TestCase.
    #!! Fill in TestCase details for each test
    #Example:
    #if ( $testid eq "test01" )
    #{
    #  $comstr  = "${toolexe} infile=/file/dne"; # Build Test Command
    #
    #  $t->desc("input file dne.");  # describe what is being tested
    #  $t->type("unit");             # flavor "unit" or "regression"
    #  $t->command( $comstr );       # set command string
    #  $t->verify('out.fits', dmdiff);  # validation criteria (dmdiff)
    #  $t->verify('out.txt', diff);     # validation criteria (diff)
    #  $t->verify('fail', 256);         # validation criteria (for fail test)
    #}
    if ( $testid eq "test01" )
    {
      $comstr  = "${unitexe} ${indir} > ${outdir}/unit_tolerance.out"; # Build Test Command
    
      $t->desc("#2001-4: Tolerance record handling unit tests.");  # describe what is being tested
      $t->type("unit");                # flavor "unit" or "regression"
      $t->command( $comstr );          # set command string
      $t->verify('unit_tolerance.out', diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test02" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf ${indir}/file2.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/bad_value_A.tol ";
      $comstr .= "keys=yes data=no subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=yes error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#3001: 'keys' flag operation.");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
      $t->verify( "fail", 256);            # validation criteria (fail)
    }
    elsif ( $testid eq "test03" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf ${indir}/file2.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/bad_value_A.tol ";
      $comstr .= "keys=yes data=no subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#2005,3002: Value comparison failure threads: No tolerance case. Checks tolerance interpretation, application, message display.");  # describe what is being tested
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test04" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf ${indir}/file2.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/bad_value_B.tol ";
      $comstr .= "keys=yes data=no subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#2006: Value comparison failure threads: Exact match case. Checks tolerance interpretation, application, message display.");  # describe what is being tested
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test05" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf ${indir}/file2.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/bad_value_C.tol ";
      $comstr .= "keys=yes data=no subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#2007: Value comparison failure threads: Range case. Checks tolerance interpretation, application, message display.");  # describe what is being tested
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test06" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf ${indir}/file2.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/bad_value_D.tol ";
      $comstr .= "keys=yes data=no subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#2008: Value comparison failure threads: Percent case. Checks tolerance interpretation, application, message display.");  # describe what is being tested
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test07" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf ${indir}/file2.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/bad_value_E.tol ";
      $comstr .= "keys=yes data=no subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#2009: Value comparison failure threads: Bounds case. Checks tolerance interpretation, application, message display.");  # describe what is being tested
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test08" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf'[cols BADUNIT]' ${indir}/file2.dtf'[cols BADUNIT]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/not_bad_value.tol ";
      $comstr .= "keys=yes data=yes subspaces=no units=yes comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#3003,4003: 'units' and 'error_on_unit' flag operation.");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
      $t->verify( "fail", 256);            # validation criteria (fail)
    }
    elsif ( $testid eq "test09" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf'[cols BADUNIT]' ${indir}/file2.dtf'[cols BADUNIT]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/not_bad_value.tol ";
      $comstr .= "keys=yes data=yes subspaces=no units=yes comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#3004,4004: 'units' and 'error_on_unit' flag operation.");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test10" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf'[cols BADDESC]' ${indir}/file2.dtf'[cols BADDESC]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/not_bad_value.tol ";
      $comstr .= "keys=yes data=yes subspaces=no units=no comments=yes wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=yes error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#3005,4005: 'comments' and 'error_on_comment' flag operation.");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
      $t->verify( "fail", 256);            # validation criteria (fail)
    }
    elsif ( $testid eq "test11" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf'[cols BADDESC]' ${indir}/file2.dtf'[cols BADDESC]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/not_bad_value.tol ";
      $comstr .= "keys=yes data=yes subspaces=no units=no comments=yes wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#3006,4006: 'comments' and 'error_on_comment' flag operation.");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test12" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf ${indir}/file2.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/not_bad_value.tol ";
      $comstr .= "keys=yes data=no subspaces=no units=no comments=no wcs=no missing=yes ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=yes ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#3007-8: 'missing' and 'error_on_missing' flag operation.");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
      $t->verify( "fail", 256);            # validation criteria (fail)
    }
    elsif ( $testid eq "test13" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf ${indir}/file2.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/not_bad_value.tol ";
      $comstr .= "keys=yes data=no subspaces=no units=no comments=no wcs=no missing=yes ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#3007-8: 'missing' and 'error_on_missing' flag operation.");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test14" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf'[cols BADDTYP]' ${indir}/file2.dtf'[cols BADDTYP]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/bad_dtype.tol ";
      $comstr .= "keys=yes data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#3009,4009: dtype comparison and 'error_on_datatype' flag operation.");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
      $t->verify( "fail", 256);            # validation criteria (fail)
    }
    elsif ( $testid eq "test15" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf'[cols BADDTYP]' ${indir}/file2.dtf'[cols BADDTYP]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/bad_dtype.tol ";
      $comstr .= "keys=yes data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#3010,4010: dtype comparison and 'error_on_datatype' flag operation.");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test16" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf ${indir}/file2.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#4001,xxxx: 'data' flag operation; different number of columns.");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test17" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf'[cols -BADLEN2]' ${indir}/file2.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#4008: mismatched column lists");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test18" )
    {
      $comstr  = "${toolexe} ${indir}/vectors_a.dtf ${indir}/vectors_b.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#4018-4020: vector column/component match handling");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test19" )
    {
      $comstr  = "${toolexe} ${indir}/vectors_a.dtf'[cols sky,spec]' ${indir}/vectors_b.dtf'[cols sky,spec]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#4021-4022: vector component spec. detail diff");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test20" )
    {
      $cols = "'[cols BADDIM1,BADDIM2,BADLEN1,BADLEN2,BADLEN3,BADLEN4,BADLEN5]'";
      $comstr  = "${toolexe} ${indir}/file1.dtf${cols} ${indir}/file2.dtf${cols} ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#4011-4016: column dimensionality/length differences.");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test21" )
    {
      $comstr  = "${toolexe} ${indir}/virtual_a.dtf ${indir}/virtual_b.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=yes comments=yes wcs=yes missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#4023-4029: virtual column comparisons.");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test22" )
    {
      $comstr  = "${toolexe} ${indir}/table_a.dtf ${indir}/table_b.dtf'[#row=1:4]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#5001: Table data comparisons - bad nrows");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test23" )
    {
      $comstr  = "${toolexe} ${indir}/table_a.dtf ${indir}/table_b.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#5001: Table data comparisons - no tolerance");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test24" )
    {
      $comstr  = "${toolexe} ${indir}/table_a.dtf'[#row=2:3]' ${indir}/table_b.dtf'[#row=2:3]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='${indir}/${testid}.tol' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#5001: Table data comparisons - exact tolerance");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test25" )
    {
      $comstr  = "${toolexe} ${indir}/table_a.dtf'[#row=2:3]' ${indir}/table_b.dtf'[#row=2:3]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='${indir}/${testid}.tol' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#5001: Table data comparisons - bounds tolerance");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test26" )
    {
      $comstr  = "${toolexe} ${indir}/table_a.dtf'[#row=2:4]' ${indir}/table_b.dtf'[#row=2:4]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='${indir}/${testid}.tol' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#5001: Table data comparisons - range tolerance");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test27" )
    {
      $comstr  = "${toolexe} ${indir}/table_a.dtf'[#row=2:4]' ${indir}/table_b.dtf'[#row=2:4]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='${indir}/${testid}.tol' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#5001: Table data comparisons - percent tolerance");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test28" )
    {
      $comstr  = "${toolexe} ${indir}/table_a.dtf'[#row=7:8]' ${indir}/table_b.dtf'[#row=7:8]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='${indir}/${testid}.tol' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#5001: Table data comparisons - ignorepath tolerance");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test29" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[HISTOGRAM]' ${indir}/image_b.dtf'[HISTOGRAM]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - 1D");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test30" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[PLANE]' ${indir}/image_b.dtf'[PLANE]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - 2D");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test31" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[CUBE]' ${indir}/image_b.dtf'[CUBE]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - 3D");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
      #$t->verify( "fail", 256); --- apparently not an error?
    }
    elsif ( $testid eq "test32" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[EMPTY]' ${indir}/image_c.dtf'[EMPTY]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=no error_on_unit=no error_on_range=no ";
      $comstr .= "error_on_datatype=no error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - column names do not match");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test33" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[PLANE]' ${indir}/image_c.fits'[PLANE]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=yes comments=yes wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - image column details do not match");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
      $t->verify( "fail", 256);
    }
    elsif ( $testid eq "test34" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[PLANE]' ${indir}/image_d.dtf'[PLANE]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=yes comments=yes wcs=no missing=no ";
      $comstr .= "error_on_value=no error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=no error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - image column dimensionality");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
      $t->verify( "fail", 256);
    }
    elsif ( $testid eq "test35" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[PLANE]' ${indir}/image_e.dtf'[PLANE]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=yes comments=yes wcs=yes missing=no ";
      $comstr .= "error_on_value=no error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - # axis groups (1-2D vs 2-1D)");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
      $t->verify( "fail", 256);
    }
    elsif ( $testid eq "test36" )
    {
      # This case is not currently supported (CUBE)..
      # o will FAIL when cube images are supported
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[CUBE]' ${indir}/image_e.dtf'[CUBE]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=yes comments=yes wcs=yes missing=no ";
      $comstr .= "error_on_value=no error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - # axis groups (2D+1D vs 1D+2D)");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
      $t->verify( "fail", 0); #256
    }
    elsif ( $testid eq "test37" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[HISTOGRAM]' ${indir}/image_f.dtf'[HISTOGRAM]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=yes comments=yes wcs=yes missing=no ";
      $comstr .= "error_on_value=no error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - # axis group name mismatch");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
      $t->verify( "fail", 256);
    }
    elsif ( $testid eq "test38" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[PLANE]' ${indir}/image_f.dtf'[PLANE2][PLANE]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=yes comments=yes wcs=yes missing=no ";
      $comstr .= "error_on_value=no error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - # axis group component name mismatch");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
      $t->verify( "fail", 256);
    }
    elsif ( $testid eq "test39" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[HISTOGRAM]' ${indir}/image_g.dtf'[HISTOGRAM]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=yes comments=yes wcs=yes missing=no ";
      $comstr .= "error_on_value=no error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - # axis group details differ");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
      $t->verify( "fail", 256);
    }
    elsif ( $testid eq "test40" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[PLANE]' ${indir}/image_g.dtf'[PLANE]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=yes comments=yes wcs=yes missing=no ";
      $comstr .= "error_on_value=no error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - # axis group details differ");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
      $t->verify( "fail", 256);
    }
    elsif ( $testid eq "test41" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[HISTOGRAM]' ${indir}/image_h.dtf'[HISTOGRAM]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=yes comments=yes wcs=yes missing=no ";
      $comstr .= "error_on_value=no error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - # axis transform specs differ");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
      $t->verify( "fail", 256);
    }
    elsif ( $testid eq "test42" )
    {
      $comstr  = "${toolexe} ${indir}/image_a.dtf'[PLANE]' ${indir}/image_h.dtf'[PLANE]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=yes comments=yes wcs=yes missing=no ";
      $comstr .= "error_on_value=no error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - # axis transform specs differ");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
      $t->verify( "fail", 256);
    }
    elsif ( $testid eq "test43" )
    {
      $comstr  = "${toolexe} ${indir}/image_nowcs_a.dtf'[PLANE]' ${indir}/image_nowcs_b.dtf'[PLANE]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=yes comments=yes wcs=yes missing=no ";
      $comstr .= "error_on_value=no error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=no error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#6001: Image data comparisons - No world coordinates axes defined.");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test44" )
    {
      $comstr  = "${toolexe} ${indir}/subspace_a.dtf'[SSTESTA]' ${indir}/subspace_b.dtf'[SSTESTA]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=no subspaces=yes units=yes comments=yes wcs=no missing=no ";
      $comstr .= "error_on_value=yes error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=no error_on_subspace=yes error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#8001: Subspace comparisons - basic");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test45" )
    {
      $comstr  = "${toolexe} ${indir}/subspace_a.dtf'[SSTESTB]' ${indir}/subspace_b.dtf'[SSTESTB]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=no subspaces=yes units=yes comments=yes wcs=no missing=no ";
      $comstr .= "error_on_value=yes error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=no error_on_subspace=yes error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#8001: Subspace comparisons - different # subspace columns.");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test46" )
    {
      $comstr  = "${toolexe} ${indir}/subspace_a.dtf'[SSTESTC]' ${indir}/subspace_b.dtf'[SSTESTC]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=no subspaces=yes units=yes comments=yes wcs=no missing=no ";
      $comstr .= "error_on_value=yes error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=no error_on_subspace=yes error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#8001: Subspace comparisons - subspace column list differs.");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test47" )
    {
      $comstr  = "${toolexe} ${indir}/subspace_a.dtf'[SSTESTD]' ${indir}/subspace_b.dtf'[SSTESTD]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=no subspaces=yes units=yes comments=yes wcs=no missing=no ";
      $comstr .= "error_on_value=yes error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=no error_on_subspace=yes error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#8001: Subspace comparisons - subspace ranges differ.");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test48" )
    {
      $comstr  = "${toolexe} ${indir}/subspace_a.dtf'[SSTESTE]' ${indir}/subspace_b.dtf'[SSTESTE]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=no subspaces=yes units=yes comments=yes wcs=no missing=no ";
      $comstr .= "error_on_value=yes error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=no error_on_subspace=yes error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#8001: Subspace comparisons - subspace regions differ.");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test49" )
    {
      $comstr  = "${toolexe} ${indir}/subspace_a.dtf'[SSTESTF]' ${indir}/subspace_b.dtf'[SSTESTF]' ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=no subspaces=yes units=yes comments=yes wcs=no missing=no ";
      $comstr .= "error_on_value=yes error_on_comment=yes error_on_unit=yes error_on_range=no ";
      $comstr .= "error_on_datatype=yes error_on_wcs=no error_on_subspace=yes error_on_missing=no ";
      $comstr .= "verbose=1 clobber=no";

      $t->desc("#8001: Subspace comparisons - # subspace components differ.");
      $t->type("regression");
      $t->command( $comstr );
      $t->filters(0, "sub $ENV{'ASCDS_INSTALL'} -> <ASCDS_INSTALL>" );
      $t->verify( "${testid}.txt", diff);
    }
    elsif ( $testid eq "test50" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf ${indir}/file2.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/dne.tol ";
      $comstr .= "keys=no data=no subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=yes error_on_comment=yes error_on_unit=yes error_on_range=yes ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=yes error_on_missing=yes ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#1006: Tolerance file does not exist.\n#1900: all check flags off.");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test51" )
    {
      $comstr  = "${toolexe} ${indir}/image_unsup_a.dtf ${indir}/image_unsup_b.dtf ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=no data=yes subspaces=no units=no comments=no wcs=no missing=no ";
      $comstr .= "error_on_value=yes error_on_comment=yes error_on_unit=yes error_on_range=yes ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=yes error_on_missing=yes ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#7001: Unsupported image datatype");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
      $t->verify( "fail", 256);
    }
    elsif ( $testid eq "test52" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf[TAB2] ${indir}/file2.dtf[TAB2] ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile=${indir}/exclude.tol ";
      $comstr .= "keys=yes data=yes subspaces=no units=yes comments=yes wcs=yes missing=yes ";
      $comstr .= "error_on_value=yes error_on_comment=yes error_on_unit=yes error_on_range=yes ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=yes error_on_missing=yes ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#5002: exclude column via tolerance; keyword too");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }
    elsif ( $testid eq "test53" )
    {
      $comstr  = "${toolexe} ${indir}/file1.dtf[TAB3] ${indir}/file2.dtf[TAB3] ";
      $comstr .= "outfile=${outdir}/${testid}.txt tolfile='' ";
      $comstr .= "keys=yes data=no subspaces=no units=no comments=no wcs=no missing=yes ";
      $comstr .= "error_on_value=yes error_on_comment=yes error_on_unit=yes error_on_range=yes ";
      $comstr .= "error_on_datatype=yes error_on_wcs=yes error_on_subspace=yes error_on_missing=yes ";
      $comstr .= "verbose=1 clobber=no";
    
      $t->desc("#3007: header compare - one empty, other not");
      $t->type("regression");              # flavor "unit" or "regression"
      $t->command( $comstr );              # set command string
      $t->verify( "${testid}.txt", diff);  # validation criteria (diff)
    }

    $ndx = $ii - 1;
    $testlist[$ndx] = $t;
  }

  return(@testlist);
}