#!/usr/bin/perl
# 
#  Copyright (C) 2002,2015-2016,2020  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

#
#############################################################
# Regression Test Script Template
#############################################################
# Turn off buffering
select(STDOUT); $| = 1;

my $tool = "dmhistory"; # Name of tool to be tested

#----------------------------------------------------------------------
# Find Test Utilities File
#----------------------------------------------------------------------
use File::Basename;
$cwd = dirname($0); # relative to where this script is run from (cwd).
$tutils = "${cwd}/regtest_utils.pl";
if ( ! -e "$tutils" )
{
   print "ERROR: Test Utilities file not found. '${tutils}' \n";
   exit(1);
}
else
{
   require "$tutils";
}

#----------------------------------------------------------------------
# Define base directories
#----------------------------------------------------------------------
#Get test paths with tool specific extensions
($indir, $outdir, $savdir, $logdir) = &get_test_dirs( $tool );

#Create output directories as needed
&gen_test_dirs( $outdir, $logdir );
  
#----------------------------------------------------------------------
# Define Test Suite.
#----------------------------------------------------------------------
my @testlist = define_test_suite();

#----------------------------------------------------------------------
# Standard Test Run.
#----------------------------------------------------------------------
do_standard_test( $tool, $outdir, $logdir, $savdir, \@testlist );



#----------------------------------------------------------------------
# SUBROUTINES
#----------------------------------------------------------------------
# ===================================================================
# define_test_suite( void )
#   Create TestCase objects defining each test.
#
#   Arguments
#     none
#
#   Globals
#     tool    = Tool being tested
#     indir   = Test input directory
#     outdir  = Test output directory
#
#   Returns
#     tests: array of TestCase objects
# --------------------------------------------------------------------
sub define_test_suite
{
  my @testlist = [];
  my $ntests   = 27;   #!! how many test cases

  # pre-size the testslist array
  $endndx = $ntests - 1;
  $#testlist = $endndx;  # this sizes the array

  #!! Find test executable - adjust as needed to support command generation.
  my $toolexe = "";
  $toolexe = find_in_path( $tool, $ENV{'PATH'}, ':');
  if ( $toolexe eq "" )
  {
    print "ERROR: Tool executable not found. \n";
  }

  # Find system utilities used
  #!! Add any system utilities used in test command strings
  my $CP = find_in_path("cp", $ENV{'PATH'}, ':');
  my $CHMOD = find_in_path("chmod", $ENV{'PATH'}, ':');
  my $PSET = find_in_path("pset", $ENV{'PATH'}, ':');
  my $PLIST = find_in_path("plist", $ENV{'PATH'}, ':');
  my $PUNLEARN = find_in_path("punlearn", $ENV{'PATH'}, ':');

  my $comstr = ""; # command string

  for $ii (1 .. $ntests) 
  {
    my $testid = sprintf("test%02d", $ii);
    my $t = TestCase->new( id => $testid, mksubdir => 0 );

    #Setup each TestCase.
    if ( $testid eq "test01" )
    {
      $outfile = "test01.txt";
      $comstr  = "${toolexe} infile=${indir}/test_nohist.dtf outfile=stdout tool='all' expand=no clobber=yes verbose=1 2> ${outdir}/${outfile}";
    
      $t->desc("Test input with no HISTORY keys.");  # describe what is being tested
      $t->type("unit");                   # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test02" )
    {
      $outfile = "test02.txt";
      $comstr  = "${toolexe} infile=${indir}/test.fits outfile=stdout tool='all' expand=no clobber=yes 1> ${outdir}/${outfile}";
      $t->desc("Test output to stdout.");  # describe what is being tested
      $t->type("unit");                   # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test03" )
    {
      $outfile = "test03.txt";
      $comstr  = "${toolexe} infile=${indir}/test.fits outfile=stderr tool='all' expand=no clobber=yes 2> ${outdir}/${outfile}";
      $t->desc("Test output to stderr.");  # describe what is being tested
      $t->type("unit");                   # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test04" )
    {
      $outfile = "test04.txt";
      $comstr  = "${toolexe} infile=${indir}/test.fits outfile=${outdir}/${outfile} tool='all' expand=no clobber=yes";
      $t->desc("Test output to file.");  # describe what is being tested
      $t->type("unit");                   # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test05" )
    {
      $outfile = "test05.txt";
      $comstr  = "${toolexe} infile=${indir}/test.fits outfile='' tool='all' expand=no clobber=yes 1> ${outdir}/${outfile}";
      $t->desc("Test outfile='' goes to stdout.");  # describe what is being tested
      $t->type("unit");                   # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test06" )
    {
      $outfile = "test06.txt";
      $comstr  = "${toolexe} infile=${indir}/test.fits outfile='none' tool='all' expand=no clobber=yes 1> ${outdir}/${outfile}";
      $t->desc("Test outfile='none' goes to stdout.");  # describe what is being tested
      $t->type("unit");                   # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test07" )
    {
      $outfile = "test_all.txt";
      $comstr  = "${toolexe} infile=${indir}/test.fits outfile=${outdir}/${outfile} tool='all' expand=no clobber=yes";    
      $t->desc("Test action='get', tool='all'");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test08" )
    {
      $outfile = "test08.txt";
      $comstr  = "${toolexe} infile=${indir}/test.fits outfile=${outdir}/${outfile} tool='' expand=no clobber=yes";    
      $t->desc("Test tool='', same as 'all'.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test09" )
    {
      $outfile = "test_single.txt";
      $comstr  = "${toolexe} infile=${indir}/test.fits outfile=${outdir}/${outfile} tool='aconvolve' action=get expand=no clobber=yes";
      $t->desc("Test action=get, with single tool match.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test10" )
    {
      $outfile = "test10.txt";
      $comstr  = "${toolexe} infile=${indir}/test.fits outfile=${outdir}/${outfile} tool='dmhedit' action=get expand=no clobber=yes";
      $t->desc("Test action=get, with multiple matches.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test11" )
    {
      $outfile = "test_expand.txt";
      $comstr  = "${toolexe} infile=${indir}/test.fits outfile=${outdir}/${outfile} tool='all' action=get expand=yes clobber=yes";
      $t->desc("Test expand=yes parameter");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test12" )
    {
      $outfile = "test12.txt";
      $comstr  = "${toolexe} infile=${indir}/test.fits outfile=${outdir}/${outfile} tool='all' action=get expand=no clobber=yes";
      $t->desc("Test expand=no parameter.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test13" )
    {
      $outfile = "test_put.txt";

      $comstr  = "$PSET aconvolve infile=/tmp/img.fits outfile=/tmp/foobar.fits kernelspec='' writekernel=no kernelfile=./. writefft=no fftroot=./. method=slide edges=renorm const=0 pad=no center=no normkernel=none clobber=yes verbose=0 ;" ;
      $comstr .= "$CP ${indir}/test_put.fits ${outdir}/test_put.fits ;" ;
      $comstr .= "${toolexe} infile=${outdir}/test_put.fits tool='aconvolve' action=put ;" ;
      $comstr .= "${toolexe} infile=${outdir}/test_put.fits outfile=${outdir}/${outfile} tool='aconvolve' expand=no clobber=yes ;";
      $comstr .= "$PUNLEARN aconvolve";

      $t->desc("Test action=put with valid tool.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test14" )
    {
      $outfile = "test14.txt";

      $comstr  = "$CP ${indir}/test_put.fits ${outdir}/test14.fits ;" ;
      $comstr .= "${toolexe} infile=${outdir}/test14.fits tool='sometool' action=put 2> ${outdir}/${outfile}" ;

      $t->desc("Test action=put with invalid tool (no parameter file).");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
      $t->verify('fail', 65280);          # validation criteria (for fail test)
    }
    elsif ( $testid eq "test15" )
    {
      $outfile = "test15.txt";
      $outfits = "test15.fits";
      $comstr  = "$CP ${indir}/test_put.fits ${outdir}/${outfits} ; $CHMOD a-w ${outdir}/${outfits} ;" ;
      $comstr .= "${toolexe} infile=${outdir}/${outfits} tool='dmsort' action=put 2> ${outdir}/${outfile} " ;
      $t->desc("Test action=put with valid tool on file with closed permissions.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
      $t->verify('fail', 65280);          # validation criteria (for fail test)
    }
    elsif ( $testid eq "test16" )
    {
      $outfile = "test16.txt";
      $comstr  = "${toolexe} infile=${indir}/test_pset.fits tool='dmcopy' action=pset expand=no ;" ;
      $comstr .= "$PLIST dmcopy > ${outdir}/${outfile}; $PUNLEARN dmcopy;" ;
      $t->desc("Test action=pset with single valid tool match.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->filters(0, "del Parameters for");
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test17" )
    {
      $outfile = "test17.txt";
      $comstr  = "${toolexe} infile=${indir}/test_pset.fits tool='dmhedit' action=pset expand=yes ;" ;
      $comstr .= "$PLIST dmhedit > ${outdir}/${outfile}; $PUNLEARN dmhedit;" ;
      $t->desc("Test action=pset with multiple matching records.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->filters(0, "del Parameters for");
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test18" )
    {
      $outfile = "test18.txt";
      $comstr  = "${toolexe} infile=${indir}/test_pset.fits tool='sometool' action=pset expand=no 2> ${outdir}/${outfile}" ;
      $t->desc("Test action=pset for tool with no parameter file (user).");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test19" )
    {
      $outfile = "test19.txt";
      $comstr  = "${toolexe} infile=${indir}/test.fits outfile=stdout tool='bogus' action=get expand=no clobber=yes 2> ${outdir}/${outfile}";
      $t->desc("Test specified tool not matched.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
      $t->verify('fail', 65280);          # validation criteria (for fail test)
    }
    elsif ( $testid eq "test20" )
    {
      $outfile = "test20.txt";
      $comstr  = "${toolexe} infile=${indir}/test_nohist.dtf outfile=stdout tool='dmcopy' action=get expand=no clobber=yes 2> ${outdir}/${outfile}";
      $t->desc("Test specified tool not matched, on file with no history.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
      $t->verify('fail', 65280);          # validation criteria (for fail test)
    }
    elsif ( $testid eq "test21" )
    {
      $outfile = "test_partial.txt";
      $comstr  = "${toolexe} infile=${outdir}/test_put.fits outfile=${outdir}/${outfile} tool='volve' expand=no clobber=yes";
      $t->desc("Test match to partial tool name.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test22" )
    {
      $outfile = "test_part_with_path.txt";
      $comstr  = "${toolexe} infile=${indir}/test2.fits outfile=${outdir}/${outfile} tool='volve' expand=no clobber=yes";
      $t->desc("Test match to partial tool name when command includes path-to-tool.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
    }
    elsif ( $testid eq "test23" )
    {
      $outfile = "test23.txt";
      $outerr  = "test23_err.txt";
      $comstr  = "${toolexe} infile=${indir}/test_special.dtf outfile=${outdir}/${outfile} tool='all' expand=no clobber=yes 2>${outdir}/${outerr}";
      $t->desc("Test PIXLIB history handling; non-std TOOL format; non-std PARAM format.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
      $t->verify("${outerr}", diff);      # validation criteria (diff)
    }
    elsif ( $testid eq "test24" )
    {
      $outfile = "test24.txt";
      $outerr  = "test24_err.txt";
      $comstr  = "${toolexe} infile=${indir}/test_special.dtf[2] outfile=stdout tool='all' expand=yes clobber=yes verbose=5 1>${outdir}/${outfile} 2>${outdir}/${outerr}";
      $t->desc("Test unrecognized TOKEN; PARAM before TOOL; STCK before TOOL.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
      $t->verify("${outerr}", diff);      # validation criteria (diff)
    }
    elsif ( $testid eq "test25" )
    {
      $outfile = "test25.txt";
      $outerr  = "test25_err.txt";
      $comstr  = "${toolexe} infile=${indir}/test_special.dtf[3] outfile=stdout tool='all' expand=no clobber=yes verbose=5 1>${outdir}/${outfile} 2>${outdir}/${outerr}";
      $t->desc("Test file with >HISTNUM history records.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
      $t->verify("${outerr}", diff);      # validation criteria (diff)
    }
    elsif ( $testid eq "test26" )
    {
      $outfile = "test26.txt";
      $outerr  = "test26_err.txt";
      $comstr  = "${toolexe} infile=${indir}/test_special.dtf[4] outfile=stdout tool='acis_format_events' expand=no clobber=yes verbose=5 1>${outdir}/${outfile} 2>${outdir}/${outerr}";
      $t->desc("Test file with PIXLIB history without Version record.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
      $t->verify("${outerr}", diff);      # validation criteria (diff)
    }

    elsif ( $testid eq "test27" )
    {
      $outfile = "test27.txt";
      $outerr  = "test27_err.txt";
      $comstr  = "${toolexe} infile=${indir}/test_special.dtf[5] outfile=stdout tool='all' expand=no clobber=yes verbose=5 1>${outdir}/${outfile} 2>${outdir}/${outerr}";
      $t->desc("Test file with no HISTNUM key but containing HISTORY records.");  # describe what is being tested
      $t->type("regression");             # flavor "unit" or "regression"
      $t->command( $comstr );             # set command string
      $t->verify("${outfile}", diff);     # validation criteria (diff)
      $t->verify("${outerr}", diff);      # validation criteria (diff)
    }


    $ndx = $ii - 1;
    $testlist[$ndx] = $t;
  }

  return(@testlist);
}
